#include <math.h>
#include <qmessagebox.h>
#include "counter.h"
#include "exchange.h"
#include "trberror.h"
#include "names.h"
#include "messages.h"
#include "tools.h"
#include "cfg.h"

BlockCounter::BlockCounter() {
	number_=0;
}

int BlockCounter::set( int _n ) {
	if( (_n >= 0) && (_n <= 0xffff) ) {
		number_ = _n;
		return(1);
	} else return(0);
}

int BlockCounter::incr( void ) {
	if( number_ >= 0xffff ) number_ = 0;
	else number_++;
	return number_;
}

int BlockCounter::decr( void ) {
	if( number_ <= 0 ) number_ = 0xffff;
	else number_--;
	return number_;
}

int Counter::get( char module, char command, const char* name,
		const char* warning, unsigned char** p_respbuf, int dialog ) {
	int ret;
	int resp;

	for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
		resp = exch::sendCommand( module,command,p_respbuf );
		if( resp == 1 ) return(1);	// data
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	QString str = name;
	if( (*p_respbuf)[0] == DSC ) {
		str += " disconnected!";
		ret = -1;
	} else if( (*p_respbuf)[0] == NOP ) {
		str = "MASS is off or disconnected!";
		ret = -1;
	} else if( (*p_respbuf)[0] == NDV ) {
		str = NO_DEVICE;
		ret = -1;
	} else {
		str += ": "; str += warning;
		ret = -2;
	}
	if( !dialog ) return(ret);
	switch ( showWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return ret;
		case  QMessageBox::Ignore: return(ret);
		case  QMessageBox::Retry : return(0);
		default : return(0);
	}
}

int Counter::set( char module, char command, const char* name,
		const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf );
			if( respbuf[0] == ACY ) return(1);
//			if( respbuf[0] == ACW ) stop( dialog );
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Counter::setRun( char module, char command, const char* name,
		const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;
	int errorACW = 0;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			if( respbuf[0] == ACW ) {
				errorACW++;
				if( errorACW > N_REPEAT_COMMAND ) goto ABORT;
				stop( dialog );
				i--;
			}
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
ABORT:	
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Counter::set( char module, char command, char data, const char* name,
			const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf,3,data );
			if( respbuf[0] == ACY ) return(1);
//			if( respbuf[0] == ACW ) stop( dialog );
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0);
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Counter::set( char module, char command, char data_low, char data_high,
			const char* name, const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf,4,data_low,data_high );
			if( respbuf[0] == ACY ) return(1);
//			if( respbuf[0] == ACW ) stop( dialog );
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

Counter::Counter( void ) {
	_currentLevel = -1;
	_currentNumber = -1;
	_currentExpos = -1.0;
	_maxNData = 0;
	ready = FALSE;
}

void Counter::create( int _sizeOfData ) {
	_nData = 0;
	_maxNData = _sizeOfData;
	data = new count_t[ _maxNData ];
	_nBlockLabel = 0;
	_blockSize = 0;
	blockLabel.resize( 100 );
}

void Counter::resizeData( int _newSize ) {
	if( _newSize > _maxNData ) {
		jdebug( "!!!!!!!!! Counter::resizeData!!!!!!!!!!!!!!!!!!!!!!" );
		_maxNData = _newSize;
		delete[] data;
		data = new count_t[ _maxNData ];
	}
}

void Counter::addBlockLabel( int _newLabel ) {
	if( blockLabel.size() <= (uint)_nBlockLabel ) {
		blockLabel.resize( blockLabel.size() + 100 );
	}
	blockLabel[ _nBlockLabel ] = _newLabel;
	_nBlockLabel++;
}

bool Counter::expandData( void ) {
	int nCountInBlock = _blockSize / BYTE_PER_COUNT;

	int finiteLabel = _currentNumber / nCountInBlock;

	bool isCorrected = false;
	for( int label=0; label<finiteLabel; label++ ) {
		if( label >= _nBlockLabel ) {
			insertBlock( label, INVALID_COUNT );
			isCorrected = true;
			continue;
		}
		if( blockLabel[label] > label ) { // skipped
			jdebug( "blockLabel=%d > label=%d",
					blockLabel[label],label );
			insertBlock( label,INVALID_COUNT );
			isCorrected = true;
			continue;
		}
		if( blockLabel[label] < label ) { // multiplieds
			jdebug( "blockLabel=%d < label=%d",
					blockLabel[label],label );
			if( removeBlock( label ) == 0 ) break;
			label--;
			continue;
		}
	}
	return isCorrected;
}

int Counter::repairData( void ) {
	double avg_d = 0;
	int nValid = 0 ;
	for( int i=0; i<_nData; i++ ) {
		if( data[i] != INVALID_COUNT ) {
			avg_d += data[i];
			nValid++;
		}
	}
//	printf( "nValid=%d\n",nValid );
	
	if ( nValid == 0) {
		// bad... nothing to divide for: return number of "corrected" = nData
		return _nData ;
	} 
	count_t avg = (count_t)rint( avg_d / nValid );
//	printf( "avg=%d\n",avg );

	// Replace INVALIDs with average:
	int nCorrected = 0;
	for( int i=0; i<_nData; i++ ) {
		if( data[i] == INVALID_COUNT ) {
//			printf( "replace: i=%d data[i]=%d avg=%d\n",
//					i,data[i],avg );
			data[i] = avg ;
			nCorrected++;
		}
	}
	return nCorrected ;
}


int Counter::insertBlock( int label, count_t value ) {
	int _nCountInBlock = _blockSize / BYTE_PER_COUNT;

	int start = 0;
	int startLabel = 0;
	for( int i=0; i<_nBlockLabel; i++ ) {
		if( blockLabel[i] == label-1 ) {
			start = (i+1)*_nCountInBlock;
			startLabel = i+1;
		}
	}
	if( (_nData+_nCountInBlock) > _maxNData ) {
		jdebug( "_nData+_nCountInBlock = %d > _maxNData = %d",
			_nData+_nCountInBlock,_maxNData );
		return(0);
	}
	_nData += _nCountInBlock;
	for( int j=(_nData-1); j>=(start+_nCountInBlock); j-- ) {
		data[j] = data[j-_nCountInBlock];
	}
	for( int j=start; j<(start+_nCountInBlock); j++ ) {
		data[j] = value;
	}

	_nBlockLabel++;
	for( int j=_nBlockLabel; j>startLabel; j-- ) {
		blockLabel[j] = blockLabel[j-1];
	}
	blockLabel[startLabel] = label;
	jdebug( "%s: block %d inserted",_name,label );
	return(1);
}

int Counter::removeBlock( int label ) {
	int start;
	int _nCountInBlock = _blockSize / BYTE_PER_COUNT;

	for( int i=0; i<_nBlockLabel; i++ ) {
		if( blockLabel[i] == label ) {
			start = i*_nCountInBlock;
			if( start+_nCountInBlock >= _nData ) {
				jdebug( "start+_nCountInBlock = %d > _nData = %d",
					start+_nCountInBlock, _nData );
				return(0); // not found
			}
			_nData -= _nCountInBlock;
			for( int j=start; j<_nData; j++ ) {
				data[j] = data[j+_nCountInBlock];
			}
			_nBlockLabel--;
			for( int j=i; j<_nBlockLabel; j++ ) {
				blockLabel[j] = blockLabel[j+1];
			}
			jdebug( "%s: block %d removed",_name,label );
			return(1);
		}
	}
	jdebug( "block with label %d not found!", label );
	return(0); // not found
}

int Counter::putData( count_t dataVal ) {
	if( _nData < _maxNData ) {
		data[_nData] = dataVal;
		_nData++;
		return 1;
	}
	return 0;
}

int Counter::setOutNumber( int number ) {
	if( number <= _maxNData ) {
		_nData = number;
		return 1;
	}
	else return 0;
}

void Counter::loadConst( void ) {
	QString str;

	_module = leafDataToChar( deviceCfgFile,"Counters",_name,"Module" );
/*
	const char* _sync_status = leafDataToStr( deviceCfgFile,"Counters",_name,"Status" )
				->data();
	if( !strcasecmp(_sync_status, "MASTER") ) _master = 1;
	else if( !strcasecmp(_sync_status, "SLAVE") ) _master = 0;
	else {
		str.sprintf( "%s in channel %s:\nInvalid status (Master or Slave)",
			_name,_channel );
		showFatalError( str );
	}

	_inductor	= leafDataToStr( deviceCfgFile,"Counters",_name,"Inductor" )
				->data();
*/
	_reset		= leafDataToChar( deviceCfgFile,"Counters","Command","Reset" );
	_activeOn	= leafDataToChar( deviceCfgFile,"Counters","Command","ActiveOn" );
	_activeOff	= leafDataToChar( deviceCfgFile,"Counters","Command","ActiveOff" );
	_runMst		= leafDataToChar( deviceCfgFile,"Counters","Command","Run" );
	_stop		= leafDataToChar( deviceCfgFile,"Counters","Command","Stop" );
	_runTest	= leafDataToChar( deviceCfgFile,"Counters","Command","RunTest" );
	_getIdent	= leafDataToChar( deviceCfgFile,"Counters","Command","GetIdent" );
	const char* _id = leafDataToStr( deviceCfgFile,"Counters",_name,
					"Identification" )->data();
	_nIdent = sscanf( _id,"%X%X%X%X", &_ident1,&_ident2,&_ident3,&_ident4 );
	_getCrc		= leafDataToChar( deviceCfgFile,"Counters","Command","GetCRC" );
	_getStatus	= leafDataToChar( deviceCfgFile,"Counters","Command","GetStatus" );
	_getConst	= leafDataToChar( deviceCfgFile,"Counters","Command","GetConst" );
	_getNumber	= leafDataToChar( deviceCfgFile,"Counters","Command","GetNumber" );
	_setNumber	= leafDataToChar( deviceCfgFile,"Counters","Command","SetNumber" );
	_getBlSize	= leafDataToChar( deviceCfgFile,"Counters","Command","GetBlSize" );
	_setBlSize	= leafDataToChar( deviceCfgFile,"Counters","Command","SetBlSize" );
	_setLevel	= leafDataToChar( deviceCfgFile,"Counters","Command","SetLevel" );
	_getLevel	= leafDataToChar( deviceCfgFile,"Counters","Command","GetLevel" );
	_setExpos	= leafDataToChar( deviceCfgFile,"Counters","Command","SetExpos" );
	_getExpos	= leafDataToChar( deviceCfgFile,"Counters","Command","GetExpos" );
	_masterOn	= leafDataToChar( deviceCfgFile,"Counters","Command","MasterOn" );
	_masterOff	= leafDataToChar( deviceCfgFile,"Counters","Command","MasterOff" );
	_induceOn 	= leafDataToChar( deviceCfgFile,"Counters","Command","InduceOn" );
	_induceOff	= leafDataToChar( deviceCfgFile,"Counters","Command","InduceOff" );
	_setInduc	= leafDataToChar( deviceCfgFile,"Counters","Command","SetInduc" );
	_getInduc	= leafDataToChar( deviceCfgFile,"Counters","Command","GetInduc" );

	ready = true;
}

Counter::~Counter( void ) {
	delete[] data;
}

int Counter::searchTestErrors( count_t test_number ) {
	_test_errors = 0;
	for( int i=0; i<_nData; i++ ) {
		test_number--;

		count_t derr = abs(data[i]-test_number);
		if( derr != 0 ) {
			_test_errors += derr; 
			test_number = data[i];
		}
	}
	_test_errors += data[_nData-1]; //yesli ne vse otshioty prishli
	return( _test_errors );
}

int Counter::getIdent( void ) {
	QString str;
	unsigned char *respbuf;

	while( 1 ) {
		str.sprintf( "%s in channel %s",_name,_channel );
		switch ( get( _module, _getIdent, str.data(),
				"Error in getIdent!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( (respbuf[0] != _nIdent+1 ) ||
				(_ident1 != respbuf[1]) || (_ident2 != respbuf[2]) ||
				(_ident3 != respbuf[3]) || (_ident4 != respbuf[4]) ) {
					str.sprintf( "%s in channel %s: Invalid identification: %X %X %X %X!", 
							_name,_channel,respbuf[1],respbuf[2],respbuf[3],respbuf[4] );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);
			}
		}
	}
}

int Counter::reset( int dialog ) {
	blockCounter.reset();
	_currentLevel = -1;
	_currentNumber = -1;
	_currentExpos = -1.0;
	while( 1 ) {
		unsigned char* respbuf;
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			exch::sendCommand( _module, _reset, &respbuf );
			if( respbuf[0] == DSC ) {	// It's OK !!!
				return(1);
			}
		}
		QString str;
		int ret = 0;
		if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
			ret = -3;	// fatal
		} else if( respbuf[0] == NDV ) {
			ret = -3;	// fatal
			str = NO_DEVICE;
		} else {
			ret = 0;
			str.sprintf( "Module \"%s\": Error in Reset!",_name );
		}
		if( !dialog ) return ret;
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return ret;
			case  QMessageBox::Ignore: return ret;
			case  QMessageBox::Retry : break;
			default : return ret;
		}
	}
	return(0);
}

int Counter::getConst( void ) {
	QString str;
	unsigned char *respbuf;
	
	while( 1 ) {
		str.sprintf( "%s in channel %s",_name,_channel );
		switch ( get( _module, _getConst, str.data(),
				"Error in getConst!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 5 ) {
					_const1 = (int)respbuf[1];
					_const2 = (int)respbuf[2];
					_const3 = (int)respbuf[3];
					_const4 = (int)respbuf[4];
					return(1);
				}
				str.sprintf( "%s in channel %s: Error in getConst!", _name,_channel );
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Counter::testCrc( void ) {
	QString str;
	unsigned char *respbuf;

	while( 1 ) {
		str.sprintf( "%s in channel %s",_name,_channel );
		switch ( get( _module, _getCrc, str.data(),
				"Error in getCrc!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( (respbuf[0] != 2) || (respbuf[1] != 0) ) {
					str.sprintf( "%s in channel %s: EEPROM corrupted!", _name,_channel );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);
			}
		}
	}
}

int Counter::getStatus( uchar& status ) {
	QString str;
	unsigned char *respbuf;

	while( 1 ) {
		str.sprintf( "%s in channel %s",_name,_channel );
		switch ( get( _module, _getStatus,str.data(),
				"Error in getStatus!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					str.sprintf( "%s in channel %s: Invalid status!", _name,_channel );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { status = respbuf[1]; return(1); }
			}
		}
	}
}

int Counter::stop( int dialog ) {
	QString str;
	str.sprintf( "%s in channel %s",_name,_channel );
	return( set( _module,_stop,str.data(),
			"Error in stop!",dialog ) );
}

int Counter::setMasterOn( int dialog ) {
	QString str;
	str.sprintf( "%s in channel %s",_name,_channel );
	return( set( _module,_masterOn,str.data(),
			"Error in setMasterOn!",dialog ) );
}

int Counter::setMasterOff( int dialog ) {
	QString str;
	str.sprintf( "%s in channel %s",_name,_channel );
	return( set( _module,_masterOff,str.data(),
		"Error in setMasterOff!",dialog ) );
}

int Counter::setActiveOn( int dialog ) {
	QString str;
	str.sprintf( "%s in channel %s",_name,_channel );
	return( set( _module,_activeOn,str.data(),
		"Error in setActiveOn!",dialog ) );
}

int Counter::setActiveOff( int dialog ) {
	QString str;
	str.sprintf( "%s in channel %s",_name,_channel );
	return( set( _module,_activeOff,str.data(),
		"Error in setActiveOff!",dialog ) );
}

int Counter::setInduceOn( int dialog ) {
	QString str;
	str.sprintf( "%s in channel %s",_name,_channel );
	return( set( _module,_induceOn,str.data(),
		"Error in setInduceOn!",dialog ) );
}

int Counter::setInduceOff( int dialog ) {
	QString str;
	str.sprintf( "%s in channel %s",_name,_channel );
	return( set( _module,_induceOff,str.data(),
		"Error in setInduceOff!",dialog ) );
}

int Counter::setInduc( char _master_address, int dialog ) {
	QString str;
	unsigned char *respbuf;

	while( 1 ) {
		str.sprintf( "%s in channel %s",_name,_channel );
		if( set( _module,_setInduc, _master_address, str.data(),"Error in setInduc!",dialog ) == 0 ) {
			return(0);
		}
		switch ( get( _module, _getInduc, str.data(),
				"Error in getInduc!", &respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
 				if( (respbuf[0] != 2) || 
					((char)respbuf[1] != _master_address) ) {
					str.sprintf( "%s in channel %s: Error in setInuc!", _name,_channel );
					if( !dialog ) return(0);
					switch( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);
			}
		}
	}
}

int Counter::setBlSize( int _blSize, int dialog ) {
	QString str;
	unsigned char *respbuf;

	if( (_blSize > MAX_BLOCK_SIZE) || (_blSize < MIN_BLOCK_SIZE) ) {
		str.sprintf( "%s in channel %s: Invalid block size %d outside [%d..%d]!",
			_name,_channel,_blSize, MIN_BLOCK_SIZE,MAX_BLOCK_SIZE );
		if( !dialog ) return(0);
		switch( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}
	while( 1 ) {
		str.sprintf( "%s in channel %s",_name,_channel );
		if( set( _module,_setBlSize, (char)_blSize, str.data(),
				"Error in setBlSize!",dialog ) == 0 ) {
			return(0);
		}
		switch ( get( _module, _getBlSize, str.data(),
				"Error in getBlSize!", &respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
 				if( (respbuf[0] != 2) || 
					((int)respbuf[1] != _blSize) ) {
					str.sprintf( "%s in channel %s: Error in setBlSize!", _name,_channel );
					if( !dialog ) return(0);
					switch( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else {
					_blockSize = _blSize;
					return(1);
				}
			}
		}
	}
}

int Counter::setNumber( int _exposition_number, int dialog ) {
	QString str;
	unsigned char *respbuf;
	int _exposition_number_;

	if( _exposition_number == _currentNumber ) return(1);
	if( (_exposition_number > MAX_EXPOSITION_NUMBER) ||
		(_exposition_number < MIN_EXPOSITION_NUMBER) ) {
		if( !dialog ) return(0);
		str.sprintf( "%s in channel %s: Invalid exposition number %d outside [%d..%d]!",
			_name,_channel,_exposition_number, MIN_EXPOSITION_NUMBER,MAX_EXPOSITION_NUMBER );
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	} 

	resizeData( _exposition_number );
	int _number_high = _exposition_number >> 8;
	int _number_low = _exposition_number - ( _number_high << 8 ); 
	while( 1 ) {
		str.sprintf( "%s in channel %s",_name,_channel );
		_currentNumber = -1;
		if( set( _module,_setNumber, (char)_number_low, (char)_number_high,
				str.data(), "Error in setNumber!",dialog ) == 0 ) {
			return(0);
		}
		switch ( get( _module, _getNumber, str.data(),
				"Error in getNumber!", &respbuf,dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
 				if( respbuf[0] == 3 ) {
					_number_low = (int)respbuf[1];
					_number_high = (int)respbuf[2];
					_exposition_number_ = (_number_high << 8)+_number_low;
					if( _exposition_number_  == _exposition_number ) {
						_currentNumber = _exposition_number;
						return(1);
					} else {
						str.sprintf( "%s in channel %s: Error in setNumber: %d != %d",
							_name,_channel,_exposition_number_,_exposition_number );
					}
				} else {
					str.sprintf( "%s in channel %s: Error in setNumber: Numb. of ret. val. != 3!",
					_name,_channel );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Counter::setLevel( double _level, int dialog ) {
	QString str;
	int dataLev,dataLev_;
//	double _level_;
	unsigned char* respbuf;
//	int value;

	if( _currentLevel == _level ) return(1);
	dataLev = 255 + _const2 -
		(int)(_level*(double)(255 + _const2 - _const1));
	if( dataLev < MIN_LEVEL_VALUE ) dataLev = MIN_LEVEL_VALUE;
	if( dataLev > MAX_LEVEL_VALUE ) dataLev = MAX_LEVEL_VALUE;
	while( 1 ) {
		_currentLevel = -1;
		str.sprintf( "%s in channel %s:",_name,_channel );
		if( set( _module, _setLevel, (char)dataLev,
			str.data(),"Error in setLevel!",dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module, _getLevel, str.data(),
			"Error in getLevel!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
//					if( (value=255+_const2-_const1) != 0 ) {
					dataLev_ = (int)respbuf[1];
//						jdebug( "dataLev_=%d dataLev=%d",dataLev_,dataLev );
//						_level_ = (double)(255 + _const2 - dataLev_)
//							/(double)value;
//						if( fabs(_level-_level_)/
						if( dataLev == dataLev_ ) {
//							_level <= VALUE_TOLERANCE ) {
							_currentLevel = _level;
							return(1);
						}
//					}
				}
				str.sprintf( "%s in channel %s: Error in setLevel!",_name,_channel );
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Counter::setExpos( double expos, int dialog ) {
	QString str;
	unsigned char *respbuf;
	int dataExpos_; 
	double expos_;
	double tmp_val;

	if( expos == _currentExpos ) return(1);
	if( (expos < MIN_EXPOSE ) || (expos > MAX_EXPOSE ) ) {
		if( !dialog ) return(0);
		str.sprintf( "%s in channel %s: Invalid expose %.1f outside[%.1f..%.1f]!",
				_name,_channel,expos,MIN_EXPOSE,MAX_EXPOSE );
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	} 

	int dataExpos = (int)(
		(((double)( _const3+(_const4<<8) ))*expos 
				- 1.0 ) / 8.0);
	int expos_high = dataExpos >> 8;
	int expos_low = dataExpos - ( expos_high << 8 );

	while( 1 ) {
		str.sprintf( "%s in channel %s",_name,_channel );
		_currentExpos = -1.0;
		if( set( _module,_setExpos, (char)expos_low, (char)expos_high,
				str.data(),"Error in setExpos!", dialog ) == 0 ) {
			return(0);
		}
		switch ( get( _module, _getExpos, str.data(),
				"Error in getExpos!", &respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
 				if( respbuf[0] == 3 ) {
					expos_low = (int)respbuf[1];
					expos_high = (int)respbuf[2];
					dataExpos_ = ( expos_high << 8 ) + expos_low;
					tmp_val = (double)(_const3 + (_const4<<8 ));
					if( tmp_val != 0 ) {
						expos_ = (8.0 * (double)dataExpos_ + 1.0) / tmp_val;
					} else expos_ = 0;
					if( fabs(expos_-expos) / expos <= VALUE_TOLERANCE ) {
						_currentExpos = expos;
						return(1);
					} else {
						str.sprintf( "%s in channel %s: Error in setExpos: %.3f != %.3f!",
						_name,_channel,expos_,expos );
					}
				} else {
					str.sprintf( "%s in channel %s: Error in setExpos: Numb. of ret. val. != 3!", 
						_name,_channel );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Counter::runMst( void ) {
	QString str;

	str.sprintf( "%s in channel %s",_name,_channel );
	return( setRun( _module, _runMst,str.data(),"Error in Run!",0 ) );
}

int Counter::runTest( void ) {
	QString str;

	str.sprintf( "%s in channel %s",_name,_channel );
	return( setRun( _module, _runTest,str.data(),"Error in runTest!",0 ) );
}
