/*--------------------------------------------------------------------------*/
/** @file counter.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with photometric modules.

                  MASS project: TURBINA module file
                       header file for counter.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef COUNTER_H
#define COUNTER_H

#include <qarray.h>
#include "iocount.h"

#define BYTE_PER_COUNT	2	// in principle, may be "short" regime, then - one BYTE
#define NS2MS			(1.0E-6)	// nseconds to millisends conversion

#define MAX_LEVEL_VALUE			255
#define MIN_LEVEL_VALUE			0
#define MIN_EXPOSITION_NUMBER	1
#define MAX_EXPOSITION_NUMBER	32767
#define MIN_EXPOSE		0.9	// ms
#define MAX_EXPOSE		32.0	// ms
#define MIN_BLOCK_SIZE		1
#define MAX_BLOCK_SIZE		16
					// znacheniya ot ustanavlivayemogo ( hv,expos...)

//#define MAX_DATA	2000	// Max dlina buffera, nakoplennogo za baseTime

// Status of counter:

#define ACTIVE		0x01	// active_on
#define INDUCE		0x02	// induce_on
#define SHORT		0x04	// 1-byte data
#define SLAVE		0x08	// slave - 1, master - 0
#define SHUTTER		0x10	// shutter_on
#define TEST		0x20	// test
#define COUNT		0x80	// count

class BlockCounter {
private:
	int number_;
public:
	BlockCounter( void );
	int incr( void );
	int decr( void );
	int get( void ) { return number_; }
	int set( int );
	void reset( void ) { number_ = 0; }
};

class Counter {
private:
	bool		ready;

	const char*	_name;
	const char*	_channel;
//	char		_master;	// if master - 1, if slave - 0
//	const char*	_inductor;
	char		_module;
	char		_getIdent;
	int			_nIdent, _ident1,_ident2,_ident3,_ident4;
	char		_reset;
	char		_stop;
	char		_getConst;
	int 		_const1,_const2,_const3,_const4;
	char		_getCrc;
	char		_getStatus;
	char		_masterOn;
	char		_masterOff;
	char		_activeOn;
	char		_activeOff;
	char		_induceOn;
	char		_induceOff;
	char		_setInduc;
	char		_getInduc;

	char		_runMst;
	char		_runTest;
	char		_getNumber;
	char		_setNumber;
	char		_getBlSize;
	char		_setBlSize;
	char		_setLevel;
	char		_getLevel;
	char		_setExpos;
	char		_getExpos;

	int		_maxNData;
	static const count_t INVALID_COUNT = max_count_t;
	count_t		*data;
	int		_nData;
	QArray<int>	blockLabel;
	int		_nBlockLabel;

	int		_test_errors;

	double		_currentLevel;
	double		_currentExpos;
	int		_currentNumber;
	int		_blockSize;
	int		_lastLabelNumber;
	int		_lastLabel;

public:
	Counter( void );
	~Counter();
	void	create( int );
	void	loadConst( void );
	bool    isReady( void ) { return( ready ); }

	void	resizeData( int );
	void	resetData( void ) { _nData = 0; _nBlockLabel = 0; }
	void	resetTest( void ) { _test_errors = -1; }
	void	addBlockLabel( int );
	/**
	@return	The fact that data were expanded due to the loss */
	bool	expandData( void );
	int	repairData( void );
	int	insertBlock( int,count_t value );
	int	removeBlock( int );
	int	putData( count_t );
	count_t*	getData( void ) {return( data );} 
	int	getOutNumber( void ) { return( _nData ); }
	int	setOutNumber( int number );
	int	searchTestErrors( count_t test_number );


//	char	master( void ) { return( _master ); }
//	const char* getInductor( void ) { return( _inductor ); }
	char	getModule( void ) { return( _module ); }
	void	setChannel( const char* _channel_ ) { _channel = _channel_; }
	const char* getChannel( void ) { return( _channel ); }
	void	setName( const char* _name_ ) { _name = _name_; }
	const char* getName( void ) { return( _name ); }


	int	get( char, char, const char*, const char*, unsigned  char**, int );
	int	set( char, char, const char*, const char*,int );
	int	setRun( char, char, const char*, const char*,int );
	int	set( char, char, char, const char*, const char*,int );
	int	set( char, char, char, char, const char*, const char*,int );

	int	getIdent( void );	//
	int	reset( int );		//
	int	getConst( void );	//
	int	testCrc( void );	//
	int	getStatus( uchar & );	//

	int	stop( int );		//
	int	setMasterOn( int );	//
	int	setMasterOff( int );	//
	int	setActiveOn( int );	//
	int	setActiveOff( int );	//
	int	setInduceOn( int );	//
	int	setInduceOff( int );	//
	int	setInduc( char,int );	//
	int	setBlSize( int,int );	//
	int	setNumber( int,int );	//
	int	setLevel( double,int );	// Napisat'!!!
	double	getLevel( void ) { return( _currentLevel ); }
	int	setExpos( double,int );	//

	int	runMst( void );
	int	runTest( void );

	BlockCounter blockCounter;
};

#endif