#include <math.h>
#include <qmessagebox.h>
#include "names.h"
#include "trberror.h"
#include "light.h"
#include "tools.h"
#include "messages.h"
#include "cfg.h"

Light::Light( void ) {
	_name = "Lighters & Buttons";
	_currentBrightness = 0;
	_illumination = -1;
	_light = -1;
	_buttons = -1;
	_currentLight = 0;
	ready = FALSE;
}

Light::~Light() {
}

void Light::loadConst( void ) {
	jdebug( "light: LoadConst" );

	_module		= leafDataToChar( deviceCfgFile,_name,"Common","Module" );
	_setIllum	= leafDataToChar( deviceCfgFile,_name,"Command","SetIllum" );
	_getIllum	= leafDataToChar( deviceCfgFile,_name,"Command","GetIllum" );
	_setLight	= leafDataToChar( deviceCfgFile,_name,"Command","SetLight" );
	_getLight	= leafDataToChar( deviceCfgFile,_name,"Command","GetLight" );
	_getCrc		= leafDataToChar( deviceCfgFile,_name,"Command","GetCrc" );
	_getStatus	= leafDataToChar( deviceCfgFile,_name,"Command","GetStatus" );
	_setVAmpl	= leafDataToChar( deviceCfgFile,_name,"Command","SetVAmpl" );
	_getVAmpl	= leafDataToChar( deviceCfgFile,_name,"Command","GetVAmpl" );
	_setBMode	= leafDataToChar( deviceCfgFile,_name,"Command","SetBMode" );
	_getBMode	= leafDataToChar( deviceCfgFile,_name,"Command","GetBMode" );
	_setParam	= leafDataToChar( deviceCfgFile,_name,"Command","SetParam" );
	_getParam	= leafDataToChar( deviceCfgFile,_name,"Command","GetParam" );
	_illumOn	= leafDataToChar( deviceCfgFile,_name,"Command","IllumOn" );
	_illumOff	= leafDataToChar( deviceCfgFile,_name,"Command","IllumOff" );
	_lightOn	= leafDataToChar( deviceCfgFile,_name,"Command","LightOn" );
	_lightOff	= leafDataToChar( deviceCfgFile,_name,"Command","LightOff" );
	_varyOn		= leafDataToChar( deviceCfgFile,_name,"Command","VaryOn" );
	_varyOff	= leafDataToChar( deviceCfgFile,_name,"Command","VaryOff" );
	_buttLock	= leafDataToChar( deviceCfgFile,_name,"Command","ButtLock" );
	_buttUnLock	= leafDataToChar( deviceCfgFile,_name,"Command","ButtUnLock" );
	_getIdent	= leafDataToChar( deviceCfgFile,_name,"Command","GetIdent" );
	_getConst	= leafDataToChar( deviceCfgFile,_name,"Command","GetConst" );
	_reset		= leafDataToChar( deviceCfgFile,_name,"Command","Reset" );

	 const char* _id = leafDataToStr( deviceCfgFile,_name,"Common","Identification" )
				->data();
	_nIdent = sscanf( _id,"%X%X%X%X", &_ident1,&_ident2,&_ident3,&_ident4 );

	ready = true;
}

int Light::get( char module, char command, const char* name,
		const char* warning, unsigned char** p_respbuf, int dialog ) {
	int i;
	int ret;
	int resp;

	for( i=0; i<N_REPEAT_COMMAND; i++ ) {
		resp = exch::sendCommand( module,command,p_respbuf );
		if( resp == 1 ) return(1);	// data
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	QString str = name;
	if( (*p_respbuf)[0] == DSC ) {
		str += " disconnected!";
		ret = -1;
	} else if( (*p_respbuf)[0] == NOP ) {
		str = "MASS is off or disconnected!";
		ret = -1;
	} else if( (*p_respbuf)[0] == NDV ) {
		str = NO_DEVICE;
		ret = -1;
	} else {
		str += ": "; str += warning;
		ret = -2;
	}
	if( !dialog ) return(ret);
	switch ( showWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return ret;
		case  QMessageBox::Ignore: return(ret);
		case  QMessageBox::Retry : return(0);
		default : return(0);
	}
}

int Light::set( char module, char command, const char* name,
		const char* warning, int dialog ) {
	int resp;
	int i;
	unsigned char *respbuf;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Light::set( char module, char command, char data, const char* name,
			const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;
	int i;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf,3,data );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Light::set( char module, char command, char data_low, char data_high,
			const char* name, const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;
	int i;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf,4,data_low,data_high );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}


int Light::getIdent( void ) {
	QString str;
	unsigned char *respbuf;

	jdebug( "Light::getIdent:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getIdent, str.data(),
			"Error in getIdent!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
			if( (respbuf[0] != _nIdent+1 ) ||
				(_ident1 != respbuf[1]) || (_ident2 != respbuf[2]) ||
				(_ident3 != respbuf[3]) || (_ident4 != respbuf[4]) ) {
					str.sprintf( "Module \"%s\": Invalid identification: %X %X %X %X!",
						_name,respbuf[1],respbuf[2],respbuf[3],respbuf[4] );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);

			}
		}
	}
}

int Light::reset( int dialog ) {
	jdebug( "Light::reset:" );
	int cycle = 1;
	_currentBrightness = 0;
	_illumination = -1;
	_light = -1;
	_buttons = -1;
	_currentLight = -1;
	while( cycle ) {
		unsigned char* respbuf;
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			exch::sendCommand( _module, _reset, &respbuf );
			if( respbuf[0] == DSC ) {	// It's OK !!!
				return(1);
			}
		}
		QString str;
		int ret = 0;
		if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
			ret = -3;	// fatal
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
			ret = -3;	// fatal
		} else {
			str.sprintf( "Module \"%s\": Error in Reset!",_name );
			ret = 0;
		}
		if( !dialog ) return ret;
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return ret;
			case  QMessageBox::Ignore: return ret;
			case  QMessageBox::Retry : break;
			default	: return ret;
		}
	}
	return(0);
}

int Light::getConst( void ) {
	QString str;
	unsigned char *respbuf;

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getConst, str.data(),
				"Error in getConst!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 5 ) {
					_const1 = (int)respbuf[1];
					_const2 = (int)respbuf[2];
					_const3 = (int)respbuf[3];
					_const4 = (int)respbuf[4];
					return(1);
				}
				str.sprintf( "Module \"%s\": Error in getConst!",_name );
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::testCrc( void ) {
	QString str;
	unsigned char *respbuf;

	jdebug( "testCrc:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getCrc, str.data(),
				"Error in getCrc!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( (respbuf[0] != 2) || (respbuf[1] != 0) ){
					str.sprintf( "Module \"%s\": EEPROM corrupted!",_name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);
			}
		}
	}
}

int Light::testButtons( void ) {
	uchar _status;
	if( getStatus( _status ) != 1 ) return(0);
	if( _status & BUTT_LOCK ) _buttons = 0;
	else _buttons = 1;
	return(1);
}

int Light::testIllum( void ) {
	uchar _status;
	if( getStatus( _status ) != 1 ) return(0);
	if( _status & ILLUM_ON ) _illumination = ON;
	else _illumination = OFF;
	if( _status & LIGHT_ON ) _light = ON;
	else _light = OFF;
// !!!	getIllum();
	return(1);
}

int Light::getStatus( uchar& _status ) {
	QString str;
	unsigned char *respbuf;

	jdebug( "getStatus:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getStatus, str.data(),
				"Error in getStatus!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					str.sprintf( "Module \"%s\": Invalid status!", _name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { _status = respbuf[1]; return(1); }
			}
		}
	}
}

int Light::turnIllum( const QString& on, int dialog ) {
	int on_int = 0;
	if( on.upper() == "ON" ) on_int = 1;
	else if( on.upper() == "OFF" ) on_int = 0;
	return( turnIllum( on_int,dialog ) );
}

int Light::turnIllum( int on, int dialog ) {
	if( on == ON ) return setIllumOn( dialog );
	else return setIllumOff( dialog );
}

int Light::setIllumOn( int dialog ) {
	int ret=1;
	QString str;

	jdebug( "setIllumOn:" );
	if( _illumination != ON ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_illumOn,str.data(),
			"Error in setIllumOn!",dialog );
		if( ret == 1 ) _illumination = ON;
	}
	return( ret );
}

int Light::setIllumOff( int dialog ) {
	int ret=1;
	QString str;

	jdebug( "setIllumOff:" );
	if( _illumination != OFF ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_illumOff,str.data(),
			"Error in setIllumOff!",dialog );
		if( ret == 1 ) _illumination = OFF;
	}
	return ret;
}

int Light::turnLight( int on, int dialog,
			int waitInterval=LIGHT_WAIT_INTERVAL ) {
	if( on == ON ) return setLightOn( dialog,waitInterval );
	else return setLightOff( dialog );
}

int Light::setLightOn( int dialog,
			int waitInterval=LIGHT_WAIT_INTERVAL ) {
	int ret = 1;
	jdebug( "setLightOn:" );
	if( _light != ON ) {
		QString str = "Module " + QString("\"") +
			_name + QString("\"");
//		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_lightOn,str.data(),
			"Error in setLightOn!", dialog );
		if( ret == 1 ) _light = ON;
		if( waitInterval && _currentLight )
				exch::wait(waitInterval);
	}
	return ret;
}

int Light::setLightOff( int dialog ) {
	int ret = 1;
	jdebug( "setLightOff:" );
	QString str;
	if( _light != OFF ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_lightOff,str.data(),
			"Error in setLightOff!", dialog );
		if( ret == 1 ) _light = OFF;
	}
	return ret;
}

int Light::setVaryOn( int dialog ) {
	jdebug( "setVaryOn:" );
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	return( set( _module,_varyOn,str.data(),
		"Error in setVaryOn!", dialog ) );
}

int Light::setVaryOff( int dialog ) {
	jdebug( "setVaryOff:" );
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	return( set( _module,_varyOff,str.data(),
		"Error in setVaryOff!", dialog ) );
}

int Light::turnButtons( const QString& _on, int dialog ) {
	int _on_int = 0;
	if( operator==( _on.upper(),"ON" ) ) _on_int = 1;
	if( operator==( _on.upper(),"OFF" ) ) _on_int = 0;
	return( turnButtons( _on_int,dialog ) );
}

int Light::turnButtons( int _on, int dialog ) {
	int ret;
	if( _on ) ret = setButtUnLock( dialog );
	else ret = setButtLock( dialog );
	return( ret );
}

int Light::setButtLock( int dialog ) {
	int ret=1;
	jdebug( "setButtLock:" );
	QString str;
	if( _buttons != 0 ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_buttLock,str.data(),
			"Error in setButtLock!", dialog );
		if( ret == 1 ) _buttons = 0;
	}
	return( ret );
}

int Light::setButtUnLock( int dialog ) {
	int ret=1;
	jdebug( "setUnButtLock:" );
	QString str;
	if( _buttons != 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_buttUnLock,str.data(),
			"Error in setButtUnLock!", dialog );
		if( ret == 1 ) _buttons = 1;
	}
	return( ret );
}

int Light::setIllum( int il, int dialog ) {
	QString str;
	int il_;
	unsigned char* respbuf;

	if( (il < MIN_ILL_VALUE ) || (il > MAX_ILL_VALUE ) ) {
		str.sprintf( "Module \"%s\": Invalid illumination value %d outside [%d..%d]!",
			_name,il,MIN_ILL_VALUE,MAX_ILL_VALUE );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setIllum, (char)il,
				str.data(),"Error in setIllum!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getIllum,str.data(),
				"Error in getIllum!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					il_ = (int)respbuf[1]; //(double)respbuf[1]/255.0;
					jdebug( "il=%d il_=%d",il,il_ );
					if( il == il_ ) {
						return(1);
					} else {
						str.sprintf( "Module \"%s\": Error in setIllum: %d != %d!",
							_name,il,il_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setIllum: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::setLight( double light, int dialog,
			int waitInterval=LIGHT_WAIT_INTERVAL ) {

	if( light == _currentLight ) return 1;
	if( (light < MIN_LIGHT_VALUE ) || (light > MAX_LIGHT_VALUE ) ) {
		QString str;
		str.sprintf( "Module \"%s\": Invalid light value %.2f outside [%.2f..%.2f]!",
			_name,light,MIN_LIGHT_VALUE,MAX_LIGHT_VALUE );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	int light_int = doubleTo255int( light );
	while( 1 ) {
		QString str;
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setLight, (char)light_int,
				str.data(),"Error in setLight!", dialog ) == 0 ) {
			return 0;
		}
		_currentLight = -1;
		unsigned char* respbuf;
		switch( get( _module,_getLight,str.data(),
				"Error in getLight!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return -1;
			case -2 : return -2;
			case  1 : {
				if( respbuf[0] == 2 ) {
					int light_int_ = (int)respbuf[1];
					jdebug( "light=%d light_=%d",light_int,light_int_ );
					if( light_int == light_int_ ) {
						_currentLight = light;
						if( waitInterval
							&& _currentLight
							&& _light )
							exch::wait(waitInterval);
						return 1;
					} else {
						str.sprintf( "Module \"%s\": Error in setLight: %d != %d!",
							_name,light_int,light_int_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setLight: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return 0;
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::setVAmpl( double _dl, int dialog ) {
	QString str;
	unsigned char* respbuf;

	if( (_dl < MIN_VAMPL_VALUE ) || (_dl > MAX_VAMPL_VALUE ) ) {
		str.sprintf( "Module \"%s\": Invalid vampl value %.2f outside [%.2f..%.2f]!",
			_name,_dl,MIN_VAMPL_VALUE,MAX_VAMPL_VALUE );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	int _dl_int = doubleTo255int( _dl );

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setVAmpl, (char)_dl_int,
				str.data(),"Error in setVampl!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getVAmpl,str.data(),
				"Error in getVampl!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					int _dl_int_ = (int)respbuf[1];
					jdebug( "_dl=%d _dl_=%d",_dl_int,_dl_int_ );
					if( _dl_int == _dl_int_ ) {
						return(1);
					} else {
						str.sprintf( "Module \"%s\": Error in setVampl: %d != %d!",
							_name,_dl_int,_dl_int_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setVampl: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::getBMode( int& _mode, int dialog ) {
	QString str;
	unsigned char *respbuf;

//	jdebug( "getBMode:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getBMode, str.data(),
				"Error in getBMode!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					if( !dialog ) return(0);
					str.sprintf( "Module \"%s\": Error in getBMode! Numb. of ret. par. != 2!", _name );
				} else {
					_mode = (int)respbuf[1];
//					jdebug( "MODE = %d",_mode );
					if( (_mode < MIN_BMODE ) || (_mode > MAX_BMODE ) ) {
						if( !dialog ) return(0);
						str.sprintf( "Module \"%s\": Invalid mode: %d outside [%d..%d]!",
						_name,_mode,MIN_BMODE,MAX_BMODE );
					} else {
						_mode = _mode % N_BMODES;
//						jdebug( "MODE = %d",_mode );
						return(1);
					}
				}
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : continue;
				}
			}
		}
	}
}

int Light::setBMode( int _mode, int dialog ) {
	QString str;
	int _mode_;
	unsigned char* respbuf;

	if( (_mode < MIN_BMODE ) || (_mode > MAX_BMODE ) ) {
		if( !dialog ) return(0);
		str.sprintf( "Module \"%s\": Invalid mode %d outside [%d..%d]!",
			_name,_mode,MIN_BMODE,MAX_BMODE );
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setBMode, (char)_mode,
				str.data(),"Error in setBMode!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getBMode,str.data(),
				"Error in getBMode!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					_mode_ = (int)respbuf[1];
					jdebug( "_mode=%d _mode_=%d",_mode,_mode_ );
					if( _mode == _mode_ ) return(1);
					else {
						str.sprintf( "Module \"%s\": Error in setBMode: %d != %d!",
							_name,_mode,_mode_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setBMode: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::getParam( int& _param, int dialog ) {
	QString str;
	unsigned char *respbuf;

//	jdebug( "getParam:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getParam, str.data(),
				"Error in getParam!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					if( !dialog ) return(0);
					str.sprintf( "Module \"%s\": Numb. of ret. par != 2!", _name );
				} else {
					_param = (int)respbuf[1];
//					jdebug( "PARAM = %d",_param );
					if( (_param < MIN_PARAM_VALUE ) || (_param > MAX_PARAM_VALUE ) ) {
						if( !dialog ) return(0);
						str.sprintf( "Module \"%s\": Invalid param: %d outside [%d..%d]!",
						_name,_param,MIN_PARAM_VALUE,MAX_PARAM_VALUE );
					} else return(1);
				}
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : continue;
				}
			}
		}
	}
}

int Light::setParam( int value, int dialog ) {
	QString str;
	int value_;
	unsigned char* respbuf;

	if( (value < MIN_PARAM_VALUE ) || (value > MAX_PARAM_VALUE ) ) {
		str.sprintf( "Module \"%s\": Invalid param. value %d outside [%d..%d]!",
			_name,value,MIN_PARAM_VALUE,MAX_PARAM_VALUE );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setParam, (char)value,
				str.data(),"Error in setParam!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getParam,str.data(),
				"Error in getParam!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					value_ = (int)respbuf[1];
					jdebug( "value=%d value_=%d",value,value_ );
					if( value == value_ ) return(1);
					else {
						str.sprintf( "Module \"%s\": Error in setParam: %d != %d!",
							_name,value,value_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setParam: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::setBrightnessUp( int dialog ) {
	int illum;
	int brightness;
	if( _currentBrightness < MAX_BRIGHT_PARAM ) brightness = _currentBrightness+1;
	else return 1;

	if( brightness == 0 ) illum = 0;
	else illum = (int) floor( pow( 2.0,
		(8.0*(double)brightness / 15.01) ) + 0.5 );
	jdebug( "BrightnessUp:%d",illum );
	int ret = setIllum( illum,dialog );
	if( ret == 1 ) _currentBrightness = brightness;
	return ret;
}

int Light::setBrightnessDown( int dialog ) {
	int illum;
	int brightness;
	if( _currentBrightness > MIN_BRIGHT_PARAM ) brightness = _currentBrightness-1;
	else return 1;

	if( brightness == 0 ) illum = 0;
	else illum = (int) floor( pow( 2.0,
		(8.0*(double)brightness / 15.01) ) + 0.5 );
	jdebug( "BrightnessDown:%d",illum );
	int ret = setIllum( illum,dialog );
	if( ret == 1 ) _currentBrightness = brightness;
	return ret;
}

int Light::setBrightness( int brightness, int dialog ) {
	int illum;
	if( brightness < MIN_BRIGHT_PARAM ) brightness = MIN_BRIGHT_PARAM;
	if( brightness > MAX_BRIGHT_PARAM ) brightness = MAX_BRIGHT_PARAM;
	if( brightness == 0 ) illum = 0;
	else illum = (int) floor( pow( 2.0,
		(8.0*(double)brightness / 15.01) ) + 0.5 );
	jdebug( "setBrightness:%d",illum );
	int ret = setIllum( illum,dialog );
	if( ret == 1 ) _currentBrightness = brightness;
	return ret;
}
