#include <qmessagebox.h>
#include "names.h"
#include "motor.h"
#include "exchange.h"
#include "tools.h"
#include "messages.h"
#include "trberror.h"
#include "cfg.h"

extern FILE * out_file;

QString _undefinedAperture = "Undefined";

Motor::Motor( void ) {
	_name = "Apertures";
	_currentAperture = -1;
	_currentDoor = -1;
	ready = false;
}

Motor::~Motor() {
}

void Motor::loadConst( void ) {
	jdebug( "motor: LoadConst" );

	_module		= leafDataToChar( deviceCfgFile,_name,"Common","Module" );
	_getStatus	= leafDataToChar( deviceCfgFile,_name,"Command","GetStatus" );
	_getSensors	= leafDataToChar( deviceCfgFile,_name,"Command","GetSensors" );
	_getCrc		= leafDataToChar( deviceCfgFile,_name,"Command","GetCrc" );
	_getPosition	= leafDataToChar( deviceCfgFile,_name,"Command","GetPosition" );

	_moveTo		= leafDataToChar( deviceCfgFile,_name,"Command","MoveTo" );
	_shiftAt	= leafDataToChar( deviceCfgFile,_name,"Command","ShiftAt" );
	_synchAt	= leafDataToChar( deviceCfgFile,_name,"Command","SynchAt" );
	_stopMotion	= leafDataToChar( deviceCfgFile,_name,"Command","StopMotion" );
	_loadStops	= leafDataToChar( deviceCfgFile,_name,"Command","LoadStops" );
	_getStops	= leafDataToChar( deviceCfgFile,_name,"Command","GetStops" );
	_setAperture= leafDataToChar( deviceCfgFile,_name,"Command","SetAperture" );
	_getAperture= leafDataToChar( deviceCfgFile,_name,"Command","GetAperture" );
	_setStep	= leafDataToChar( deviceCfgFile,_name,"Command","SetStep" );
	_getStep	= leafDataToChar( deviceCfgFile,_name,"Command","GetStep" );
	_testMotion	= leafDataToChar( deviceCfgFile,_name,"Command","TestMotion" );
	_nullSet	= leafDataToChar( deviceCfgFile,_name,"Command","NullSet" );
	_openDoor	= leafDataToChar( deviceCfgFile,_name,"Command","OpenDoor" );
	_closeDoor	= leafDataToChar( deviceCfgFile,_name,"Command","CloseDoor" );
	_powerDown	= leafDataToChar( deviceCfgFile,_name,"Command","PowerDown" );
	_getIdent	= leafDataToChar( deviceCfgFile,_name,"Command","GetIdent" );
	_getConst	= leafDataToChar( deviceCfgFile,_name,"Command","GetConst" );
	_reset		= leafDataToChar( deviceCfgFile,_name,"Command","Reset" );

	const char* _id = leafDataToStr( deviceCfgFile,_name,"Common","Identification" )
				->data();
	_nIdent = sscanf( _id,"%X%X%X%X", &_ident1,&_ident2,&_ident3,&_ident4 );

	_nStops = 0;

	_stop[_nStops] = 1;
	_stopCorrect[_nStops] = leafDataToApertureData( cfgFile,"Devices",
		_name,"Stop1",_stopName[_nStops],_stopFoclen[_nStops],out_file );
	if( _stopName[_nStops].upper() != "SPARE" ) _nStops++;

	_stop[_nStops] = 2;
	_stopCorrect[_nStops] = leafDataToApertureData( cfgFile,"Devices",
		_name,"Stop2",_stopName[_nStops],_stopFoclen[_nStops],out_file );
	if( _stopName[_nStops].upper() != "SPARE" ) _nStops++;

	_stop[_nStops] = 3;
	_stopCorrect[_nStops] = leafDataToApertureData( cfgFile,"Devices",
		_name,"Stop3",_stopName[_nStops],_stopFoclen[_nStops],out_file );
	if( _stopName[_nStops].upper() != "SPARE" ) _nStops++;

	_stop[_nStops] = 4;
	_stopCorrect[_nStops] = leafDataToApertureData( cfgFile,"Devices",
		_name,"Stop4",_stopName[_nStops],_stopFoclen[_nStops],out_file );
	if( _stopName[_nStops].upper() != "SPARE" ) _nStops++;

	_stop[_nStops] = 5;
	_stopCorrect[_nStops] = leafDataToApertureData( cfgFile,"Devices",
		_name,"Stop5",_stopName[_nStops],_stopFoclen[_nStops],out_file );
	if( _stopName[_nStops].upper() != "SPARE" ) _nStops++;

	_stop[_nStops] = 6;
	_stopCorrect[_nStops] = leafDataToApertureData( cfgFile,"Devices",
		_name,"Stop6",_stopName[_nStops],_stopFoclen[_nStops],out_file );
	if( _stopName[_nStops].upper() != "SPARE" ) _nStops++;

	_stop[_nStops] = 7;
	_stopCorrect[_nStops] = leafDataToApertureData( cfgFile,"Devices",
		_name,"Stop7",_stopName[_nStops],_stopFoclen[_nStops],out_file );
	if( _stopName[_nStops].upper() != "SPARE" ) _nStops++;

	_stop[_nStops] = 8;
	_stopCorrect[_nStops] = leafDataToApertureData( cfgFile,"Devices",
		_name,"Stop8",_stopName[_nStops],_stopFoclen[_nStops],out_file );
	if( _stopName[_nStops].upper() != "SPARE" ) _nStops++;

	jdebug( "_nStops=%d,Stop2=%s focus[Stop2]=%f",
				_nStops,_stopName[1].data(),_stopFoclen[1] );
	ready = true;
}

int Motor::get( char module, char command, const char* name,
		const char* warning, unsigned char** p_respbuf, int dialog ) {
	int ret;

	for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
		int resp = exch::sendCommand( module,command,p_respbuf );
		if( resp == 1 ) return(1);	// data
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	QString str = name;
	if( (*p_respbuf)[0] == DSC ) {
		str += " disconnected!";
		ret = -1;
	} else if( (*p_respbuf)[0] == NOP ) {
		str = "MASS is off or disconnected!";
		ret = -1;
	} else if( (*p_respbuf)[0] == NDV ) {
		str = NO_DEVICE;
		ret = -1;
	} else {
		str += ": "; str += warning;
		ret = -2;
	}
	if( !dialog ) return(ret);
	switch ( showWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return ret;
		case  QMessageBox::Ignore: return(ret);
		case  QMessageBox::Retry : return(0);
		default : return(0);
	}
}

int Motor::set( char module, char command, const char* name,
		const char* warning, int dialog ) {
	unsigned char *respbuf;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			exch::sendCommand( module,command,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Motor::set( char module, char command, char data, const char* name,
			const char* warning, int dialog ) {
	unsigned char *respbuf;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			exch::sendCommand( module,command,&respbuf,3,data );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Motor::set( char module, char command, char data_low, char data_high,
			const char* name, const char* warning, int dialog ) {
	unsigned char *respbuf;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			exch::sendCommand( module,command,&respbuf,4,data_low,data_high );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}


int Motor::getIdent( void ) {
	QString str;
	unsigned char *respbuf;

	jdebug( "Motor::getIdent:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getIdent, str.data(),
			"Error in getIdent!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
			if( (respbuf[0] != _nIdent+1 ) ||
				(_ident1 != respbuf[1]) || (_ident2 != respbuf[2]) ||
				(_ident3 != respbuf[3]) || (_ident4 != respbuf[4]) ) {
					str.sprintf( "Module \"%s\": Invalid identification: %X %X %X %X!",
						_name,respbuf[1],respbuf[2],respbuf[3],respbuf[4] );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);

			}
		}
	}
}

int Motor::reset( int dialog ) {
	jdebug( "Motor::reset:" );
	int cycle = 1;
	_currentAperture = -1;
	_currentDoor = -1;
	while( cycle ) {
		unsigned char* respbuf;
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			exch::sendCommand( _module, _reset, &respbuf );
			if( respbuf[0] == DSC ) {	// It's OK !!!
				return(1);
			}
		}
		QString str;
		int ret = 0;
		if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
			ret = -3;	// fatal
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
			ret = -3;	// fatal
		} else {
			str.sprintf( "Module \"%s\": Error in Reset!",_name );
			ret = 0;
		}
		if( !dialog ) return ret;
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return ret;
			case  QMessageBox::Ignore: return ret;
			case  QMessageBox::Retry : break;
			default : return ret;
		}
	}
	return(0);
}

int Motor::getConst( void ) {
	unsigned char *respbuf;

	while( 1 ) {
		QString str;
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getConst, str.data(),
				"Error in getConst!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 5 ) {
					_const1 = (int)respbuf[1];
					_const2 = (int)respbuf[2];
					_const3 = (int)respbuf[3];
					_const4 = (int)respbuf[4];
					return(1);
				}
				str.sprintf( "Module \"%s\": Error in getConst!",_name );
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Motor::testCrc( void ) {
	unsigned char *respbuf;

	jdebug( "testCrc:" );
	while( 1 ) {
		QString str;
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getCrc, str.data(),
				"Error in getCrc!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( (respbuf[0] != 2) || (respbuf[1] != 0) ){
					str.sprintf( "Module \"%s\": EEPROM corrupted!",_name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);
			}
		}
	}
}

int Motor::getStatus( uchar& _status, int dialog ) {
	unsigned char *respbuf;

	jdebug( "getStatus:" );
	while( 1 ) {
		QString str;
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getStatus, str.data(),
				"Error in getStatus!",&respbuf,dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					if( !dialog ) return(0);
					str.sprintf( "Module \"%s\": Invalid status!", _name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { _status = respbuf[1]; return(1); }
			}
		}
	}
}

int Motor::getSensors( uchar& status, int dialog ) {
	QString str;
	unsigned char *respbuf;

	jdebug( "getSensors:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getSensors, str.data(),
				"Error in getSensors!",&respbuf,dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					if( !dialog ) return(0);
					str.sprintf( "Module \"%s\": Invalid status of sensors!", _name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { status = respbuf[1]; return(1); }
			}
		}
	}
}

int Motor::getPosition( int& _position, int dialog ) {
	QString str;
	unsigned char *respbuf;
	uchar _low;
	uchar _high;

	jdebug( "getPosition:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getPosition , str.data(),
				"Error in getPosition!",&respbuf,dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 3 ) {
					if( !dialog ) return(0);
					str.sprintf( "Module \"%s\": Invalid Position!", _name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { 
					_low = respbuf[1];
					_high = respbuf[2];
					_position = (int)((_high << 8)+_low);
					return(1);
				}
			}
		}
	}
}

int Motor::setStep( uchar _step, int dialog ) {
	QString str;
	uchar _step_;
	unsigned char* respbuf;

	if( (_step < MIN_STEP ) || (_step > MAX_STEP ) ) {
		str.sprintf( "Module \"%s\": Invalid step value %d outside [%d..%d]!",
			_name,_step,MIN_STEP,MAX_STEP );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setStep, _step,
				str.data(),"Error in setStep!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getStep,str.data(),
				"Error in getStep!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					_step_ = respbuf[1];
					jdebug( "_step=%d _step_=%d",_step,_step_ );
					if( _step == _step_ ) {
						return(1);
					} else {
						str.sprintf( "Module \"%s\": Error in setStep: %d != %d!",
							_name,_step,_step_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setStep: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Motor::loadStops( int dialog ) {
	QString str;
	char _stopCorrect_[MAX_N_STOPS];
	unsigned char* respbuf;
	int ret;
	char i;
	int j,_nApp;
	int abort = 0;

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		for( i=0; i<_nStops; i++ ) {
			ret = set( _module,_loadStops,_stop[i],_stopCorrect[i],
				str.data(),"Error in loadStops!", dialog );
			if( ret == 0 ) return(0);
		}

		switch( get( _module,_getStops,str.data(),
				"Error in getStops!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == MAX_N_STOPS+1 ) {
					for( i=0; i<MAX_N_STOPS; i++ ) {
						_nApp = -1;
						for( j=0; j<_nStops; j++ ) {
							if( i == _stop[j] ) {
								_nApp = j;
								break;
							}
						}
						if( _nApp > -1 ) {
							_stopCorrect_[_nApp] = respbuf[i+1];
							if( _stopCorrect[_nApp] != _stopCorrect_[_nApp] ) {
								str.sprintf( "Module \"%s\": Error in loadStops, aperture N %d: %d != %d!",
								_name,i,_stopCorrect[_nApp],_stopCorrect_[_nApp] );
								abort=1;
								break;
							}
						}
					}
					if( !abort ) return(1);
				} else {
					str.sprintf( "Module \"%s\": Error in loadStops: Numb. of ret. val. != MAX_N_STOPS+1!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Motor::getApertureNumber( const QString& _apName, int dialog ) {
	for( int i=0; i<_nStops; i++ ) {
		if( _apName.upper() == _stopName[i].upper() ) {
			return( i );
		}
	}
	QString str;
	str.sprintf( "Module \"%s\": Invalid aperture %s!",
			_name,_apName.data() );
	if( !dialog ) return(-1);
	switch ( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return -1; 
		case  QMessageBox::Ignore: return(-1);
		default : return(-1);
	}
}


double Motor::getFoclen( const QString& _apName,int dialog ) {
	int _aperture = getApertureNumber( _apName,dialog );
	if( _aperture >= 0 ) return( getFoclen(_aperture ) );
	return(0);
}

double Motor::getFoclen( int _aperture, int dialog=0 ) {
	if( (_aperture < 0 ) || (_aperture >= _nStops ) ) {
		return( 0 );
	}
	return( _stopFoclen[ _aperture ] );
}

int Motor::setAperture( const QString& apName, int dialog,
			bool hard=false ) {
	int aperture = getApertureNumber( apName,dialog );
	if( aperture >= 0 ) return( setAperture(aperture,dialog,hard) );
	return(0);
}

int Motor::setAperture( int aperture, int dialog,
			bool hard=false ) {

//	jdebug( "setAperture: Stop%d",_stop[_aperture] );
	if( !hard ) { if( aperture == _currentAperture ) return(1); }
	if( (aperture < 0 ) || (aperture >= _nStops ) ) {
		QString str;
		str.sprintf( "Module \"%s\": Invalid aperture number %d outside [0..%d]!",
			_name,aperture,_nStops-1 );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	_currentAperture = -1;
RETRY:
	while( 1 ) {
		QString str;
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setAperture, _stop[aperture],
				str.data(),"Error in setAperture!",
				dialog ) == 0 ) {
			return(0);
		}
		unsigned char* respbuf;
		switch( get( _module,_getAperture,str.data(),
				"Error in getAperture!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return -1;
			case -2 : return -2;
			case  1 : {
				if( respbuf[0] != 2 ) {
					str.sprintf( "Module \"%s\": Error in getAperture: Numb. of ret. val. != 2!",
						_name );
				} else {
					char _stop_ = (char)respbuf[1];
					if( _stop[aperture] == _stop_ ) goto CONT;
 					else {
						str.sprintf( "Module \"%s\": Error in setAperture: %d != %d!",
						_name,_stop[aperture],_stop_ );
					}
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
CONT:		
	for( int i=0; i<MAX_MOTOR_TEST_INTERVALS; i++ ) {
		switch( testMotion(dialog) ) {
			case  0 : goto RETRY;
			case -1 : return -1;
			case -2 : return -2;
			case  2 : break;
			case  1 : {
				_currentAperture = aperture;
// Posle uspeshoi ustanovki apertury echio zhdiom 50 ms:
				exch::wait( APERTURE_DELAY );
				return 1;
			}
		}
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	debug( "stop Motion!" );
	stopMotion( dialog );
	if( !dialog ) return(0);
	QString str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::setStop( int _absolute_stop_number, int dialog ) {
	QString str;
	unsigned char* respbuf;
	int _stop_;

	jdebug( "setStop: Stop%d",_absolute_stop_number );
	if( (_absolute_stop_number < 0 ) || (_absolute_stop_number >= MAX_N_STOPS ) ) {
		str.sprintf( "Module \"%s\": Invalid absolute stop number %d outside [0..%d]!",
			_name,_absolute_stop_number,MAX_N_STOPS-1 );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	_currentAperture = -1;
RETRY:
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setAperture, (char)_absolute_stop_number,
				str.data(),"Error in setStop!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getAperture,str.data(),
				"Error in getAperture!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					str.sprintf( "Module \"%s\": Error in getAperture: Numb. of ret. val. != 2!",
						_name );
				} else {
					_stop_ = (int)respbuf[1];
					if( _stop_ == _absolute_stop_number ) goto CONT;
 					else {
						str.sprintf( "Module \"%s\": Error in setStop: %d != %d!",
						_name,_absolute_stop_number,_stop_ );
					}
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
CONT:
	for( int i=0; i<MAX_MOTOR_TEST_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				for( int j=0; j<_nStops; j++ ) {
					if( _stop_ == _stop[j] ) {
	 					_currentAperture = j;
						break;
					}
				}
				jdebug( "Current Aperture = %d",_currentAperture );
				return(1);
			}
		}
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::moveTo( int _position, int dialog ) {
	QString str;
	int _high,_low;
	int _position_;
	int ret;

	jdebug( "moveTo:%d",_position );
	_high = _position >> 8;
	_low = _position - ( _high << 8 );

RETRY:
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_moveTo, _low, _high,
			str.data(),"Error in moveTo!", dialog ) == 0 ) {
		return(0);
	}
	for( int i=0; i<MAX_MOTOR_TEST_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				goto CONT;
			}
		}
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);

CONT:
	ret = getPosition( _position_,dialog );
	if( ret == 1 ) {
		if( _position_ == _position ) return(1);
		else {
			if( !dialog ) return(0);
			str.sprintf( "Module \"%s\": Error in moveTo: %d != %d",
				_name,_position,_position_ );
			switch ( showWarning( str ) ) {
				case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
				case	QMessageBox::Ignore: return(0);
				case	QMessageBox::Retry : goto RETRY;
			}
		}
	}
	return(0);
}

int Motor::synchAt( char _shift, int dialog ) {
	QString str;

//	jdebug( "synchAt:%d",_shift );

	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_synchAt, _shift,
			str.data(),"Error in synchAt!", dialog ) == 0 ) {
		return(0);
	}
	return(1);
}

int Motor::shiftAt( char _shift, int dialog ) {
	QString str;

	jdebug( "shiftAt:%d",_shift );

RETRY:
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_shiftAt, _shift,
			str.data(),"Error in shiftAt!", dialog ) == 0 ) {
		return 0;
	}
	for( int i=0; i<MAX_MOTOR_TEST_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : return(1);
		}
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::testMotion( int dialog ) {
	QString str;
	int ret;
	unsigned char *respbuf;

	for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
		exch::sendCommand( _module,_testMotion,&respbuf,2,0,0,false );
		if( respbuf[0] == ACY ) return(1);
		if( respbuf[0] == ACW ) return(2);
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	if( respbuf[0] == DSC ) {
		str.sprintf( "%s disconnected!", _name );
		ret = -1;
	} else if( respbuf[0] == NOP ) {
		str = "MASS is off or disconnected!";
		ret = -1;
	} else if( respbuf[0] == NDV ) {
		str = NO_DEVICE;
		ret = -1;
	} else {
		str.sprintf( "Module %s: Error in testMotion!",_name );
		ret = -2;
	}
	if( !dialog ) return( ret ); 	
	switch ( showWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return ret; 
		case  QMessageBox::Ignore: return(ret); 
		case  QMessageBox::Retry : return(0); 
	}
	return(0);
}

int Motor::turnDoor( const QString& _open, int dialog ) {
	int int_open = 0;
	if( _open.upper() == "OPEN" ) int_open = OPEN;
	if( _open.upper() == "CLOSE" ) int_open = CLOSE;
	jdebug( "Motor::turnDoor %d",int_open );
	return( turnDoor( int_open,dialog ) );
}

int Motor::turnDoor( int _open, int dialog ) {
	jdebug( "Motor::turnDoor %d",_open );
	if( _open == OPEN ) return( openDoor( dialog ) );
	if( _open == CLOSE ) return( closeDoor( dialog ) );
	if( !dialog ) return(0);
	QString str = "Invalid door parametr!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
		case  QMessageBox::Ignore: return(0);
		default: return(0);
	}
}

int Motor::openDoor( int dialog ) {
	jdebug( "openDoor:" );
	jdebug( "CurrentDoor=%d",_currentDoor );
	if( _currentDoor == OPEN ) return(1);
	int door_ = testDoor(dialog);
	if( door_ == OPEN ) { _currentDoor=OPEN; return(1); }
	if( door_ == INDEFINITE ) return(0);
RETRY:
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_openDoor, str.data(),
		"Error in openDoor!", dialog ) == 0 ) {
		return(0);
	}
	_currentDoor = -1;
	_currentAperture = -1;
	for( int i=0; i<MAX_MOTOR_TEST_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				_currentDoor = testDoor( dialog );
				jdebug( "CurrentDoor=%d",_currentDoor );
				if( _currentDoor == OPEN ) { return(1); }
				else {
					if( !dialog ) return(0);
					str = "Input Door is not opened!";
					switch ( showStrongWarning( str,"Ignore","Retry" ) ) {
						case  QMessageBox::Ignore: return(0); 
						case  QMessageBox::Abort : goto RETRY; 
					}
				}
			}
		}
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::closeDoor( int dialog ) {
	jdebug( "closeDoor:" );
	jdebug( "CurrentDoor=%d",_currentDoor );
	if( _currentDoor == CLOSE ) return(1);
	int door_ = testDoor(dialog);
	if( door_ == CLOSE ) { _currentDoor = CLOSE; return(1); }
	if( door_ == INDEFINITE ) return(0);
RETRY:
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_closeDoor, str.data(),
		"Error in closeDoor!", dialog ) == 0 ) {
		return(0);
	}
	_currentDoor = -1;
	_currentAperture = -1;
	for( int i=0; i<MAX_MOTOR_TEST_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				_currentDoor = testDoor( dialog );
				jdebug( "CurrentDoor=%d",_currentDoor );
				if( _currentDoor == CLOSE ) { return(1); }
				else {
					if( !dialog ) return(0);
					str = "Input Door is not closed!";
					switch ( showStrongWarning( str,"Ignore","Retry" ) ) {
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Abort : goto RETRY;
					}
				}
			}
		}
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::testDoor( int dialog ) {
	uchar status;
	int ret = getSensors( status,dialog );
	if( ret != 1 ) return( INDEFINITE );
	if( status & DOOR ) return( CLOSE );
	else return( OPEN );
}

int Motor::stopMotion( int dialog ) {
	jdebug( "stopMotion:" );
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	return( set( _module,_stopMotion,str.data(),
		"Error in stopMotion!",dialog ) );
}

int Motor::testInitStatus( int dialog ) {
	QString str;
	uchar _status;
	int ret = 1;

	while( 1 ) {
		if( getStatus( _status,dialog ) != 1 ) return( INDEFINITE );
		_currentDoor = CLOSE;
		if( !(_status & DOOR_CLOSE) ){
			_currentDoor = OPEN;
			if( !dialog ) {ret=0; goto CONT; }
			str = "Input Door is not closed!";
			switch ( showWarning( str ) ) {
				case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
				case  QMessageBox::Ignore: ret=0; break;
				case  QMessageBox::Retry : continue; 
			}
		}
CONT:
		if( !(_status & ZERO_POINT) ){
			if( !dialog ) return(0);
			str = "Null-Point in not defined!";
			switch ( showWarning( str ) ) {
				case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
				case  QMessageBox::Ignore: ret=0; break; 
				case  QMessageBox::Retry : continue; 
			}
		}
		jdebug( "PPPPPPP Test Init !!!!!!!!!!!!!!!!!!!!!!! CurrentDoor=%d",_currentDoor );
		return(ret);
	}
}

int Motor::nullSet( int dialog ) {
	jdebug( "nullSet:" );

	if( _currentDoor != OPEN ) {
		if( openDoor(dialog) != 1 ) return 0;
	}

RETRY:

	QString str;
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_nullSet, str.data(),
		"Error in nullSet!", dialog ) == 0 ) {
		return(0);
	}
	_currentAperture = -1;
	_currentDoor = -1;
	for( int i=0; i<MAX_MOTOR_TEST_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				return(1);
			}
		}
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0; 
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::movePlus( int dialog ) {
	char _shift = 1;
	jdebug( "Move Plus:" );
	if( _currentAperture < 0 ) return(0);
//	int _position;
//	int ret = getPosition( _position,dialog );
//	if( ret != 1 ) return(0);
//	jdebug( "_position=%d",_position );
//	if( _position >= MAX_POSITION ) return(0);
	if( _stopCorrect_new[_currentAperture] >= MAX_CORRECTION ) return(0);
	(_stopCorrect_new[_currentAperture])++;
	jdebug( "StopCorrect:%d",_stopCorrect_new[_currentAperture] );
	return( shiftAt( _shift,dialog ) );
}

int Motor::moveMinus( int dialog ) {
	char _shift = -1;
	jdebug( "Move Minus:" );
	if( _currentAperture < 0 ) return(0);
//	int _position;
//	int ret = getPosition( _position,dialog );
//	if( ret != 1 ) return(0);
//	jdebug( "_position=%d",_position );
//	if( _position <= MIN_POSITION ) return(0);
	if( _stopCorrect_new[_currentAperture] <= MIN_CORRECTION ) return(0);
	(_stopCorrect_new[_currentAperture])--;
	jdebug( "StopCorrect:%d",_stopCorrect_new[_currentAperture] );
	return( shiftAt( _shift,dialog ) );
}

int Motor::shiftAperture( int _shift, int dialog ) {
	jdebug( "Shift Aperture: %d", _shift );
	int _position;
	int ret = getPosition( _position,dialog );
	if( ret != 1 ) return(0);
	jdebug( "_position=%d",_position );
	_currentAperture = -1;
	return( moveTo( _position+_shift,dialog ) );
}

QString& Motor::getStopName( int stopNumber ) {
	if( (stopNumber < 0 ) || (stopNumber >= _nStops ) ) {
		return( _undefinedAperture );
	}
	return( _stopName[stopNumber] );
}

int Motor::getStopCorrect( void ) {
	jdebug( "Motor: currentApert = %d",_currentAperture ); 
	return( (int)_stopCorrect[ _currentAperture ] );
}

int Motor::getStopCorrectNew( void ) {
	jdebug( "Motor: currentApert = %d",_currentAperture ); 
	return( (int)_stopCorrect_new[ _currentAperture ] );
}

void Motor::startCorrection( void ) {
	jdebug( "Motor: Start Correction" );
	for( int i=0; i<_nStops; i++ ) {
		_stopCorrect_new[i] = _stopCorrect[i];
	}
}

void Motor::acceptCorrection( int dialog ) {
	jdebug( "Motor: Accept Correction" );
	for( int i=0; i<_nStops; i++ ) {
		_stopCorrect[i] = _stopCorrect_new[i];
	}
	loadStops( dialog );

// Copy to tree:
	QString _str,_num,_stopLeaf;
	for( int i=0; i<_nStops; i++ ) {
		_str = _stopName[i];
		_str.append( "\t" );
		_num.setNum( _stopFoclen[i],'f',2 );
		_str.append( _num );
		_str.append( " " );
		_num.setNum( _stopCorrect[i] );
		_str.append( _num );
		_stopLeaf = "Stop";
		_num.setNum( _stop[i] );
		_stopLeaf.append( _num );
		jdebug( " ________ _str = %s leaf=%s",_str.data(),
			_stopLeaf.data() );
		setLeafData( cfgFile,"Devices",_name,
			_stopLeaf.data(),_str );
	}
}
