/*--------------------------------------------------------------------------*/
/** @file motor.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with stepper motor of the aperture 
wheel.

                  MASS project: TURBINA module file
                       header file for motor.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef MOTOR_H
#define MOTOR_H

#include <qstring.h>

#define	MIN_STEP	1
#define	MAX_STEP	8
#define MAX_N_STOPS	8
#define MAX_MOTOR_TEST_INTERVALS	500	//  2.5 sec if dev_wait_int = 5ms

#define MIN_POSITION	0
#define MAX_POSITION	3199
#define MAX_CORRECTION	127
#define MIN_CORRECTION	-127
#define MAX_SYNCH_SHIFT	127
#define SYNCH_STEP	1

// Status of motor:

#define	DOOR_CLOSE	0x04
#define	ZERO_POINT	0x08
#define	ACCEL		0x10
#define	FORWARD		0x20
#define	SYNCHRON	0x40
#define	MOTION		0x80

// Status of sensors:

#define	ZERO	0x02
#define	DOOR	0x04

class Motor {
private:
	bool	ready;

	const char* _name;
	char	_module;
	int	_const1,_const2,_const3,_const4;
	int	_nIdent, _ident1,_ident2,_ident3,_ident4;

	int	_currentAperture;
	char	_stop[MAX_N_STOPS];
	char	_stopCorrect[MAX_N_STOPS];
	char	_stopCorrect_new[MAX_N_STOPS];
	QString	_stopName[MAX_N_STOPS];
	double	_stopFoclen[MAX_N_STOPS];
	int _nStops;

	char _getStatus;
	char _getSensors;
	char _getCrc;
	char _getPosition;
	char _moveTo;
	char _shiftAt;
	char _synchAt;
	char _stopMotion;
	char _loadStops;
	char _getStops;
	char _setAperture;
	char _getAperture;
	char _setStep;
	char _getStep;
	char _testMotion;
	char _nullSet;
	char _openDoor;
	char _closeDoor;
	char _powerDown;
	char _getIdent;
	char _getConst;
	char _reset;

	int _currentDoor;

public:
	Motor( void );
	~Motor();
	void loadConst( void );
	bool	isReady( void ) { return( ready ); }
//	int loadStopValue( void );

	int	get( char, char, const char*, const char*, unsigned  char**, int );
	int	set( char, char, const char*, const char*,int );
	int	set( char, char, char, const char*, const char*,int );
	int	set( char, char, char, char, const char*, const char*,int );

	int reset( int dialog );
	int getIdent( void );
	int getConst( void );
	int testCrc( void );
	int getStatus( uchar&, int );
	int getSensors( uchar&, int );
	int getPosition( int&, int );
	int setStep( uchar, int );
	int loadStops( int );
	int getApertureNumber( const QString&, int );
	double getFoclen( const QString&,int );
	double getFoclen( int, int _dialog=0 );
	int setAperture( const QString&, int, bool _hard=false );
	int setAperture( int, int, bool _hard=false );
	int setStop( int, int );
	int moveTo( int,int );
	int synchAt( char,int );
	int shiftAt( char,int );
	int testMotion( int );
	int turnDoor( int,int );
	int turnDoor( const QString&,int );
	int openDoor( int );
	int closeDoor( int );
	int testDoor( int );
	int stopMotion( int );
	int testInitStatus( int );
	int nullSet( int );
	int movePlus( int );
	int moveMinus( int );
	int shiftAperture( int,int );

	int isDoorOpen( void ) { return( _currentDoor ); }
	int currentAperture( void ) { return( _currentAperture ); }
	int getNStops( void ) { return( _nStops ); }
	int getMaxNStops( void ) { return( MAX_N_STOPS ); }
	QString& getStopName( int );
	int getStopCorrect( void );
	int getStopCorrectNew( void );
	void startCorrection( void );
	void acceptCorrection( int );
};

#endif
