#include <qmessagebox.h>
#include "names.h"
#include "power.h"
#include "exchange.h"
#include "trberror.h"
#include "tools.h"
#include "messages.h"
#include "cfg.h"


Power::Power( void ) {
	_name = "HV power";
	_safety = -1;
	_currentVoltage = -1;
	ready = FALSE;
}

Power::~Power() {
}

void Power::loadConst( void ) {
	jdebug( "power: LoadConst" );

	_module		= leafDataToChar( deviceCfgFile,_name,"Common","Module" );
	_getStatus	= leafDataToChar( deviceCfgFile,_name,"Command","GetStatus" );
	_getIdent	= leafDataToChar( deviceCfgFile,_name,"Command","GetIdent" );
	_getCrc		= leafDataToChar( deviceCfgFile,_name,"Command","GetCrc" );
	_setVoltage	= leafDataToChar( deviceCfgFile,_name,"Command","SetVoltage" );
	_getVoltage	= leafDataToChar( deviceCfgFile,_name,"Command","GetVoltage" );
	_highOn		= leafDataToChar( deviceCfgFile,_name,"Command","HighON" );
	_highOff	= leafDataToChar( deviceCfgFile,_name,"Command","HighOFF" );
	_safetyOn	= leafDataToChar( deviceCfgFile,_name,"Command","SafetyON" );
	_safetyOff	= leafDataToChar( deviceCfgFile,_name,"Command","SafetyOFF" );
	_getTemper	= leafDataToChar( deviceCfgFile,_name,"Command","GetTemper" );
	_getConst	= leafDataToChar( deviceCfgFile,_name,"Command","GetConst" );
	_reset		= leafDataToChar( deviceCfgFile,_name,"Command","Reset" );

	 const char* _id = leafDataToStr( deviceCfgFile,_name,"Common","Identification" )
				->data();
	_nIdent = sscanf( _id,"%X%X%X%X", &_ident1,&_ident2,&_ident3,&_ident4 );

	ready = true;
	jdebug( "power: end of LoadConst" );
}

int Power::get( char module, char command, const char* name,
		const char* warning, unsigned char** p_respbuf, int dialog ) {
	int resp;

	for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
		resp = exch::sendCommand( module,command,p_respbuf );
		if( resp == 1 ) return(1);	// data
		exch::wait( DEVICE_WAIT_INTERVAL );
	}
	QString str = name;
	int ret;
	if( (*p_respbuf)[0] == DSC ) {
		str += " disconnected!";
		ret = -1;
	} else if( (*p_respbuf)[0] == NOP ) {
		str = "MASS is off or disconnected!";
		ret = -1;
	} else if( (*p_respbuf)[0] == NDV ) {
		str = NO_DEVICE;
		ret = -1;
	} else {
		str += ": "; str += warning;
		ret = -2;
	}
	if( !dialog ) return(ret);
	switch ( showWarning( str ) ) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return ret;
		case  QMessageBox::Ignore: return(ret);
		case  QMessageBox::Retry : return(0);
		default : return(0);
	}
}

int Power::set( char module, char command, const char* name,
		const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Power::set_hvOn( char module, char command, const char* name,
		const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( respbuf[0] == ACW ) return(-2);
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Power::set( char module, char command, char data, const char* name,
			const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;
	int i;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf,3,data );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Power::set( char module, char command, char data_low, char data_high,
			const char* name, const char* warning, int dialog ) {
	int resp;
	unsigned char *respbuf;

	while( 1 ) {
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = exch::sendCommand( module,command,&respbuf,4,data_low,data_high );
			if( respbuf[0] == ACY ) return(1);
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		if( !dialog ) return(0); 	
		QString str = name;
		if( respbuf[0] == DSC ) {
			str += " disconnected!";
		} else if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
		} else {
			str += ": "; str += warning;
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}


int Power::getIdent( void ) {
	QString str;
	unsigned char *respbuf;

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getIdent, str.data(),
			"Error in getIdent!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
			if( (respbuf[0] != _nIdent+1 ) ||
				(_ident1 != respbuf[1]) || (_ident2 != respbuf[2]) ||
				(_ident3 != respbuf[3]) || (_ident4 != respbuf[4]) ) {
					str.sprintf( "Module \"%s\": Invalid identification: %X %X %X %X!",
						_name,respbuf[1],respbuf[2],respbuf[3],respbuf[4] );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);

			}
		}
	}
}

int Power::reset( int dialog ) {
	jdebug( "Power::reset:" );
	_safety = -1;
	_currentVoltage = -1;
	int cycle = 1;
	while( cycle ) {
		unsigned char* respbuf;
		for( int i=0; i<N_REPEAT_COMMAND; i++ ) {
			exch::sendCommand( _module, _reset, &respbuf );
			if( respbuf[0] == DSC ) {	// It's OK !!!
				return(1);
			}
		}
		QString str;
		int ret = 0;
		if( respbuf[0] == NOP ) {
			str = "MASS is off or disconnected!";
			ret = -3;	// fatal
		} else if( respbuf[0] == NDV ) {
			str = NO_DEVICE;
			ret = -3;	// fatal
		} else {
			str.sprintf( "Module \"%s\": Error in Reset!",_name );
			ret = 0;
		}
		if( !dialog ) return ret;
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return ret;
			case  QMessageBox::Ignore: return ret;
			case  QMessageBox::Retry : break;
			default : return ret;
		}
	}
	return(0);
}

int Power::getConst( void ) {
	QString str;
	unsigned char *respbuf;

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getConst, str.data(),
				"Error in getConst!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 5 ) {
					_const1 = (int)respbuf[1];
					_const2 = (int)respbuf[2];
					_const3 = (int)respbuf[3];
					_const4 = (int)respbuf[4];
					return(1);
				}
				str.sprintf( "Module \"%s\": Error in getConst!",_name );
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Power::testCrc( void ) {
	QString str;
	unsigned char *respbuf;

	jdebug( "testCrc:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getCrc, str.data(),
				"Error in getCrc!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( (respbuf[0] != 2) || (respbuf[1] != 0) ){
					str.sprintf( "Module \"%s\": EEPROM corrupted!",_name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);
			}
		}
	}
}

int Power::getStatus( uchar& status, int dialog ) {
	QString str;
	unsigned char *respbuf;

	jdebug( "getStatus:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getStatus, str.data(),
				"Error in getStatus!",&respbuf,dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					if( !dialog ) return(0);
					str.sprintf( "Module \"%s\": Invalid status!", _name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { status = respbuf[1]; return(1); }
			}
		}
	}
}

int Power::setVoltage( int hvLev, int dialog, bool _wait=true ) {
	QString str;
	int hvLev_;
	int dataHV,dataHV_;
	unsigned char* respbuf;

	jdebug( "setVoltage:" );
	if( _currentVoltage == hvLev ) return(1);

	dataHV = (int)((double)hvLev * 0.001 * (double)_const1) - _const2;
	jdebug( "dataHV = %d",dataHV );
	if( dataHV < MIN_HV_VALUE ) dataHV = MIN_HV_VALUE;
	if( dataHV > MAX_HV_VALUE ) dataHV = MAX_HV_VALUE;
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module, _setVoltage, (char)dataHV,
			str.data(),"Error in setVoltage!", dialog ) == 0 ) {
			return(0);
		}
		_currentVoltage = -1;
		switch( get( _module, _getVoltage, str.data(),
			"Error in getVoltage!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					if( _const1 == 0 ) {
						hvLev_ = -1;
					} else {
						dataHV_ = (int)respbuf[1];
						hvLev_ = (int)(1000.0*(double)(_const2+dataHV_)
							/ (double)_const1);
						if( hvLev != 0 ) {
							if( fabs((double)(hvLev-hvLev_))/
								(double)hvLev <= VALUE_TOLERANCE ) {
								_currentVoltage = hvLev;
								if( _wait ) exch::wait( HV_WAIT_INTERVAL ); // msec
								return(1);
							}
						} else if( hvLev_ == 0 ) {
							_currentVoltage = 0;
							return(1);
						}
					}
					str.sprintf( "Module \"%s\": Error in setVoltage: %d != %d!",
						_name, hvLev, hvLev_ );
				} else {
					str.sprintf( "Module \"%s\": Error in setVoltage: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Power::getTemper( double& temper, int dialog ) {
	QString str;
	int high, low, dataTemper;
	unsigned char* respbuf;

//	jdebug( "getTemper:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch( get( _module, _getTemper, str.data(),
			"Error in getTemper!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 3 ) {
					low = (int)respbuf[1];
					high = (int)respbuf[2];
					dataTemper = (high << 8) + low;
					temper = -103.0 + (double)dataTemper / 4.0;
					return(1);
				} else {
					str.sprintf( "Module \"%s\": Error in getTemper: Numb. of ret. val. != 3!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Power::turnHV( const QString& on, int dialog ) {
	int on_int = 0;
	if( on.upper() == "ON" ) on_int = ON;
	if( on.upper() == "OFF" ) on_int = OFF;
	return( turnHV( on_int,dialog ) );
}

int Power::turnHV( int on, int dialog ) {
	int ret;
	if( on == ON ) ret = setHVOn( dialog );
	else ret = setHVOff( dialog );
	return( ret );
}

int Power::setHVOn( int dialog ) {
	int ret;
	QString str;
	int lock_hv;

	jdebug( "setHVOn:" );
//	if( isHVOn() ) return(1);
	if( _safety ) {
		setSafetyOff( dialog );
		setSafetyOn( dialog );
	}

RETRY:
	str.sprintf( "Module \"%s\"",_name );
	ret = set_hvOn( _module,_highOn,str.data(),
		"Error in setHVOn!",dialog );
	if( ret == 0 ) return(0);
	exch::wait( HV_LOCK_WAIT ); // 100 millisec
	lock_hv = isHVLock(dialog);
	if( (lock_hv == 0) && (ret == 1) ) return(1); // It's OK
	if( !dialog ) return(0);

	if( lock_hv != 1 ) {
		str.sprintf( "Module \"%s\": Error in setHVOn!",_name );
		switch ( showWarning( str ) ) {
			case  QMessageBox::Retry : goto RETRY;
			case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); return 0;
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

//	OVER_LIGHT && SAFETY_ON:

	str = "Overlight! High Voltage turned off!";
	switch ( showStrongWarning( str,"Ignore","Correct..." ) ) {
		case  QMessageBox::Ignore: return(0);
		case  QMessageBox::Abort : {
			str = "Remove light pollution!";
			switch( showStrongWarning( str,"Ignore","Set HV On" ) ) {
				case	QMessageBox::Ignore: return(0);
				case	QMessageBox::Abort : break;
			}
			setHVOff( dialog );
			setSafetyOff( dialog );
			setSafetyOn( dialog );
			goto RETRY;
		}
		default : return(0);
	}
}

int Power::setHVOff( int dialog ) {
	int ret;
	QString str;

	jdebug( "setHVOff:" );
//	if( isHVOn() ) {
	str.sprintf( "Module \"%s\"",_name );
	ret = set( _module,_highOff,str.data(),
		"Error in setHVOff!",dialog );
//	}
	return( ret );
}

int Power::turnSafety( const QString& _on, int dialog ) {
	int _on_int = 0;
	if( _on.upper() == "ON" ) _on_int = ON;
	if( _on.upper() == "OFF" ) _on_int = OFF;
	return( turnSafety( _on_int,dialog ) );
}

int Power::turnSafety( int _on, int dialog ) {
	int ret;
	if( _on ) ret = setSafetyOn( dialog );
	else ret = setSafetyOff( dialog );
	return( ret );
}

int Power::setSafetyOn( int dialog ) {
	int ret=1;
	QString str;

	jdebug( "setSafetyOn:" );
	if( _safety != ON ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_safetyOn,str.data(),
			"Error in setSafetyOn!",dialog );
		if( ret == 1 ) _safety = ON;
	}
	return( ret );
}

int Power::setSafetyOff( int dialog ) {
	int ret=1;
	QString str;

	jdebug( "setSafetyOff:" );
	if( _safety != OFF ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_safetyOff,str.data(),
			"Error in setSafetyOff!",dialog );
		if( ret == 1 ) _safety = OFF;
	}
	return( ret );
}

int Power::testOverLight( int dialog ) {
	QString str;
	int ret;

//	jdebug( "testOverLight!" );
	while( 1 ) {
		ret = isHVLock(dialog);
		if( ret == 0 ) return(1);
		if( !dialog ) return(0);
		if( ret == -1 ) {
/*	Vsio uzhe skazano v power:get()...
			str.sprintf( "Module \"%s\" : Invalid status!",_name );
			switch ( showWarning( str ) ) {
				case  QMessageBox::Ignore: break;
				case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); break;
				case  QMessageBox::Retry : continue;
			}
*/
		} else {
			jdebug( "power: OverLight!!!" );
			str = "Overlight! High Voltage turned off!";
			switch ( showStrongWarning( str,"Continue","Exit" ) ) {
				case  QMessageBox::Ignore: break;
				case  QMessageBox::Abort : trberr::setCode(TRB_EXIT); break;
			}
			jdebug( "power: setHVOff()" );
			setHVOff( dialog );
			jdebug( "power: setSafetyOff()" );
			setSafetyOff( dialog );
			jdebug( "power: setSafetyOn()" );
			setSafetyOn( dialog );
		}
		return(0);
	}
}

int Power::isOverLight( void ) {
	uchar status;

	if( getStatus( status,0 ) != 1 ) return(-1);
	if( status & OVR_LIGHT ) return(1);
	else return(0);
}

int Power::isHVLock( int dialog ) {
	uchar status;

	if( getStatus( status,dialog ) != 1 ) return(-1);
	if( status & HV_LOCK ) return(1);
	else return(0);
}

int Power::isHVOn( void ) {
	uchar status;

	if( getStatus( status,0 ) != 1 ) return(-1);
	if( status & HV_ON ) {
		jdebug( "HV is ON" );
		return(ON);
	}
	else {
		jdebug( "HV is OFF" );
		return(OFF);
	}
}
