/*--------------------------------------------------------------------------*/
/** @file power.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with high voltage supply module.

                  MASS project: TURBINA module file
                       header file for power.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef POWER_H
#define POWER_H

#include <qstring.h>

#define HV_LOCK_WAIT		100	// 100 millisec
#define HV_WAIT_INTERVAL	3000	// 3 sec
#define MIN_HV_VALUE		0
#define MAX_HV_VALUE		255

// Status of power:

#define	HV_ON		0x01
#define SAFETY_ON	0x02
#define	OVR_LIGHT	0x04
#define HV_LOCK		0x08
#define	TEMPER_READY	0x40

class Power {
private:
	bool	ready;

	const char* _name;
	char	_module;
	int	_const1,_const2,_const3,_const4;
	int	_nIdent, _ident1,_ident2,_ident3,_ident4;

	char _getStatus;
	char _getCrc;
	char _getIdent;
	char _getConst;
	char _reset;
	char _setVoltage;
	char _getVoltage;
	char _highOn;
	char _highOff;

	char _safetyOn;
	char _safetyOff;
	char _getTemper;

	int _safety;
	int _currentVoltage;

public:
	Power( void );
	~Power();
	void	loadConst( void );
	bool	isReady( void ) { return( ready ); }

	int	get( char, char, const char*, const char*, unsigned  char**, int );
	int	set( char, char, const char*, const char*,int );
	int	set_hvOn( char, char, const char*, const char*,int );
	int	set( char, char, char, const char*, const char*,int );
	int	set( char, char, char, char, const char*, const char*,int );

	int getIdent( void );
	int reset( int dialog );
	int getConst( void );
	int testCrc( void );
	int getStatus( uchar&, int );

	int setVoltage( int, int, bool _wait=true );
	int getVoltage( void ) { return( _currentVoltage ); }
	int getTemper( double&, int );
	int turnHV( const QString&, int );
	int turnHV( int, int );
	int setHVOn( int );
	int setHVOff( int );
	int turnSafety( const QString&, int );
	int turnSafety( int, int );
	int setSafetyOn( int );
	int setSafetyOff( int );
	int testOverLight( int );

	int isHVOn( void );
	int isHVLock( int dialog );
	int isSafetyOn( void ) {return( _safety );}
	int isOverLight( void );
};

#endif
