/*--------------------------------------------------------------------------*/
/** @file tracker.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with autoguide.

                  MASS project: TURBINA module file
                       header file for tracker.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef TRACKER_H
#define TRACKER_H

#include <qstring.h>

#define MAX_TRACKER_TEST_INTERVALS	1000	//  5 sec if dev_wait_int = 5 ms

class Tracker {
private:
	bool	ready;

	const char* _name;
	char	_module;
	int	_const1,_const2,_const3,_const4;
	double	_maxShift;
	int	_nIdent, _ident1,_ident2,_ident3,_ident4;

	char _getStatus;
	char _getCrc;
	char _shiftRA;
	char _shiftDec;
	char _moveRA;
	char _moveDec;
	char _forceMotion;
	char _stopMotion;
	char _testMotion;
	char _getIdent;
	char _getConst;
	char _reset;

public:
	Tracker( void );
	~Tracker();
	void	loadConst( void );
	bool	isReady( void ) { return ready; }

	int	get( char, char, const char*, const char*, unsigned  char**, int );
	int	set( char, char, const char*, const char*,int );
	int	set( char, char, char, const char*, const char*,int );
	int	set( char, char, char, char, const char*, const char*,int );

	int getIdent( void );
	int reset( int dialog );
	int getConst( void );
	int testCrc( void );
	int getStatus( uchar& status, int dialog );
	char toCharShift( double shift );
	int shiftRA( double shift, int dialog ); // shift in time-sec
	int shiftDec( double shift, int dialog ); // shift in time-sec
	int smallShiftRA( double shift, int dialog ); // shift in time-sec
	int smallShiftDec( double shift, int dialog ); // shift in time-sec
	int moveRA( char direct, int dialog ); 
	int moveDec( char direct, int dialog );
	int testMotion( int dialog );
	int stopMotion( int dialog );
	int forceMotion( int dialog );
/*
	int getStopCorrect( void );
	int getStopCorrectNew( void );
	void startCorrection( void );
	void acceptCorrection( int );
*/
};

#endif
