#define QT_THREAD_SUPPORT
#include <qthread.h>
#include <math.h>
#include "trbexport.h"
#include "names.h"

QMutex stateExportMutex;

ExportTable::ExportTable( void ) {
}

ExportTable::~ExportTable() {
}

void ExportTable::setNum( int row, int col, double val,
			const char* format ) {
	QString str;
	int prec = 0;
	double val_abs = fabs( val );
	if( strcmp( format,fluxFormat ) == 0 ) {
		if( val_abs < 1.0 ) prec = 4;
		else if( val_abs < 10.0 ) prec = 3;
		else if( val_abs < 100.0 ) prec = 2;
		str = str.setNum( val,'f',prec );
	} else {
		str.sprintf( format,val );
	}
	if( (row < MAX_ROW) && (col < MAX_COL) ) {
		table[row][col] = str;
	}
}

void ExportTable::setNum( int row, int col, int val,
			const char* format ) {
	QString str;
//	str.sprintf( format, val );
	if( (row < MAX_ROW) && (col < MAX_COL) ) {
		table[row][col] = str.setNum( val );
	}
}

void ExportTable::clearRow( int row ) {
	if( row >= MAX_ROW ) return;
	for( int col=0; col<MAX_COL; col++ ) {
		setText( row,col,"" );
	}
}

void ExportTable::setHHead( int row, const char* col0, const char* col1=NULL,
			const char* col2=NULL, const char* col3=NULL,
			const char* col4=NULL, const char* col5=NULL,
			const char* col6=NULL, const char* col7=NULL,
					const char* col8=NULL ) {
	if( row >= MAX_ROW ) return;
	clearRow( row );
	setText( row,0,col0 );
	if( col1 ) setText( row,1,col1 );
	if( col2 ) setText( row,2,col2 );
	if( col3 ) setText( row,3,col3 );
	if( col4 ) setText( row,4,col4 );
	if( col5 ) setText( row,5,col5 );
	if( col6 ) setText( row,6,col6 );
	if( col7 ) setText( row,7,col7 );
	if( col8 ) setText( row,8,col8 );
}

void ExportTable::setText( int row, int col, const char* text ) {
	if( (row < MAX_ROW) && (col < MAX_COL) ) {
		table[row][col] = text;
	}
}

QString& ExportTable::getData( int row, int col ) {
	return( table[row][col] );
}

namespace StateExport {
	bool _isStartCounting = false;
	bool _isEndCounting = false;
	bool _isWithDevice = false;
	bool _isWithoutDevice = false;
	bool _isChanged = false;
	DeviceState state;
	bool _isCorrectionReady = false;
	bool _isError = false;

	bool _isUserTrack = false;
	int _trackDirection;
	int _trackCoordinate;

	QString errMessage;
}

void StateExport::startUserTrack( int coordinate, int direction ) {
	stateExportMutex.lock();
	_isUserTrack = true;
	_trackCoordinate = coordinate;
	_trackDirection = direction;
	stateExportMutex.unlock();
}

void StateExport::stopUserTrack( void ) {
	stateExportMutex.lock();
	_isUserTrack = false;
	stateExportMutex.unlock();
}

bool StateExport::isUserTrack( void ) {
        stateExportMutex.lock();
	bool ret = _isUserTrack;
	stateExportMutex.unlock();
	return ret;
}

int StateExport::trackDirection( void ) {
        stateExportMutex.lock();
	int ret = _trackDirection;
	stateExportMutex.unlock();
	return ret;
}

int StateExport::trackCoordinate( void ) {
        stateExportMutex.lock();
	int ret = _trackCoordinate;
	stateExportMutex.unlock();
	return ret;
}

int StateExport::door( void ) {
	stateExportMutex.lock();
	int on = state.door;
	stateExportMutex.unlock();
	return on;
}

int StateExport::hv( void ) {
	stateExportMutex.lock();
	int on = state.hv;
	stateExportMutex.unlock();
	return on;
}

int StateExport::buttons( void ) {
	stateExportMutex.lock();
	int on = state.buttons;
	stateExportMutex.unlock();
	return on;
}

int StateExport::illum( void ) {
	stateExportMutex.lock();
	int on = state.illum;
	stateExportMutex.unlock();
	return on;
}

int StateExport::light( void ) {
	stateExportMutex.lock();
	int on = state.light;
	stateExportMutex.unlock();
	return on;
}

int StateExport::safety( void ) {
	stateExportMutex.lock();
	int on = state.safety;
	stateExportMutex.unlock();
	return on;
}

int StateExport::aperture( void ) {
	stateExportMutex.lock();
	int apert = state.aperture;
	stateExportMutex.unlock();
	return apert;
}

double StateExport::temperature( void ) {
	stateExportMutex.lock();
	double t = state.temperature;
	stateExportMutex.unlock();
	return t;
}

int StateExport::brightness( void ) {
	stateExportMutex.lock();
	int value = state.brightness;
	stateExportMutex.unlock();
	return value;
}

bool StateExport::error( void ) {
	stateExportMutex.lock();
	bool err = _isError;
	stateExportMutex.unlock();
	return err;
}

bool StateExport::correction( void ) {
	stateExportMutex.lock();
	bool ready = _isCorrectionReady;
	stateExportMutex.unlock();
	return ready;
}

void StateExport::exportError( QString& mess ) {
	stateExportMutex.lock();
	_isError = true;
	errMessage = mess;
	stateExportMutex.unlock();
}

QString StateExport::message( void ) {
	stateExportMutex.lock();
	QString mess = errMessage;
	stateExportMutex.unlock();
	return mess;
}

bool StateExport::isStartCounting( void ) {
	stateExportMutex.lock();
	bool startC = _isStartCounting;
	stateExportMutex.unlock();
	return startC;
}

bool StateExport::isWithDevice( void ) {
	stateExportMutex.lock();
	bool d = _isWithDevice;
	stateExportMutex.unlock();
	return d;
}

bool StateExport::isWithoutDevice( void ) {
	stateExportMutex.lock();
	bool d = _isWithoutDevice;
	stateExportMutex.unlock();
	return d;
}

bool StateExport::isEndCounting( void ) {
	stateExportMutex.lock();
	bool endC = _isEndCounting;
	stateExportMutex.unlock();
	return endC;
}

bool StateExport::isChanged( void ) {
	stateExportMutex.lock();
	bool chng = _isChanged;
	stateExportMutex.unlock();
	return chng;
}

void StateExport::clearError( void ) {
	stateExportMutex.lock();
	_isError = false;
	stateExportMutex.unlock();
}

void StateExport::setCorrectionReady( bool ready ) {
	stateExportMutex.lock();
	_isCorrectionReady = ready;
	stateExportMutex.unlock();
}

void StateExport::clearCorrectionReady( void ) {
	stateExportMutex.lock();
	_isCorrectionReady = false;
	stateExportMutex.unlock();
}

void StateExport::setHV( int on ) {
	stateExportMutex.lock();
	state.hv = on;
	_isChanged = true;
	stateExportMutex.unlock();
}

void StateExport::setTemperature( double value ) {     
	stateExportMutex.lock();
	state.temperature = value;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setBrightness( int value ) {     
	stateExportMutex.lock();
	state.brightness = value;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setDoor( int open ) {     
	stateExportMutex.lock();
	state.door = open;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setButtons( int on ) {     
	stateExportMutex.lock();
	state.buttons = on;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setIllum( int on ) {     
	stateExportMutex.lock();
	state.illum = on;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setLight( int on ) {     
	stateExportMutex.lock();
	state.light = on;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setSafety( int on ) {     
	stateExportMutex.lock();
	state.safety = on;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setAperture( int aperture ) {     
	stateExportMutex.lock();
	state.aperture = aperture;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setStartCounting( void ) {
	stateExportMutex.lock();
	_isEndCounting = false; 
	_isStartCounting = true;
	stateExportMutex.unlock();
}

void StateExport::clearStartCounting( void ) {
	stateExportMutex.lock();
	_isStartCounting = false;
	stateExportMutex.unlock();
}

void StateExport::setEndCounting( void ) {
        stateExportMutex.lock();
        _isStartCounting = false;
        _isEndCounting = true;
        stateExportMutex.unlock();
}

void StateExport::clearEndCounting( void ) {
	stateExportMutex.lock();
	_isEndCounting = false;
	stateExportMutex.unlock();
}

void StateExport::setWithDevice( void ) {
	stateExportMutex.lock();
	_isWithDevice = true; 
	_isChanged = true;
	stateExportMutex.unlock();
}

void StateExport::clearWithDevice( void ) {
	stateExportMutex.lock();
	_isWithDevice = false;
	stateExportMutex.unlock();
}

void StateExport::setWithoutDevice( void ) {
	stateExportMutex.lock();
	_isWithoutDevice = true; 
	_isChanged = true;
	stateExportMutex.unlock();
}

void StateExport::clearWithoutDevice( void ) {
	stateExportMutex.lock();
	_isWithoutDevice = false;
	stateExportMutex.unlock();
}

void StateExport::clearChanged( void ) {
	stateExportMutex.lock();
	_isChanged = false;
	stateExportMutex.unlock();
}
