/*--------------------------------------------------------------------------*/
/** @file trbexport.h
@brief  The class is presented with static fields which serve for exchange of 
data between the Scenario thread and Main-thread of the program.

                  MASS project: TURBINA module file
                       header file for trbexport.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef TRBEXPORT_H
#define TRBEXPORT_H

#include <qstring.h>

struct DeviceState {
	DeviceState( void ) {hv=-1;illum=-1;light=-1;safety=-1;
			door=-1;buttons=-1;aperture=-1;
			temperature=99.9;brightness=-1; }
	int	hv;		// on/off
	int	illum;		// on/off
	int	light;		// on/off
	int	safety;		// on/off
	int	door;		// on/off (open/close)
	int	buttons;	// unlock/lock
	int	aperture;	// number
	double	temperature;
	int	brightness;	// of illumination
};

struct Progress {
	QString	message;
	double	progress;
};

const int MAX_ROW=40;
const int MAX_COL=7;

class ExportText {
public:
	static QString text;
	static bool isChanged;
};

class ExportTable {
private:
	static QString table[MAX_ROW][MAX_COL];
public:
	static bool isChanged;
	static bool clear;
	static bool adjust;
	static int startRow,nRows;
	static int startCol,nCols;

	ExportTable( void );
	~ExportTable();

	void setNum( int row, int col, double val, const char* format );
	void setNum( int row, int col, int val, const char* format );
	void setHHead( int,const char*,const char* = NULL,
		const char* = NULL, const char* = NULL,
		const char* = NULL, const char* = NULL,
		const char* = NULL, const char* = NULL,
					const char* = NULL );
	void clearRow( int );
	void setText( int row, int col, const char* text );
	QString& getData( int row, int col );
};

                              
class InfoExport {
public:
	static bool	isProgressChanged;
	static Progress	progress;
	static bool	isModeChanged;
	static Progress	mode;
	static bool	isScenarioChanged;
	static Progress	scenario;
	static bool	isScenario;
	static bool	isScenarioStateChanged;
	static bool	isMessage;
	static QString	message;
	static int	ms;
	static bool	isStarPositionChanged;
	static int	starPosition;
};

namespace StateExport {

	void exportError( QString& mess );
	void clearError( void );
	bool error( void );
	QString message( void );
	void setCorrectionReady( bool ready );
	void clearCorrectionReady( void );

	void startUserTrack( int coordinate, int direction );
	void stopUserTrack( void );
	bool isUserTrack( void );
	int trackDirection( void );
	int trackCoordinate( void );

	bool correction( void );
	int door(void);
	int hv(void);
	int buttons(void);
	int illum(void);
	int light(void);
	int safety(void);
	int aperture(void);
	double temperature(void);
	int brightness(void);

	bool isStartCounting( void );
	bool isEndCounting( void );

	bool isWithDevice( void );
	bool isWithoutDevice( void );
	bool isChanged( void );

	void setDoor( int open );
	void setHV( int on );
	void setButtons( int on );
	void setIllum( int on );
	void setLight( int on );
	void setSafety( int on );
	void setAperture( int aperture );
	void setTemperature( double value );
	void setBrightness( int value );

	void setStartCounting( void );
	void clearStartCounting( void );
	void setEndCounting( void );
	void clearEndCounting( void );
	void setWithDevice( void );
	void setWithoutDevice( void );
	void clearWithDevice( void );
	void clearWithoutDevice( void );
	void clearChanged( void );
};
#endif