#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include "tree.h"
#include "access.h"
#include "tools.h"

//	---------------		class Link:		-----------------

void Link::initLink( char _id, void* _data, CFGBranch* _branch ) {
	data = _data;
	id = _id;
	next = NULL;
	_branch->addLink( this );
}

void Link::initLink( char _id, void* _data, CFGTree* _tree ) {
	data = _data;
	id = _id;
	next = NULL;
	_tree->addLink( this );
}

Link::~Link( void ) {
}

const char* Link::BranchName( void ) {
	CFGBranch* branch;	
	
	if( id == 'b' ) {
		branch = (CFGBranch*)data;
		return branch->getName();
	}
	return NULL;
}

const char* Link::getLeafKey( void ) {
	CFGLeaf* leaf;

	if( id == 'l' ) {
		leaf = (CFGLeaf*)data;
		return leaf->getKey();
	}
	return NULL;
}

char* Link::getLeafData( void ) {
	CFGLeaf* leaf;

	if( id == 'l' ) {
		leaf = (CFGLeaf*)data;
		return (char*)(leaf->getData()->data());
	}
	return NULL;
}

//	---------------		class Leaf:		-----------------

void CFGLeaf::initLeaf( const char* key_,
			bool for_root = true,
			bool enable_change = true,
			bool count_disable = true ) {
	data = "Empty";
	_changed = true;
	key = key_;
	validator_type = NULL;
	_for_root = for_root;
	_enable_change = enable_change;
	_count_disable = count_disable;
}

void CFGLeaf::initLeaf( const char* key_,
			const char* validator_type_,
			bool for_root = true,
			bool enable_change = true,
			bool count_disable = true ) {
	_changed = true;
	data = "Empty";
	key = key_;
	validator_type = validator_type_;
	_for_root = for_root;
	_enable_change = enable_change;
	_count_disable = count_disable;
}

CFGLeaf::~CFGLeaf() {
}

int CFGLeaf::replaceData( const QString& data_ ) {
	data = data_;
	return 1;
}

void CFGLeaf::setHelp( const char* help_ ) {
	help = help_;
}

//	---------------		class CFGBranch:		-----------------

void CFGBranch::initBranch( const char* _name ) {
	name = _name;
	first_link = NULL;
	last_link = NULL;
	current_link = NULL;
	n_leaf = 0;
	n_subbranch = 0;
}

/*
CFGBranch::CFGBranch( const char* _name, Link* _first_link ) {
	name = _name;
	first_link = _first_link;
	last_link = NULL;
	current_link = NULL;
	n_leaf = 0;
	n_subbranch = 0;
}
*/

CFGBranch::~CFGBranch( void ) {
}

void CFGBranch::addLink( Link* _link ) {
	if( last_link ) {
		last_link->setNext( _link );
		last_link = _link;
	} else {
		first_link = _link;
		last_link = _link;
	}
	if( _link->getId() == 'l' ) n_leaf++;
	else if( _link->getId() == 'b' ) n_subbranch++;
	else jdebug( "addLink: ERROR!!!");
}

CFGBranch* CFGBranch::findBranch( const char* branch_name ) {
	Link* link;
	link = first_link;

	if( name == "Empty" ) {jdebug( "It's Empty Branch!" );return &branch_empty;}
	do {
		if( link->getId()=='b' ) {
			if( !strcasecmp( link->BranchName(),branch_name ) )
				return( link->getBranch() );
		}
		if( !link->getNext() ) break;
		link = link->getNext();

	} while(1);
	return &branch_empty;
}

void CFGBranch::setHelp( const char* help_ ) {
	getHelp()->operator=( help_ );
}


CFGLeaf* CFGBranch::findLeaf( const char* leaf_key ) {
	Link* link;
	link = first_link;

	if( name == "Empty" ) {jdebug( "It's Empty Branch!" );return &leaf_empty;}
	do {
		if( link->getId() == 'l' ) {
			if( !strcasecmp( link->getLeafKey(),leaf_key ) ) {
				return link->getLeaf();
			}
		}
		if( !link->getNext() ) break;
		link = link->getNext();
	} while(1);
	return &leaf_empty;
}

QString* CFGBranch::findLeaf_str( const char* leaf_key ) {
	Link* link;
	link = first_link;
	do {
		if( link->getId() == 'l' ) {
			if( !strcasecmp( link->getLeafKey(),leaf_key ) ) {
				return link->getLeaf()->getData();
			}
		}
		if( !link->getNext() ) break;
		link = link->getNext();
	} while(1);
	return leaf_empty.getData();
}

char* CFGBranch::findLeaf_ch( const char* leaf_key ) {
	Link* link;
	link = first_link;
	do {
		if( link->getId() == 'l' ) {
			if( !strcasecmp( link->getLeafKey(),leaf_key ) ) {
				return(char*)(link->getLeaf()->getData()->data());
			}
		}
		if( !link->getNext() ) break;
		link = link->getNext();
	} while(1);
	return (char*)(leaf_empty.getData()->data());
}

int CFGBranch::replaceLeaf( const char* key_, const char* data_ ) {
	(findLeaf( key_ )->getData())->operator=( data_ );
	if( !findLeaf( key_ )->getData() ) return 0;	
	return(1);
}

int CFGBranch::replaceChanged( const char* key_, bool isChanged ) {
	CFGLeaf* leaf = findLeaf( key_ );
	leaf->setChanged( isChanged );
	return(1);
}

int CFGBranch::setLeafHelp( const char* key_, const char* help_ ) {
	(findLeaf( key_ )->getHelp())->operator=( help_ );
	if( !findLeaf( key_ )->getHelp() ) return 0;	
	return(1);
}

int CFGBranch::copyBranch( CFGBranch *oldbranch,CFGBranch *newbranch ) {

	CFGBranch *s_branch, *s_s_branch;
	CFGBranch *s_oldbranch, *s_s_oldbranch;
	QString text;

	oldbranch->toFirst();
	while( oldbranch->getCurrentLink() ) {
		if( oldbranch->getCurrentLink()->getId() == 'l' ) {
			newbranch->replaceLeaf( oldbranch->getCurrentLink()->getLeafKey(), 
				oldbranch->getCurrentLink()->getLeafData() ); 
			newbranch->replaceChanged( oldbranch->getCurrentLink()->getLeafKey(), 
				oldbranch->getCurrentLink()->getLeaf()->isChanged() ); 
			newbranch->setLeafHelp( oldbranch->getCurrentLink()->getLeafKey(), 
				oldbranch->getCurrentLink()->getLeaf()->getHelp()->data() ); 
		} else if( oldbranch->getCurrentLink()->getId() == 'b' ) {
			s_oldbranch = oldbranch->getCurrentLink()->getBranch();
			
			s_oldbranch->toFirst();
			s_branch = newbranch->findBranch( s_oldbranch->getName() );
			s_branch->setHelp( s_oldbranch->getHelp()->data() );	
		while( s_oldbranch->getCurrentLink() ) {
				if( s_oldbranch->getCurrentLink()->getId() == 'l' ) {
					s_branch->replaceLeaf( s_oldbranch->getCurrentLink()->getLeafKey(),
								s_oldbranch->getCurrentLink()->getLeafData() );
					s_branch->replaceChanged( s_oldbranch->getCurrentLink()->getLeafKey(), 
								s_oldbranch->getCurrentLink()->getLeaf()->isChanged() ); 
					s_branch->setLeafHelp( s_oldbranch->getCurrentLink()->getLeafKey(), 
						s_oldbranch->getCurrentLink()->getLeaf()->getHelp()->data() ); 
				} else if( s_oldbranch->getCurrentLink()->getId() == 'b' ) {
					s_s_oldbranch = s_oldbranch->getCurrentLink()->getBranch();
                  	s_s_oldbranch->toFirst();
					s_s_branch = s_branch->findBranch( s_s_oldbranch->getName() );
					s_s_branch->setHelp( s_s_oldbranch->getHelp()->data() );	

					while( s_s_oldbranch->getCurrentLink() ) {
						if( s_s_oldbranch->getCurrentLink()->getId() == 'l' ) {
							s_s_branch->replaceLeaf(s_s_oldbranch->getCurrentLink()->getLeafKey(),
								s_s_oldbranch->getCurrentLink()->getLeafData() );
							s_s_branch->replaceChanged( s_s_oldbranch->getCurrentLink()->getLeafKey(), 
								s_s_oldbranch->getCurrentLink()->getLeaf()->isChanged() ); 
							s_s_branch->setLeafHelp( s_s_oldbranch->getCurrentLink()->getLeafKey(), 
								s_s_oldbranch->getCurrentLink()->getLeaf()->getHelp()->data() ); 
						}
						s_s_oldbranch->toNext();
					}	
				}
				s_oldbranch->toNext();
			}
		}
		oldbranch->toNext();
	}
	return(1);   // It's OK
}

// 	---------------		class CFGTree:		-----------------

CFGTree::CFGTree( void ) {
	last_link = NULL;
	first_link = NULL;
	current_link = NULL;
	_changed = 0;
}

/*
CFGTree::CFGTree( Link* _first_link ) {
	last_link = NULL;
	first_link = _first_link;
	current_link = NULL;
}
*/

CFGTree::~CFGTree( void ) {
}

void CFGTree::addLink( Link* _link ) {
	if( last_link ) {
		last_link->setNext( _link );
		last_link = _link;
	} else {
		first_link = _link;
		last_link = _link;
	}
}

CFGBranch* CFGTree::findBranch( const char* branch_name ) {
	Link* link;
	link = first_link;
	do {
		if( link->getId()=='b' ) {
			if( !strcasecmp( link->BranchName(),branch_name ) )
				return( link->getBranch() );
		}
		if( !link->getNext() ) break;
		link = link->getNext();

	} while(1);
	return &branch_empty;
}

int CFGTree::copyTree( CFGTree*oldtree, CFGTree*newtree ) {

	CFGBranch *branch, *s_branch, *s_s_branch;
	CFGBranch *oldbranch, *s_oldbranch, *s_s_oldbranch;
	QString text;

//	jdebug( "copyTree!!! changed!!!" );
	setChanged(1);
	oldtree->toFirst();
	while( oldtree->getCurrentLink() ) {
		if( oldtree->getCurrentLink()->getId() == 'b' ) {
			oldbranch = oldtree->getCurrentLink()->getBranch();
			oldbranch->toFirst();
			branch = newtree->findBranch( oldbranch->getName() );
			branch->setHelp( oldbranch->getHelp()->data() );	

			while( oldbranch->getCurrentLink() ) {
				if( oldbranch->getCurrentLink()->getId() == 'l' ) {
					branch->replaceLeaf( oldbranch->getCurrentLink()->getLeafKey(), 
						oldbranch->getCurrentLink()->getLeafData() ); 
					branch->replaceChanged( oldbranch->getCurrentLink()->getLeafKey(), 
						oldbranch->getCurrentLink()->getLeaf()->isChanged() ); 
					branch->setLeafHelp( oldbranch->getCurrentLink()->getLeafKey(), 
						oldbranch->getCurrentLink()->getLeaf()->getHelp()->data() ); 
				} else if( oldbranch->getCurrentLink()->getId() == 'b' ) {
					s_oldbranch = oldbranch->getCurrentLink()->getBranch();
					s_oldbranch->toFirst();
					s_branch = branch->findBranch( s_oldbranch->getName() );
					s_branch->setHelp( s_oldbranch->getHelp()->data() );	

					while( s_oldbranch->getCurrentLink() ) {
						if( s_oldbranch->getCurrentLink()->getId() == 'l' ) {
							s_branch->replaceLeaf( s_oldbranch->getCurrentLink()->getLeafKey(),
									s_oldbranch->getCurrentLink()->getLeafData() );
							s_branch->replaceChanged( s_oldbranch->getCurrentLink()->getLeafKey(), 
									s_oldbranch->getCurrentLink()->getLeaf()->isChanged() ); 
							s_branch->setLeafHelp( s_oldbranch->getCurrentLink()->getLeafKey(), 
									s_oldbranch->getCurrentLink()->getLeaf()->getHelp()->data() ); 
						} else if( s_oldbranch->getCurrentLink()->getId() == 'b' ) {
							s_s_oldbranch = s_oldbranch->getCurrentLink()->getBranch();
        	            	s_s_oldbranch->toFirst();
							s_s_branch = s_branch->findBranch( s_s_oldbranch->getName() );
							s_s_branch->setHelp( s_s_oldbranch->getHelp()->data() );	
							while( s_s_oldbranch->getCurrentLink() ) {
								if( s_s_oldbranch->getCurrentLink()->getId() == 'l' ) {
									s_s_branch->replaceLeaf(s_s_oldbranch->getCurrentLink()->getLeafKey(),
										s_s_oldbranch->getCurrentLink()->getLeafData() );
									s_s_branch->replaceChanged( s_s_oldbranch->getCurrentLink()->getLeafKey(), 
										s_s_oldbranch->getCurrentLink()->getLeaf()->isChanged() ); 
									s_s_branch->setLeafHelp( s_s_oldbranch->getCurrentLink()->getLeafKey(), 
										s_s_oldbranch->getCurrentLink()->getLeaf()->getHelp()->data() ); 
								}
								s_s_oldbranch->toNext();
							}	
						}
						s_oldbranch->toNext();
					}
				}
				oldbranch->toNext();
			}
		}	
		oldtree->toNext();
	}
	return(1);   // It's OK
}

int CFGTree::readCfg( const char* fn ) {

	CFGTree* tree = this;

	QString text;
	QFile cfgfile( fn );
	if( !cfgfile.open( IO_ReadOnly ) ) {
		text = fn;
		text += " not found!";
		QMessageBox::critical( 0,"cfg",text.data() ); return(1);
	}
	QTextStream stream( &cfgfile );
	QString line;
	QString help;
	QString d_key;
	int i_spb,i_eol;
	CFGBranch *branch, *s_branch=NULL, *s_s_branch=NULL;
	CFGLeaf *leaf;
	branch = &branch_empty;
	s_branch = &branch_empty;
	CFGBranch* local_branch = branch;
//	char *quote;
	QString *item;	
	int pos_help;

	while( !stream.eof() ) {
		line = stream.readLine();
		if( (pos_help=line.find(";")) != -1 ) {
			help = line.mid( pos_help+1,line.length() )
				.simplifyWhiteSpace();
			line.truncate( pos_help );
		} else help = "";
		line = line.simplifyWhiteSpace();
		if( line[0] == '#' ) continue;
		if( (i_spb=line.find( " ",0 )) > -1 ) {
			i_eol = line.length();
			d_key = line.left( i_spb );
			item = new QString( line.right(i_eol-i_spb-1) );
			CHECK_PTR( item );
			if( !strcmp(d_key.upper().data(),"SECTION") ) {
				item->remove( 0,1 ); //  
				item->truncate( item->find( '\"',1 ) ); //      
				branch = tree->findBranch( item->data() );//   
				if( strcasecmp( branch->getName(),"Empty") ) {
					branch->setHelp( help.data() );
					local_branch = branch;
					delete item;
				} else {
// NEW !!!
					printf( "Invalid Section \"%s\"\n",item->data() );
					delete item;
/*
					text = fn;
					text += ": Invalid Section: \"";
					text += ( *item );
					text += "\"";
					QMessageBox::critical( 0,"invalid",text.data() );
					delete item;
					return(2);
*/
				}
//				printf( "::: %s %s \n",d_key.data(),local_branch->getName() );
			} else if( !strcmp(d_key.upper().data(),"SUBSECTION") ) { 
				item->remove( 0,1 ); //  
				item->truncate( item->find( '\"',1 ) ); //      
				s_branch = branch->findBranch( item->data() );
				if( strcasecmp( s_branch->getName(),"Empty" ) ) {
					s_branch->setHelp( help.data() );
					local_branch = s_branch;
					delete item;
				} else {
// NEW !!!
					printf( "Invalid SubSection \"%s\"\n",item->data() );
					delete item;
/*
					text = fn;
					text += ": Invalid SubSection: \"";
					text +=( *item );
					text += "\" in Section \"";
					text += branch->getName();
					text += "\"";
					QMessageBox::critical( 0,"invalid",text.data() );
					delete item;
					return(2);
*/
				}	
//				printf( "::: %s %s \n",d_key.data(),s_branch->getName() );
			} else if( !strcmp(d_key.upper().data(),"SUBSUBSECTION") ) { 
				item->remove( 0,1 ); //  
				item->truncate( item->find( '\"',1 ) ); //      
				s_s_branch = s_branch->findBranch( item->data() ); //   
				if( strcasecmp(s_s_branch->getName(),"Empty") ) {
					s_s_branch->setHelp( help.data() );
					local_branch = s_s_branch;
					delete item;
				} else {
// NEW !!!
					printf( "Invalid SubSubSection \"%s\"\n",item->data() );
					delete item;
/*
					text = fn;
					text += ": Invalid SubSubSection: \"";
					text += ( *item );
					text += "\" in Section \"";
					text += branch->getName();
					text += "\" Subsection \"";
					text += s_branch->getName();
					text += "\"";
					QMessageBox::critical( 0,"invalid",text.data() );
					delete item;
					return(2);
*/
				}	
//				printf( "::: %s %s \n",d_key.data(),s_s_branch->getName() );
			} else {
				leaf = local_branch->findLeaf( d_key.data() );
				if( strcasecmp( leaf->getKey(),"Empty") ) {
					leaf->replaceData( *item );
					leaf->setHelp( help.data() );
//					printf( "		%s %s \n",d_key.data(),item->data() );
				} else {
// NEW !!!
					printf( "Invalid Key \"%s\"\n",d_key.data() );
/*
					text = fn;
					text += ": Invalid key: ";
					text += d_key;
					text += " in Section \"";
					text += local_branch->getName();
					text += "\"";
					QMessageBox::critical( 0,"invalid",text ); 
					return(3);
*/
				}
			}
		} else {
			i_eol = line.length();
			d_key = line.left( i_spb );
			if( !strcmp(d_key.upper().data(),"ENDSUBSECTION") ) local_branch = branch;
			else if( !strcmp(d_key.upper().data(),"ENDSUBSUBSECTION") ) local_branch = s_branch;
		}
	}
	cfgfile.close();

	tree->toFirst();
	while( tree->getCurrentLink() ) {
		if( tree->getCurrentLink()->getId() == 'b' ) {
//			printf( "Current Section: %s\n",tree->CurrentName() );
			branch = tree->getCurrentLink()->getBranch();
			branch->toFirst();
			while( branch->getCurrentLink() ) {
				if( branch->getCurrentLink()->getId() == 'l' ) {
//					printf( "    Current Leaf: %c %s: %s\n",branch->getCurrentLink()->getId(),
//										branch->getCurrentLink()->getLeafKey(),
//										branch->getCurrentLink()->getLeafData() );
					if( !strcasecmp(branch->getCurrentLink()->getLeafData(),"Empty") ) {
						text = fn;
						text +=( ": Section \"" );
						text +=( branch->getName() );
						text +=( "\" key: " );
						text +=( branch->getCurrentLink()->getLeafKey() );
						text +=( " not found!" );
						QMessageBox::critical( 0,"not_found!",text );
						return(4);
					}
				} else if( branch->getCurrentLink()->getId() == 'b' ) {
					s_branch = branch->getCurrentLink()->getBranch();
					s_branch->toFirst();
//					printf( "    Current SubBranch: %s: \n",s_branch->getName() );
					while( s_branch->getCurrentLink() ) {
						if( s_branch->getCurrentLink()->getId() == 'l' ) {
//							printf( "		Current Leaf: %c %s: %s\n",s_branch->getCurrentLink()->getId(),
//											s_branch->getCurrentLink()->getLeafKey(),
//											s_branch->getCurrentLink()->getLeafData() );
							/*
							if( !strcasecmp(s_branch->getCurrentLink()->getLeafData(),"Empty") ) {
								text = fn;
								text +=( ": Section \"" );
								text +=( s_branch->getName() );
								text +=( "\" key: " );
								text +=( s_branch->getCurrentLink()->getLeafKey() );
								text +=( " not found!" );
								QMessageBox::critical( 0,"not_found!",text );
								return(4);
							}
							*/
						} else if( s_branch->getCurrentLink()->getId() == 'b' ) {
							s_s_branch = s_branch->getCurrentLink()->getBranch();
							s_s_branch->toFirst();
//							printf( "    		Current SubSubBranch: %s: \n", s_s_branch->getName() );
							while( s_s_branch->getCurrentLink() ) {
								if( s_s_branch->getCurrentLink()->getId() == 'l' ) {
//									printf( "			Current Leaf: %c %s: %s\n",s_s_branch->getCurrentLink()->getId(),
//														s_s_branch->getCurrentLink()->getLeafKey(),
//														s_s_branch->getCurrentLink()->getLeafData() );
									/*
									if( !strcasecmp(s_s_branch->getCurrentLink()->getLeafData(),"Empty") ) {
										text = fn;
										text +=( ": Section \"" );
										text +=( s_s_branch->getName() );
										text +=( "\" key: " );
										text +=( s_s_branch->getCurrentLink()->getLeafKey() );
										text +=( " not found!" );
										QMessageBox::critical( 0,"not_found!",text );
										return(4);
									}
									*/
								}
								s_s_branch->toNext();
							}	
						}
						s_branch->toNext();
					}
				}
				branch->toNext();
			}
		}	
		tree->toNext();
	}
	return(0);
}


int CFGTree::saveToFile( const char* fn ) {
	CFGBranch *branch,*s_branch,*s_s_branch;

	QFile f( fn );
	if( !f.open( IO_WriteOnly ) ) {
		QString text = fn;
		text += " I/O error!";
		QMessageBox::critical( 0,"cfg",text.data() );
		return(1);
	}

	setAccess( fn,true );

	toFirst();
	while( getCurrentLink() ) {
		if( getCurrentLink()->getId() == 'b' ) {
			branch = getCurrentLink()->getBranch();
			QString s = "Section \"";
			s +=( CurrentName() );
			s +=( "\"\t;" ); 
			s +=( branch->getHelp()->data() );
			s +=( "\n" );
			f.writeBlock( s.data(),s.length() );
			branch->toFirst();
			while( branch->getCurrentLink() ) {
				if( branch->getCurrentLink()->getId() == 'l' ) {

					s = "\t";
					s +=( branch->getCurrentLink()->getLeafKey() );
					s +=( "\t\t\t" );
					s +=( branch->getCurrentLink()->getLeafData() );
					s +=( "\t;" ); 
					s +=( branch->getCurrentLink()->getLeaf()->getHelp()->data() );
					s +=( "\n" ); 
					f.writeBlock( s.data(),s.length() );

				} else if( branch->getCurrentLink()->getId() == 'b' ) {
					s_branch = branch->getCurrentLink()->getBranch();
					s_branch->toFirst();

					s = "\tSubSection \"";
					s +=( s_branch->getName() );
					s +=( "\"\t;" ); 
					s +=( s_branch->getHelp()->data() );
					s +=( "\n" );
					f.writeBlock( s.data(),s.length() );

					while( s_branch->getCurrentLink() ) {
						if( s_branch->getCurrentLink()->getId() == 'l' ) {
							s = "\t\t";	
							s +=( s_branch->getCurrentLink()->getLeafKey() );
							s +=( "\t\t\t" );
							s +=( s_branch->getCurrentLink()->getLeafData() );
							s +=( "\t;" ); 
							s +=( s_branch->getCurrentLink()->getLeaf()->getHelp()->data() );
							s +=( "\n" ); 
							f.writeBlock( s.data(),s.length() );
						} else if( s_branch->getCurrentLink()->getId() == 'b' ) {
							s_s_branch = s_branch->getCurrentLink()->getBranch();
							s_s_branch->toFirst();

							s = "\t\tSubSubSection \"";
							s +=( s_s_branch->getName() );
							s +=( "\"\t;" ); 
							s +=( s_s_branch->getHelp()->data() );
							s +=( "\n" );
							f.writeBlock( s.data(),s.length() );

							while( s_s_branch->getCurrentLink() ) {
								if( s_s_branch->getCurrentLink()->getId() == 'l' ) {
									s = "\t\t\t";	
									s +=( s_s_branch->getCurrentLink()->getLeafKey() );
									s +=( "\t\t\t" );
									s +=( s_s_branch->getCurrentLink()->getLeafData() );
									s +=( "\t;" ); 
									s +=( s_s_branch->getCurrentLink()->getLeaf()->getHelp()->data() );
									s +=( "\n" ); 
									f.writeBlock( s.data(),s.length() );
								}
								s_s_branch->toNext();
							}	
							s = "\t\tEndSubSubSection\n";
							f.writeBlock( s.data(),s.length() );
						}
						s_branch->toNext();
					}
					s = "\tEndSubSection\n";
					f.writeBlock( s.data(),s.length() );
				}
				branch->toNext();
			}
			s = "EndSection\n";
			f.writeBlock( s.data(),s.length() );
		}	
		toNext();
	}
	f.close();
	setSaved();
	return(0);
}


void CFGTree::emitSignalChanged( void ) {
	emit changed();
}
