#define QT_THREAD_SUPPORT

// Macro to write instant. (base-time related) SIs to mass-file: disabled in
// ver1.3 and further.
//#define WRITESI

#include <unistd.h>
#include <qthread.h>
#include <qmessagebox.h>
#include <qdatetime.h>
#include "scind.hxx"
#include "device.h"
#include "tools.h"
#include "cfg.h"
#include "messages.h"
#include "trbexport.h"
#include "trberror.h"
#include "nrutil.h"
#include "atmos.hxx"
#include "weif.hxx"
#include "unroll.h"
#include "names.h"
#include "outcontrol.h"
#include "summary.h"
#include "access.h"
#include "readmass.h"
#include "star.h"
#include "trbconst.h"

static int count_cycle_condition	= 1;

static QString str_message, str_edit, str_edit1, str_edit2, str_add;

static const char*	nameA		= "A";
static const char*	nameB		= "B";
static const char*	nameC		= "C";
static const char*	nameD		= "D";

static const char*	scenarioDict	= "NGBCVDSXLUQ";

static const QString emptyLine		= "";
static const QString notClosed		= "Input Door is not closed!";
static const QString errAperture	= "Error in Set Aperture!";
static const QString errShiftApert	= "Error in Shift Aperture!";
static const QString errShiftTelesc	= "Error in Shift Telescope!";
static const QString errLight		= "Error in Set Control Light!";
static const QString errIllum		= "Error in Turn Illumination!";
static const QString errCounter		= "Error in Counter setting!";
static const QString errLevel		= "Error in set Discrimination!";
static const QString errVoltage		= "Error in set Voltage!";
static const QString errScan		= "Too many errors in scan!";
static const QString userStop		= "User's stop!";
static const QString abortProcess	= "Measurement aborted!";
static const QString successfully	= "It's OK";
static const QString fluxError		= "Invalid fluxes!";
static const QString nonPoisError	= "Invalid P-parameter!";
static const QString overLightText	= "Overlight! HV turned OFF!";
static const QString badIndex		= "Zero scintillation indices!";
static const QString noStar		= "No Star selected";
static QString invalid			= "Invalid";

static const QString normalName		= "Normal ";
static const QString generalName	= "Generalized ";
static const QString detTestName	= "DetectTest ";
static const QString statTestName	= "StatisTest ";
static const QString estimName		= "Flux ";
static const QString indTestName	= "IndividTest ";
static const QString commonTestName	= "ExchangTest ";
static const QString detCountMeasName	= "Counting ";
static const QString detStatMeasName	= "StatisMeas ";
static const QString backCorrName	= "BackgrCor";
static const QString backMeasName	= "Background ";
static const QString centerCorName	= "CenterCor ";
static const QString starCenterName	= "Centering ";
static const QString focusContrName	= "Focus ";
static const QString trackRAName	= "Tracking RA ";
static const QString trackDecName	= "Tracking Dec ";

const int size_to_proc = 11;
static char to_proc_file[size_to_proc] = {'0','1',' ','0','2',' ','0','3',' ','0','4' };

static int _count_mode = NORMAL_MODE;
static int _old_count_mode;

static FILE *srv_file=NULL, *counts_file=NULL;
extern FILE *out_file;
extern FILE *stm_file;

static int n_run = 0;

extern DevManager	devmanager;
extern Star			star;
extern OutControl	outcontrol;
extern ExportTable	table;
extern CFGTree*		tree;
extern CFGTree*		devicetree;
extern bool			withDevice;

QMutex			userStopMutex;
extern QMutex	countMutex;
extern QMutex	infoExportMutex;

void sendToScreen( const QString& text ) {
	infoExportMutex.lock();
	ExportText::isChanged = true;
	ExportText::text = text;
	infoExportMutex.unlock();
}

void sendToOutFile( const QString& text, const char* prefix="> " ) {
	Time tm = trb_tm::getCurrentUT();
	fprintf( out_file,"%s %s\n",
		getStamp( prefix,tm ), text.data() );
}

void sendErrorInfo( const QString& text ) {
	sendToScreen( text );
	sendToOutFile( text,"> " );
}

void DevManager::cleanUserStop( void ) {
	userStopMutex.lock();
	isUserStop = false;
	userStopMutex.unlock();
}

void DevManager::stopCounting( void ) {
	jdebug( "slotStopCountong!!!" );

	userStopMutex.lock();
	isUserStop = true;
	userStopMutex.unlock();

}

void DevManager::stopScenario( void ) {
	_stopScenario = 1;
}

int testNrError( bool isExportError = false ) {
	int err=0;
	if( nr::erget() ) {
		str_edit = nr::ermessage().c_str();
		sendErrorInfo( str_edit );
		if( isExportError ) StateExport::exportError( str_edit );
		nr::erreset();
		err = 1;
	}
	if( trberr::getCode() ) {
		str_edit = trberr::getMessage();
		sendErrorInfo( str_edit );
		if( isExportError ) StateExport::exportError( str_edit );
		trberr::reset();
		err = 1;
	}
	return err;
}

int atBaseTime( void ) {
	int isAbort = testNrError() || devmanager.testUserStop();
	if( !isAbort ) {
		devmanager.controlUserTrack();
	}
//	jdebug( "atBaseTime:" );

	return isAbort;
}

// new!!!:
void quit_count_cycle(void) {
	jdebug( "quit_count_cycle" );
	_count_mode = END_OF_PROGRAM;
	count_cycle_condition = 0;
	if( countMutex.locked() ) countMutex.unlock();
}

void* count_cycle( void ) {		// Scenario-thread cycle
	atexit( quit_count_cycle );
	while( count_cycle_condition ) {
		jdebug( "count_cycle: try lockcount..." );
		countMutex.lock();
		jdebug( "count_cycle: lockcount" );
		switch ( _count_mode ) {
			case PLAYBACK_MODE           : devmanager.playback_mode(); break;
			case NORMAL_MODE             : devmanager.normal_mode(); break;
			case GENERALIZED_MODE        : devmanager.generalized_mode(); break;
			case EXCHANGE_TEST_MODE      : devmanager.exchange_test_mode(); break;
			case DETECTOR_TEST_MODE      : devmanager.detector_test_mode(); break;
			case STATISTIC_TEST_MODE     : devmanager.statistic_test_mode(); break;
			case DETECTOR_COUNTING_MODE  : devmanager.detector_counting_mode(); break;
			case DETECTOR_STATISTIC_MODE : devmanager.detector_statistic_mode(); break;
			case BACKGROUND_CORR_MODE    : devmanager.background_corr_mode(); break;
			case FLUX_ESTIMATION_MODE    : devmanager.flux_estimation_mode(); break;
			case CENTERING_MODE          : devmanager.centering_mode(); break;
			case CENTERING_CORR_MODE     : devmanager.centering_corr_mode(); break;
			case TRACK_CALIBRATION_MODE  : devmanager.track_calibration_mode(); break;
			case FOCUS_CONTROL_MODE      : devmanager.focus_control_mode(); break;
			case CENTER_CONTROL_MODE     : devmanager.center_control_mode(); break;
			case BACKGROUND_MEASUR_MODE  : devmanager.background_measur_mode(); break;
			case SCENARIO_MODE           : devmanager.scenario_mode(); break;
			case QUIT_PROGRAM            : devmanager.quitProgram(); break;
			case END_OF_PROGRAM          : jdebug("count_cycle: END OF PROGRAM"); break;
			default : jdebug( "!!!!!!!! Invalid _count_mode = %d while=%d",
					_count_mode,count_cycle_condition ); break;
		}
	}
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "count_cycle: unlocked..." );
	int res = 13;
	return( &res );
}

// ----------- Play Back:

void setPlayBackTime( void ) {
	int h,m,s;
	rdm::getut( &h,&m,&s );
	trb_tm::setPlayBackTime(h,m,s);
}

int getModeNumber( void ) {
	int mode = UNDEFINED_MODE;
	QString modeName = rdm::getname().c_str();
	setPlayBackTime();
	modeName.replace( QRegExp(" "),"" );
	if( modeName == "NORMAL" ) mode = NORMAL_MODE;
	else if( modeName == "GENERALIZED" ) mode = GENERALIZED_MODE;
	return mode;
}

int DevManager::runPlayBackMode( FILE* mass_in_file,
				int mode ) {
	jdebug( "runPlayBackMode: mode=%d",mode );
	cleanUserStop();
	_count_mode = mode;
	int success = 1;
	switch( _count_mode ) {
		case NORMAL_MODE	: success = runNormal( mass_in_file );
						break;
		case GENERALIZED_MODE	: success = runGeneralized( mass_in_file );
						break;
	}
	return success;
}

long DevManager::getCountAddress( FILE* mass_in_file,
			bool isgen,
			Time& startBaseTime ) {
	// old data files:
	char expected1 = isgen ? 'j' : 'i' ;
	// new data files:
	char expected2 = isgen ? 's' : 'm' ;
	long address=0;
	bool cycle = true;
	while( cycle ) {
		char prefix=rdm::readline(mass_in_file,'I',stm_file);
		if( prefix == expected1 || prefix == expected2 ) {
			address = rdm::getaddr();
			int h,m,s;
			rdm::getut( &h,&m,&s );
//			printf( "addr=%x ut=%02d:%02d:%02d\n",
//				address,h,m,s );
			startBaseTime.setHMS(h,m,s);
			trb_tm::setPlayBackTime(h,m,s);
			cycle=false;
		} else if( prefix == 'M' ) {
			if( getModeNumber() != _count_mode ) {
				address=rdm::NOADDR;
// vozvrachaem ukazatel' faila nazad, chtoby v osnovnom cycle Play Back-a
// "sygralo" nachalo novoi mody:
// ctoby ne zacyclit'sia beskonechno iz-za fseek:
				if( massInFilePos != rdm::getstartpos() ) {
					massInFilePos = rdm::getstartpos();
					fseek( mass_in_file,massInFilePos,SEEK_SET );
					cycle=false;
				}
			}
		} else if( prefix == 0 ) { // EOF
			address=rdm::NOADDR;
			cycle = false;
		}
	}
	return address;
}

void changeConst( FILE* out_f=NULL ) {
	QString pathName = rdm::getname().c_str();
	QString newData = rdm::getvalue().c_str();
	setPlayBackTime();
	setLeafData( pathName,newData,NULL );
}

void changeStar( void ) {
	QString hr = rdm::getname().c_str();
	setPlayBackTime();
	star.setCurrentStar(hr);
}

void restoreCFG( void ) {
	fseek(out_file,0,SEEK_SET);
	char prefix=0;
	while( (prefix=rdm::readline(out_file)) != 0 ) {
		switch( prefix ) {
			case 'P' : changeConst(NULL); break;
		}
	}
	tree->emitSignalChanged();
}

void DevManager::playback_mode( void ) {
	massInFilePos = -1;
	trberr::enableDialog( false );
// Files:
	if( !_massInputFileName ) {
		terminateCountMode(1);
		sendToScreen( "I/O Error" );
		trberr::enableDialog( true );
		return;
	}
	QString pbDirName = *leafDataToStr( cfgFile, "General",
		"Inputs", "PlayBackDir" );
	prepareDirName( pbDirName,false );
	if( !testDirName( pbDirName, 0 ) ) {
		str_edit = "Invalid path: \"" + pbDirName + "\"";
		sendToScreen( str_edit );
		terminateCountMode(1);
		trberr::enableDialog( true );
		return;
	}
	QString countsFileName = _massInputFileName;
	countsFileName.replace( QRegExp( pbDirName ), _dataDirName );
	countsFileName.replace( QRegExp( pbInFileExt ),countsFileExt );

	// create stat.moments file name: new (v1.3 and later) input raw data source
	QString momentsFileName = _massInputFileName;
	momentsFileName.replace( QRegExp( pbDirName ), _dataDirName );
	momentsFileName.replace( QRegExp( pbInFileExt ),momentsFileExt );
	if( !testFileName( momentsFileName,0 ) ) {
		// *.stm file not found...: test counts file:
		if ( !testFileName( countsFileName,0 ) ) {
			str_edit = "File \"" + momentsFileName + 
					"\" or cnt-file not found!";
			sendToScreen( str_edit );
			terminateCountMode(1);
			trberr::enableDialog( true );
			return;
		} // else - either cnt or stm-file is found
	} 

	QString date_str =  _massInputFileName;
	int currentStar = star.getCurrentStar();

// DateTime:
	int pos = date_str.findRev( "/" );
	date_str = date_str.right( date_str.length()-pos-1 );
	date_str.replace( QRegExp( QString(".")+pbInFileExt ),"" );
	try {
		bool ok;
		int year = date_str.left(2).toInt(&ok)+2000;
		if( !ok ) throw LoadError("");
		int month = date_str.mid(2,2).toInt(&ok);
		if( !ok ) throw LoadError("");
		int day = date_str.right(2).toInt(&ok);
		if( !ok ) throw LoadError("");
		ok = trb_tm::setPlayBackDate( year,month,day );
		if( !ok ) throw LoadError("");
	} catch( LoadError ) {
		str_edit = "Invalid Play Back date in file Name \"" +
			date_str + "\"";
		sendToScreen( str_edit );
		terminateCountMode(1);
		trberr::enableDialog( true );
		return;
	}
	
	// stat.moments-file saving:
	FILE* old_stm_file = stm_file ;
	// counts file saving:
	FILE* old_counts_file = counts_file ;

	// Try to open stat.moments file:
	stm_file = fopen( momentsFileName.data(),"rb" );
	counts_file = fopen( countsFileName.data(),"rb" );
	if ( !stm_file && !counts_file) {
		sendToScreen( "I/O Error" );
		terminateCountMode(1);
		trberr::enableDialog( true );
		return;
	} 
	int isTreeChanged = tree->isChanged();
	FILE* old_out_file = out_file;
	QString outFileName =  _massInputFileName;
	outFileName.replace( QRegExp( pbInFileExt ),pbOutFileExt );
	out_file = fopen( outFileName.data(),"w+" );
	setAccess(  outFileName.data(),false );
	int nMassLines = rdm::grabparams( NULL ); // clear params

	FILE* mass_in_file = fopen(  _massInputFileName.data(),"r" );

	FILE* old_srv_file = srv_file;
	srv_file = NULL;

// Graph:

//	outcontrol.resetGraphReady();
	sum::done();
	sum::init( 0,0,MAX_N_COUNTERS,0 ); // without file, for play back only
	outcontrol.resetSummaryData();
	testNrError();

	int oldSciCountSave = _sciCountSave;
	_sciCountSave = 0;
	Time oldProgramStartTime = trb_tm::programStartTime();

	_isPlayBack = true;
	trb_tm::enablePlayBack( true );	

// Work:
	char prefix;
	bool first = true;
	bool pChanged = false;
	bool isFirstMode = true;
	cleanUserStop();
	bool _abort = false;
	while( !_abort && !testUserStop() &&
			// below stm_file can be null: (for old data)
			(prefix=rdm::readline(mass_in_file,'I',stm_file)) ) {
		if( first ) {
			int h,m,s;
			rdm::getut( &h,&m,&s );
			trb_tm::setStartTime( h,m,s );
			first=false;
		}
		if( prefix == 'P' ) {
			changeConst(out_file);
			pChanged = true;
		} else if( pChanged ) {
			pChanged = false;
			tree->emitSignalChanged();
		}
// Pered switch, chtoby uspet' zagruzit' VSE novye param.
		switch( prefix ) {
			case 'M' : if(isFirstMode) {
					star.init(0);
					if( testNrError() ) {
						_abort = true;
						break;
					}
					isFirstMode = false;
				}
				runPlayBackMode( mass_in_file,
						getModeNumber() );
				break;
			case 'i' :	// means old data with instant indices in mass-file
			case 'j' : 
			case 'm' :	// new data - from stm-file
			case 's' :
				if( (_count_mode == NORMAL_MODE)
					|| (_count_mode == GENERALIZED_MODE) ) {
					if( massInFilePos != rdm::getstartpos() ) {
						massInFilePos = rdm::getstartpos();
						if (prefix=='i' || prefix=='j')
							fseek( mass_in_file,massInFilePos,SEEK_SET );
						else
							fseek( stm_file,massInFilePos,SEEK_SET );
						runPlayBackMode( mass_in_file,
									_count_mode );
					}
				}
				break;
			case 'O' : if(isFirstMode) {
					star.init(0);
					if( testNrError() ) {
						_abort = true;
						break;
					}
					isFirstMode = false;
				}
				changeStar(); break;
			default  : break;
		}
		if( testNrError() ) {
			_abort = true;
			break;
		}
	}
	trb_tm::setStartTime( oldProgramStartTime );
	_sciCountSave = oldSciCountSave;
//	outcontrol.setGraphReady( true,true ); // risovat' graph

	// get back current file structure pointers:
	srv_file = old_srv_file;
	if ( stm_file ) fclose( stm_file );
	stm_file = old_stm_file;
	if ( counts_file ) fclose( counts_file );
	counts_file = old_counts_file;
	fclose( mass_in_file );
	mass_in_file = NULL;
	fclose( out_file );
	out_file = old_out_file;

	nMassLines = rdm::grabparams( out_file );
	jdebug( "grabParam = %d lines",nMassLines );  // clear params!!!
	restoreCFG();
	tree->setChanged( isTreeChanged );
	star.init(0);
	star.setCurrentStar( currentStar );
	testNrError();
	trb_tm::enablePlayBack( false );
	_isPlayBack = false;
	if( testUserStop() || _abort ) endPlayBack();
	else {
		jdebug( "Play Back done" );
		sendToScreen( "Play Back done" );
		showMessage( "Press \"End Play Back\"", -1 );
	}
	trberr::enableDialog( true );
}

void DevManager::endPlayBack( void ) {
	jdebug( "endPlayBack!" );
	terminateCountMode(1);
	sum::done();
	long ldate = trb_tm::longDate(trb_tm::programStartDate());
	sum::init( _sumFileName.data(), ldate,
			MAX_N_COUNTERS, _dumpFileName.data() );
	outcontrol.resetSummaryData();
	outcontrol.setGraphReady( true,true ); // risovat' graph
	testNrError();
}

// -----------------  End of Play Back

void DevManager::terminateCountMode( int success ) {
	endCounting();
}

void DevManager::exchange_test_mode( void ) {
	trberr::enableDialog( false );
	int success = runExchangeTest();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::detector_test_mode( void ) {
	trberr::enableDialog( false );
	int success = runDetectorTest();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::statistic_test_mode( void ) {
	trberr::enableDialog( false );
	int success = runStatisticTest();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::detector_counting_mode( void ) {
	trberr::enableDialog( false );
	int success = runDetectorCountingMeasurement();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::detector_statistic_mode( void ) {
	trberr::enableDialog( false );
	int success = runDetectorStatisticMeasurement();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::background_corr_mode( void ) {
	trberr::enableDialog( false );
	int success = runBackgroundCorrection();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::background_measur_mode( void ) {
	trberr::enableDialog( false );
	int success = runBackgroundMeasurement();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::centering_corr_mode( void ) {
	trberr::enableDialog( false );
	int success = runCenterCorrection();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::track_calibration_mode( void ) {
	trberr::enableDialog( false );
	int success = runTrackCalibration();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::centering_mode( void ) {
	trberr::enableDialog( false );
	int success = runCentering();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::normal_mode( void ) {
	trberr::enableDialog( false );
	int success = runNormal();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::flux_estimation_mode( void ) {
	trberr::enableDialog( false );
	int success = runFluxEstimation();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::generalized_mode( void ) {
	trberr::enableDialog( false );
	int success = runGeneralized();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::focus_control_mode( void ) {
	trberr::enableDialog( false );
	int success = runFocusControl();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

void DevManager::center_control_mode( void ) {
	trberr::enableDialog( false );
	int success = runCenterControl();
	terminateCountMode( success );
	trberr::enableDialog( true );
}

double DevManager::getAccumTime( char mode ) {
	switch( mode ) {
		case NORMAL_MODE            : return _normalAccumTime;
		case GENERALIZED_MODE       : return _generalAccumTime;
		case FOCUS_CONTROL_MODE     : return _focusAccumTime;
		case CENTERING_MODE         :
		case CENTER_CONTROL_MODE    :
		case CENTERING_CORR_MODE    : return _centerAccumTime;
		case TRACK_CALIBRATION_MODE : return _centerAccumTime;
//						return 2*_centerAccumTime
//							+4*_gapRA
//							+4*_gapDec;
		case BACKGROUND_MEASUR_MODE : return _backgroundTime*2;
		case BACKGROUND_CORR_MODE   : return _backgroundTime*4;

		case EXCHANGE_TEST_MODE     : return _exchangeTestTime;
		case DETECTOR_TEST_MODE     : return _detectorTestTime
							+SHORT_LIGHT_WAIT_INTERVAL/S2MS;
		case STATISTIC_TEST_MODE    : return _normalAccumTime
							+LIGHT_WAIT_INTERVAL/S2MS*
							_nStatisticTestLight;
		case DETECTOR_COUNTING_MODE	: 
				return( _detectorCountingTime*
					_nDetectorCountingDiscrimination*
					_nDetectorCountingLight*
					_nDetectorCountingVoltage
					+LIGHT_WAIT_INTERVAL/S2MS*
					_nDetectorCountingLight
					+HV_WAIT_INTERVAL/S2MS*
					_nDetectorCountingVoltage );
		case DETECTOR_STATISTIC_MODE	:
				return ( _detectorStatisticTime*
					_nDetectorStatisticLight
					+LIGHT_WAIT_INTERVAL/S2MS*
					_nDetectorStatisticLight );
		case FLUX_ESTIMATION_MODE	: return _estimationAccumTime;

		case QUIT_PROGRAM		: return 0;
		default : jdebug( "+++!!!!!!!! Invalid _count_mode = %d",_count_mode ); return(0.0);
	}
}

int DevManager::calcTimeoutInterval( double expos, int number ) {
	int interval = (int) ceil( expos * (double)number
		* TM_PREC + (double)_dataBlockSize * expos )
// debug:
		+5000
		;
	if( srv_file ) fprintf( srv_file,"Timeout Interval=%d\n",interval );
	return( interval );
}

/**
	@brief	Convert an argument into its character hexadecimal 
	representation with leading '@'
	@param	addr	Argument (normally, binary file position)
	@return C-string with '@'+hex(addr)
*/
const char * getAddrStr( long addr ) {
	// maximal long number in hex is 7fffffff
	const int maxAddrLength = 8;
	static char strAddr[1+maxAddrLength+1] = "@7fffffff" ;// foo example
	if (addr>=0) {
		sprintf(strAddr,"@%lx",addr);
		return strAddr;
	} else {
		// if addr is negative (invalid) - return empty string
		return "";
	}
}

void DevManager::setStartBaseTime( void ) {
	_startBaseTime = trb_tm::getCurrentUT();
	if( srv_file ) fprintf( srv_file,"StartTime: %s\n",
		_startBaseTime.toString().data() );
}

Time DevManager::setMidTime( Time startTime, Time stopTime ) {
	int secInterval = startTime.secsTo( stopTime );
	if( secInterval < 0 ) {	// if cherez polnoch'
		secInterval += int(SEC_IN_DAY);
	}
	Time midTime = startTime;
	midTime.addSecs(secInterval / 2 );
	if( srv_file ) fprintf( srv_file,
	"StartAccum: %s StopAccum: %s MidAccum: %s\n",
		startTime.toString().data(),
		stopTime.toString().data(),
		midTime.toString().data() );
	return( midTime );
}

int DevManager::testUserStop( void ) {

	userStopMutex.lock();
	int stop = isUserStop;	
	userStopMutex.unlock();

	if( stop ) {
		sendErrorInfo( userStop );
		return 1;
	} else return 0;
}

void DevManager::controlUserTrack( void ) {
	if( !_withAutoGuide ) return;
	if( StateExport::isUserTrack() ) {
		int coord = StateExport::trackCoordinate();
		int direct = StateExport::trackDirection();
		track( coord, direct );
// wait User's stopTracking:
//		debug( "wait...." );
		while( StateExport::isUserTrack() ) {
			exch::wait(USER_TRACK_STEP);
		}
//		debug( "Stop!" );
		stopTrack();
	}
}

void clearTable( void ) {
	infoExportMutex.lock();
	table.clear = true;
	table.isChanged = true;
	infoExportMutex.unlock();
}

void sendFluxHead( bool clear,
			int nHeadRow,
			const char* headRow1,
			const char* headRow2="",
			const char* headRow3="",
			const char* headRow4="",
			const char* headRow5="" ) {
	if( clear ) jdebug( "clear!" );
	else jdebug( "Not clear!" );

	infoExportMutex.lock();
	table.setHHead( 0,"", nameA, nameB, nameC, nameD );

	table.clearRow( 1 ); table.setText( 1,0,headRow1 );
	if( nHeadRow > 1 ) {
		table.clearRow( 2 );
		table.setText( 2,0,headRow2 );
	}
	if( nHeadRow > 2 ) {
		table.clearRow( 3 );
		table.setText( 3,0,headRow3 );
	}
	if( nHeadRow > 3 ) {
		table.clearRow( 4 );
		table.setText( 4,0,headRow4 );
	}
	if( nHeadRow > 4 ) {
		table.clearRow( 5 );
		table.setText( 5,0,headRow5 );
	}
	if( clear ) table.clear = clear;
//	int nCols = 5;
	int nCols = 9;
	int nRows = nHeadRow+1;
	if( table.isChanged ) {
// Staroe yeche ne narisovano...
		if( table.nCols < nCols) table.nCols = nCols;
		if( table.nRows < nRows) table.nRows = nRows;
	} else {
		table.nRows = nRows;
		table.nCols = nCols;
	}
	table.isChanged = true;
	table.startRow = 0;
	table.startCol = 0;
	infoExportMutex.unlock();
}

void fluxToTable( int nRows, int chan,
				double val1, 
				double val2 = 0,
				double val3 = 0,
				double val4 = 0,
				double val5 = 0 ) {
	infoExportMutex.lock();

	table.setNum( 1,chan+1, val1,fluxFormat );
	if( nRows > 1 ) table.setNum( 2,chan+1, val2,errFormat );
	if( nRows > 2 ) table.setNum( 3,chan+1, val3,poisFormat );
	if( nRows > 3 ) table.setNum( 4,chan+1, val4,poisFormat );
	if( nRows > 4 ) table.setNum( 5,chan+1, val5,poisFormat );

	infoExportMutex.unlock();
}

void sendTable( int startRow, int startCol,
				int nRows, int nCols,
				bool clear=false,bool adjust=false ) {
	infoExportMutex.lock();
	if( table.isChanged ) {
// Staroe yeche ne narisivano...
		if( table.startRow > startRow) table.startRow=startRow;
		if( table.startCol > startCol) table.startCol=startCol;
		if( table.nRows < nRows) table.nRows =
				nRows+abs(startRow-table.startRow);
		if( table.nCols < nCols) table.nCols =
				nCols+abs(startCol-table.startCol);
	} else {
		table.startRow=startRow;
		table.startCol=startCol;
		table.nRows = nRows;
		table.nCols = nCols;
	}
	table.isChanged = true;
	if( clear ) table.clear = true;
	if( adjust ) table.adjust = true;
	infoExportMutex.unlock();
}

void fluxMeanToTable( bool isgen=false, bool isStatTest=false ) {
	int currentMeasNumber = sc::getcurmeas()-1;
	if( isStatTest ) currentMeasNumber--;
	currentMeasNumber = currentMeasNumber >= 0 ?
					currentMeasNumber : 0; 
	if( isgen ) {
		int nonShiftedMeasNumber =
			sc::getcurmeas()-2 >= 0 ? 
					sc::getcurmeas()-2 : 0;
		for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			double nonshiftedFlux =
				sc::getmean(true,iCounter,nonShiftedMeasNumber);
			double currentFlux =
				sc::getmean(true,iCounter,currentMeasNumber);
			double ratio = nonshiftedFlux>0 ?
				currentFlux/nonshiftedFlux : 0; 
			fluxToTable( 2,
				iCounter, currentFlux, ratio);
		}
		sendTable( 1,1,2,4 );
	} else {
		for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			fluxToTable( 1,
				iCounter,
				sc::getmean(true,iCounter,currentMeasNumber) 
			);
		}
		sendTable( 1,1,1,4 );
	}
}

void avgErrFluxToTable( void ) {
	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		fluxToTable( 2,
			iCounter,
			sc::getavgflux(0,iCounter),
			sc::geterrflux(0,iCounter) );
	}
	sendTable( 1,1,2,4);
}

void avgErrPoisFluxToTable( double* nonPois ) {
	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		fluxToTable( 3,
			iCounter,
			sc::getavgflux(0,iCounter),
			sc::geterrflux(0,iCounter),
			nonPois[iCounter]
			);
	}
	sendTable( 1,1,3,4 );
}

void DevManager::scenario_mode ( void ) {
	trberr::enableDialog( false );
	int success=0;

	int oldIllum = light.isIllumOn();
	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		goto ABORT_SCENARIO;
	}

// Calculation total scenario time:

	{double sec = 0;
	for( int i=_scenarioStartPosition; i<(int)(currentScenario.size()); i++ ) {
		sec += getAccumTime( currentScenario[i] );
	}

	isScenario = true;
//	clearTable();
	sendToScreen( emptyLine );
	for( int i=_scenarioStartPosition; i<(int)(currentScenario.size()); i++ ) {
		sec -= getAccumTime( currentScenario[i] );
		if( i == _scenarioStartPosition ) {
			openScenario( getScenarioName(_currentScenarioNumber),sec );
		} else	setRemainTimeScenario( sec ); // do konza mody
// setMode dobavit siuda vremia do konza tekuchei mody

//		debug( "i=%d scenario=%d",i,currentScenario[i] );
		if( _stopScenario ) {
			_scenarioStartPosition = 0;
			break;
		}
		if( _pauseScenario ) {
			_scenarioStartPosition = i;
			break;
		}
		success = 0;
		_count_mode = currentScenario[i];
		if( i>0 ) _old_count_mode = currentScenario[i-1];
		else _old_count_mode = currentScenario[i];
		switch( _count_mode ) {
			case NORMAL_MODE            : success=runNormal(); break;
			case GENERALIZED_MODE       : success=runGeneralized(); break;
			case FOCUS_CONTROL_MODE     : success=runFocusControl(); break;
			case CENTER_CONTROL_MODE    : success=runCenterControl(); break;
			case BACKGROUND_MEASUR_MODE : success=runBackgroundMeasurement(); break;
			case EXCHANGE_TEST_MODE		: success=runExchangeTest(); break;
			case DETECTOR_TEST_MODE		: success=runDetectorTest(); break;
			case STATISTIC_TEST_MODE	: success=runStatisticTest(); break;
			case DETECTOR_COUNTING_MODE	: success=runDetectorCountingMeasurement(); break;
			case DETECTOR_STATISTIC_MODE	: success=runDetectorStatisticMeasurement(); break;
			case FLUX_ESTIMATION_MODE	: success=runFluxEstimation(); break;
			case QUIT_PROGRAM			: quitProgram(); break;
			default : jdebug( "Scenario:+++!!!!!!!! Invalid _count_mode = %d",
				currentScenario[i] ); break;
		}
		_old_count_mode = _count_mode;

		if( _stopOnError && (success != 1) ) {
//			debug( "Break Scenario! _stopOnError=%d success=%d",
//				_stopOnError,success );
			break;
		}
	}
	}
ABORT_SCENARIO:
	isScenario = false;

	terminateCountMode( success );
	if( !_pauseScenario ) closeScenario();
	if( isFirst ) {
		turnDoor( _inputDoor,0,false );	// startovoe polozhenie
		isFirst = false;
	}
	turnIllum(oldIllum,0);
	trberr::enableDialog( true );
}

int DevManager::runNormal( FILE* mass_in_file=NULL ) {
	int *p_n_run = &n_run;
	int _abort = 0;
	Time startAccumTime;
	long address = rdm::NOADDR;

	int oldIllum = light.isIllumOn();

	if( !_isPlayBack ) {
		if( !isScenario ) sendToScreen( emptyLine );
		openMode( normalName,getAccumTime( _count_mode ) );
		disablePolling();
	}

	jdebug( "_________ NORMAL_MODE:__________" );

	n_run = (int)ceil( _normalAccumTime / _normalBaseTime );
	exch::setNcounters( MAX_N_COUNTERS );

	int expositionNumber;
	int timeoutInterval=4000;

	if( !_isPlayBack ) {
		sendFluxHead( !( isScenario || (_old_count_mode == _count_mode) ||
			(_old_count_mode == GENERALIZED_MODE) ||
			(_old_count_mode == NORMAL_MODE) ),
			1,
			"CFLUX" );
		jdebug( "countMode = %d oldMode = %d",
			_count_mode,_old_count_mode );
	}

	star.updateCurrent() ;
	fprintf( out_file,"%s %s %s\n",
		getStamp("M ",trb_tm::getCurrentUT() ), 
		normalName.data(), star.getAirMass().data() );

	if( setBlockNumber( _normalBlockNumber,
			expositionNumber,
			0,_isPlayBack ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_NORM;
	}
	if( !star.isStarSelected() ) {
		sendErrorInfo( noStar );
		_abort = 1;
		goto ABORT_NORM;
	}
	if( !_isPlayBack ) {
		if( turnIllum(OFF,0) != 1 ) {
			sendErrorInfo( errIllum );
			_abort=1;
			goto ABORT_NORM;
		}
		if( setExpos( _normalExposition,0 ) != 1 ) {
			sendErrorInfo( errCounter );
			_abort = 1;
			goto ABORT_NORM;
		}

	}

	/* Initialize SCIND module: */
	sc::init( expositionNumber, n_run, MAX_N_COUNTERS );
#ifdef WRITESI
	sc::write( out_file,sc::DSI, "#i   UT   ", "Address", true );
#endif

	/* Update the ATMOS module: */

	{
//  WF z-grid parameters:
	double _minAltitude =	leafDataToDouble( deviceCfgFile,"WeightFunction","MinAltitude",out_file );
	double _maxAltitude =	leafDataToDouble( deviceCfgFile,"WeightFunction","MaxAltitude",out_file );
	double _gridStep =	leafDataToDouble( deviceCfgFile,"WeightFunction","Step",out_file );
	double _minGridStep =	leafDataToDouble( deviceCfgFile,"WeightFunction","MinStep",out_file );

	atm::update( star.getWFName().data(), _checkFile.data(), 
			_zShiftNormal, _zShiftGeneralSecond,
			_minAltitude, _maxAltitude, _gridStep,_minGridStep  );
	}
//	atm::alloc(n_run) ;
	if( (_abort=testNrError()) ) goto ABORT_NORM;

	/* Update the parameters of observed object: */ 


	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}
	if( !_isPlayBack ) {
		if( setAperture( _normalApert,0 ) != 1 ) {
			sendErrorInfo( errAperture );
			_abort = 1;
			goto ABORT_NORM;
		}
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );

	for( int jj=0; jj<(*p_n_run); jj++ ) {
		if( !_isPlayBack ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendErrorInfo( errCounter );
				_abort = 1;
				goto ABORT_NORM;
			}
		} else {
			address = getCountAddress( mass_in_file,
					false, // isgen = false
					_startBaseTime );

			if( address == rdm::NOADDR && !stm_file ) {
				sendErrorInfo( "Too few count data!" );
				 _abort = 1;
				goto ABORT_NORM;
			}
		}
		if( jj == 0 ) startAccumTime = _startBaseTime;

		if( !_isPlayBack ) {
			exch::waitData();

			stopDevTimer();
			outCountResult();
			catchAppendix();
		}

		if( _isPlayBack || _sciCountSave ) {
			cleanData( expositionNumber, &address,
						_isPlayBack );
		} else {
			cleanData( expositionNumber );
		}

		if( !(_isPlayBack && stm_file) )
			// compute count stat.moments:
			sc::calcmom( expositionNumber, _nCorrected );
		if( !_isPlayBack) {
			// write stat.moments in a file (BEFORE sc::calcidx!!):
			sc::write( stm_file,sc::MOM, 
			getStamp("m ",_startBaseTime),
			getAddrStr(address) );		
		}

		if( (_abort=testNrError()) ) goto ABORT_NORM;

		sc::calcidx( _normalExposition, sc::MODENORM );

		if (nr::erget() == nr::ERNOD) {
			// Bad index encountered: skipped measurement
			nr::erreset();
		} else {
			if( _isPlayBack ) address = rdm::NOADDR;
#ifdef WRITESI
			sc::write( out_file,sc::DSI,
			getStamp("i ",_startBaseTime), getAddrStr(address) );
#endif
			if( !_isPlayBack ) fluxMeanToTable();

	/* Compute Cn2-integrals: */
			int meas = sc::getcurmeas() -1 ;
			atm::calcint(
				sc::getidxptr(meas,sc::DSI),	// non-shifted (D)SI
				0 );	// shifted (D)SI - absent 
			if( (_abort=atBaseTime())  ) goto ABORT_NORM;

			if( !_isPlayBack )
				setRemainTimeMode( ((*p_n_run) - jj)/_normalBaseTime );
		}
	} // jj-cycle
ABORT_NORM:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {
		Time midAccumTime = setMidTime( startAccumTime,
					_startBaseTime ); // first & last Base Time
		if( accumResume( midAccumTime,startAccumTime,_startBaseTime,
				false,
				_normalFixedMethod,
				_isPlayBack ) != 1 ) {
			_abort = 1;
		} else {
			sendToScreen( emptyLine );
		}

		if( testNrError() ) _abort = 1;
	}
	if( !_isPlayBack ) {
		closeMode();
		enablePolling();
		if( isOverLight ) { handleOverLight(); _abort = 1; }
		turnIllum(oldIllum,0);
	}
	if( _abort ) return(0);
	else return(1);
}

void DevManager::handleOverLight( void ) {
	sendErrorInfo( overLightText );
	isOverLight = false;
}

bool DevManager::cleanData( int& expositionNumber,
			long *addr = 0,
			bool isPlayBack=false ) {

	// Detect data loss anywhere:
	bool dataLost = false ;

	// Detect data loss in counters individually:
	bool counterDataLost[MAX_N_COUNTERS];

	count_t * countDataPtr[MAX_N_COUNTERS] ;
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		if( isPlayBack ) {
			counterDataLost[i] = false;
		} else {
			counterDataLost[i] = (counters[i]).expandData();
			if( counterDataLost[i] ) dataLost = true;
		}
		countDataPtr[i] = (counters[i]).getData() ;
	}
	if( addr ) {
		if( !isPlayBack ) {
			if( counts_file )
				// cleanData with ("write count!") and count file writing is ON
				*addr = ioc::write(counts_file, countDataPtr,
					expositionNumber, MAX_N_COUNTERS) ;
			else
				*addr = rdm::NOADDR ; // Count-file is not opened: return invalid address	
		} else {
			int n_counters = MAX_N_COUNTERS;
			int exp_numb = expositionNumber;
			if( !stm_file ) {
				// true playback for old data - by counts:
				ioc::read( counts_file,countDataPtr,
					&exp_numb,&n_counters,*addr );
				for( int i=0; i<MAX_N_COUNTERS; i++ ) {
					if( !counters[i].setOutNumber(expositionNumber) ) {
						trberr::setCode(TRB_ABORT);
						trberr::setMessage( "Invalid count data" );
						return true;
					}
				}
			} else {
				// new data: playback by moments:
				sc::str2mom(rdm::getvalue().c_str());
			}
			if( n_counters != MAX_N_COUNTERS ) {
				trberr::setCode(TRB_ABORT);
				trberr::setMessage( "Count file corrupted" );
				return true;
			}
			if( expositionNumber != exp_numb ) {
				expositionNumber = exp_numb;
				dataLost = true;
			}
		}
	}

	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		if( isPlayBack || counterDataLost[i] ) {
			_nCorrected[i] = (counters[i]).repairData();
		} else {
			_nCorrected[i] = 0;
		}

		if( _nCorrected[i] ) dataLost = true ;

		if( srv_file ) {
			fprintf( srv_file,"%s: _nCorrected=%d\n",(counters[i]).getName(),
				_nCorrected[i] );
		}
	}
	if( srv_file ) fprintf( srv_file,"\n" );

	return dataLost;
}

int DevManager::runGeneralized( FILE* mass_in_file=NULL ) {
	int *p_n_run = &n_run;
	Time startAccumTime;
	long address = rdm::NOADDR;
	int timeoutInterval = 4000;

	int _abort = 0;
	int _apert = -1;
	int _blockNumber = -1;
	int expositionNumber;
	double _elapsSec = 0.0;

	int oldIllum = light.isIllumOn();

	if( !_isPlayBack ) {
		if( !isScenario ) sendToScreen( emptyLine );
		openMode( generalName,getAccumTime( _count_mode ) );
		disablePolling();
	}

	jdebug( "_________ GENERALIZED_MODE:__________" );
/*
	n_run = 1 + 2 * (int)ceil( _generalAccumTime / 
			( _generalBaseTime + _generalSecondBaseTime ) );
*/	

	n_run = 2 * (int)ceil( _generalAccumTime / 
			( _generalBaseTime + _generalSecondBaseTime ) );

	exch::setNcounters( MAX_N_COUNTERS );
	
	if( !_isPlayBack ) {
		sendFluxHead( !( isScenario || (_old_count_mode == _count_mode) ||
			(_old_count_mode == NORMAL_MODE) ||
			(_old_count_mode == GENERALIZED_MODE) ),
			2,
			"CFLUX",
			"FG/FN" );
	}

	star.updateCurrent() ;
	fprintf( out_file,"%s %s %s\n",
		getStamp( "M ",trb_tm::getCurrentUT() ), 
		generalName.data(),
		star.getAirMass().data() );
	
	if( !star.isStarSelected() ) {
		sendErrorInfo( noStar );
		_abort = 1;
		goto ABORT_GEN;
	}
	if( !_isPlayBack ) {
		if( turnIllum(OFF,0) != 1 ) {
			sendErrorInfo( errIllum );
			_abort=1;
			goto ABORT_GEN;
		}
		if( setExpos( _generalExposition,0 ) != 1 ) {
			sendErrorInfo( errCounter );
			_abort = 1;
			goto ABORT_GEN;
		}
	}
	{	// beginning of block to isolate goto-s
	/* Initialize SCIND module: */
	int maxncount = calcExpositionNumber (
		(_generalBlockNumber > _generalSecondBlockNumber) ?
		_generalBlockNumber : _generalSecondBlockNumber );
		
	sc::init( maxncount, n_run, MAX_N_COUNTERS );
#ifdef WRITESI
	sc::write( out_file,sc::DSI, "#i   UT   ", "Address", true );
#endif
	/* Update the ATMOS module: */
	{
	double _minAltitude =	leafDataToDouble( deviceCfgFile,"WeightFunction","MinAltitude",out_file );
	double _maxAltitude =	leafDataToDouble( deviceCfgFile,"WeightFunction","MaxAltitude",out_file );
	double _gridStep =	leafDataToDouble( deviceCfgFile,"WeightFunction","Step",out_file );
	double _minGridStep =	leafDataToDouble( deviceCfgFile,"WeightFunction","MinStep",out_file );

	atm::update( star.getWFName().data(), _checkFile.data(), 
			_zShiftGeneral, _zShiftGeneralSecond,
			_minAltitude, _maxAltitude, _gridStep,_minGridStep  );
	}

	if( (_abort=testNrError()) ) goto ABORT_GEN;
	/* Update the parameters of observed object: */ 

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}

	int _regime = 1;
	// declare here measurement numbers for shifted and non-shifted modes:
	int measnorm = -1;	// "-1" is no meas. for a while
	int measgen = -1;
	for( int jj=0; jj<(*p_n_run); jj++ ) {
		bool isgen = (_regime == -1) ;
		switch( _regime ) {
			case 1	:	_apert = _generalApert;
						_blockNumber = _generalBlockNumber;
						_elapsSec += _generalBaseTime;
						break;
			case -1	:	_apert = _generalSecondApert;
						_blockNumber = _generalSecondBlockNumber;
						_elapsSec += _generalSecondBaseTime;
						break;
		}
		if( !_isPlayBack ) {
			if( setAperture( _apert,0 ) != 1 ) {
				sendErrorInfo( errAperture );
				_abort = 1;
				goto ABORT_GEN;
			}
		}
		if( setBlockNumber( _blockNumber,expositionNumber,
						0,_isPlayBack ) != 1 ) {
			sendErrorInfo( errCounter );
			_abort = 1;
			goto ABORT_GEN;
		}
		timeoutInterval = calcTimeoutInterval( _generalExposition, expositionNumber );
		if( !_isPlayBack ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendErrorInfo( errCounter );
				_abort = 1;
				goto ABORT_GEN;
			}
		} else {
			address = getCountAddress( mass_in_file,
					isgen, _startBaseTime );
			if( address == rdm::NOADDR ) {
				sendErrorInfo( "Too few count data!" );
				_abort = 1;
				goto ABORT_GEN;
			}
		}
		if( (_abort=testNrError()) ) goto ABORT_GEN;

		if( jj == 0 ) startAccumTime = _startBaseTime;

		if( !_isPlayBack ) {
			exch::waitData();

			stopDevTimer();
			outCountResult();
			catchAppendix();
		}

		if( _isPlayBack || _sciCountSave ) {
			cleanData( expositionNumber, &address,
					_isPlayBack );
		} else {
			cleanData( expositionNumber );
		}

		if( (_abort=testNrError()) ) goto ABORT_GEN;

		if( !(_isPlayBack && stm_file) )
			// compute count stat.moments:
			sc::calcmom( expositionNumber, _nCorrected );
		if( !_isPlayBack) {
			// write stat.moments in a file (BEFORE sc::calcidx!!):
			const char * prefix = isgen ? "s " : "m " ;
			sc::write( stm_file,sc::MOM,
			getStamp(prefix,_startBaseTime), 
			getAddrStr(address) );		
		}

		if( (_abort=testNrError()) ) goto ABORT_GEN;

		sc::calcidx( _normalExposition, isgen );

		if (nr::erget() == nr::ERNOD) {
			// Bad index encountered: skipped measurement
			nr::erreset();
		} else {
			if (!isgen) 
				measnorm = sc::getcurmeas();
			else
				measgen = sc::getcurmeas();
#ifdef WRITESI
			const char * prefix = isgen ? "j " : "i " ;
			if( _isPlayBack ) address = rdm::NOADDR;
			sc::write( out_file,sc::DSI,
				getStamp( prefix, _startBaseTime ),
						getAddrStr(address) );
#endif
			if( !_isPlayBack ) {
				fluxMeanToTable(isgen);
			}

			/* Compute Cn2-integrals using also last good
				normal-mode index measurement "measnorm": */
			if (isgen && (measnorm>-1))
				atm::calcint(
				sc::getidxptr(measnorm-1,sc::DSI),	// non-shifted (D)SI
				sc::getidxptr(measgen-1,sc::DSI) );	// shifted (D)SI
		}

		if( (_abort=atBaseTime())  ) goto ABORT_GEN;

		if( !_isPlayBack )
			setRemainTimeMode( _generalAccumTime - _elapsSec );

		_regime *= (-1);
	}	// jj-cycle
	
	}	// end of block to isolate goto-s
ABORT_GEN:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {

		jdebug( "Generalized: write idx" );

		Time midAccumTime = setMidTime( startAccumTime,
					_startBaseTime ); // first & last Base Time
		if( accumResume( midAccumTime,startAccumTime,_startBaseTime,
				true,_generalFixedMethod,
					_isPlayBack ) != 1 ) {
			_abort = 1;
		} else {
			sendToScreen( emptyLine );
		}
		if( testNrError() ) _abort = 1;
	}
	if( !_isPlayBack ) {
		setAperture( _generalApert,0 );
		closeMode();
		enablePolling();
		if( isOverLight ) { handleOverLight(); _abort = 1; }
		turnIllum(oldIllum,0);
	}
	if( _abort ) return(0);
	else return(1);
}

int DevManager::accumResume( Time midAccumTime,
			Time startAccumTime, Time endAccumTime,
			bool isgen,
			bool isFixedLayersMethod,
			bool isPlayBack=false ) {

	int tableRow = 0;
	int success = makeAccumResultScind( midAccumTime, 
			startAccumTime, endAccumTime, isgen );

	if( !isPlayBack ) {
		infoExportMutex.lock();
		for( int i=0; i<MAX_ROW; i++ ) {
			table.clearRow( i );
		}

		tableRow = showAccumResultScind( isgen );

		if( !success ) {
			table.isChanged = true;
			table.startRow = 0;
			table.nRows = MAX_ROW;
			table.startCol = 0;
			table.nCols = MAX_COL;
		}

		infoExportMutex.unlock();
	}
	if( success ) { //return(0);

		if( makeAccumResultAtmos( midAccumTime,
			isgen, isFixedLayersMethod ) == 1 ) {

			if( !isPlayBack ) {
				infoExportMutex.lock();
				showAccumResultAtmos( tableRow,isgen );
				infoExportMutex.unlock();
			}

		} else success=0;

		if( !isPlayBack ) {
			infoExportMutex.lock();
			table.isChanged = true;
			table.startRow = 0;
			table.nRows = MAX_ROW;
			table.startCol = 0;
			table.nCols = MAX_COL;
			infoExportMutex.unlock();
		}
	}

//	if( !isPlayBack ) {
	double ut_double = midAccumTime.hour() +
		midAccumTime.minute() / 60.0 +
		midAccumTime.second() / 3600.0;

	outcontrol.addSummaryData( ut_double,
			sum::massmag( star.getMagVval(), star.getMagBVval() ),
			star.getAirMassVal(),
			true,
//			!isPlayBack, // if playBack => ne risovat' (not ready)
			isgen );
//	}

	return(success);
}

int DevManager::makeAccumResultScind( Time midAccumTime, 
			Time startAccumTime, Time endAccumTime,
			bool isgen ) {
	sc::average( isgen, sc::MAXLAG );
	double *scind = sc::getavgidxptr( sc::DSI,sc::MODENORM );

	sc::writeavg( out_file,sc::DSI,sc::MODENORM,
		"#I   UTbeg"," UTend   ",true );
	// form "I <start-time> <end-time>" prefix:
	QString tBeg = getStamp("I ",startAccumTime );
	QString tEnd = getStamp("",endAccumTime );
	sc::writeavg( out_file,sc::DSI,sc::MODENORM, tBeg.data(),tEnd.data() );

	if( isgen ) {
		sc::writeavg( out_file,sc::DSI,sc::MODEGEN,
			"#J   UTbeg"," UTend   ",true );
		tBeg[0] = 'J';
		sc::writeavg( out_file,sc::DSI,sc::MODEGEN, tBeg.data(),
			tEnd.data() );
	}
	sc::writeavg( out_file,sc::AFLUX,sc::MODENORM,
			"#F   UT   ", 0,true );
	sc::writeavg( out_file,sc::AFLUX,sc::MODENORM,
			getStamp("F ",midAccumTime) );
	if (isgen) {
		sc::writeavg( out_file,sc::AFLUX,sc::MODEGEN,
			"#G   UT   ", 0,true );
		sc::writeavg( out_file,sc::AFLUX,sc::MODEGEN,
			getStamp("G ",midAccumTime) );
	}
	if( (scind[0] <= 0) || (scind[1] <= 0)
		|| (scind[2] <= 0) || (scind[3] <= 0) ) {
		sendErrorInfo( badIndex );
		return(0);
	}
	return(1);
}

int DevManager::makeAccumResultAtmos( Time midAccumTime,
		bool isgen, bool isFixedLayersMethod ) {
	double *scind = sc::getavgidxptr( sc::DSI,sc::MODENORM );
	double *scindShft = isgen ? sc::getavgidxptr( sc::DSI,sc::MODEGEN ) : 0 ;
/* Average Cn2-integrals and compute atm. parameters */
	const int desichan = 0 ; 	/* DESI - in A-channel */
	double desi = sc::getavgidx(sc::DESI,isgen?sc::MODEGEN:sc::MODENORM, desichan) ;
	double edesi = sc::geterridx(sc::DESI,isgen?sc::MODEGEN:sc::MODENORM, desichan) ;
	atm::avgint(desi, edesi, star.getAirMassVal()) ;

/* write header and data for integrals: */
	atm::write( out_file, "#A   UT   ",
		atm::ATMPAR, atm::HEAD );
	atm::write( out_file, getStamp( "A ",midAccumTime ),
		atm::ATMPAR );

/* Restore the Cn2-profile */
	double * e2scind = sc::geter2idxptr( sc::DSI,sc::MODENORM ) ;
	double * e2scindShft = isgen ? 
		sc::geter2idxptr( sc::DSI,sc::MODEGEN ) : 0 ;
//	bool IsFixedLayersMethod = (method=='X') ;

// the returned calcn2() value is a :"seeing by Cn2 profile". How to display it ???
/* write the header and data for Cn2 profile: */
	atm::write( out_file, "#T   UT   ",
		atm::CN2PROF, atm::HEAD );
// First, compute&save with non-displayed method, then with displayed:
	for (int w=0; w<=1; w++) {
		bool which = isFixedLayersMethod ;
		if (!w) which = !which ;
		atm::calcn2( scind, scindShft, e2scind, e2scindShft, 
		star.getAirMassVal(),
		which ? atm::FIXEDLAY : atm::FLOATLAY) ;
		atm::write( out_file, getStamp( "T ",midAccumTime ),
		atm::CN2PROF );
/* write the normalized O-C for indices restored by Cn2 profile: */
		atm::write( out_file, getStamp( "R ",midAccumTime ),
		atm::O_CSCIND );
	}

	if( testNrError() ) return 0;
	else return 1;
}

int DevManager::showAccumResultScind( bool isgen,
				bool isStatTest=false ) {
	jdebug( "Show Accum Res Scind!!!" );
	int row=0;
	table.setHHead( row,"", nameA, nameB, nameC, nameD );
	row++;
	table.setText( row,0,"CFLUX" ); row++;
	int rowFgFn = 0;
	if( isgen ) {
		table.setText( row,0,"FG/FN" );
		rowFgFn = row;
		row++;
	}
	int rowFl = 0;
	if( outcontrol.showFlux() ) {
		table.setText( row,0,"AFLUX" );
		rowFl = row;
		row++;
	}
	int rowFlErr = 0;
	if( outcontrol.showFluxErr() ) {
		table.setText( row,0,"err" );
		rowFlErr = row;
		row++;
	}
	int rowFlg = 0;
	int rowFlErrG = 0;
	if(isgen) {
		if( outcontrol.showFluxG() ) {
			table.setText( row,0,"AFLUXg" );
			rowFlg = row;
			row++;
		}
		if( outcontrol.showFluxGerr() ) {
			table.setText( row,0,"err" );
			rowFlErrG = row;
			row++;
		}
	}
	int rowSI = 0;
	if( outcontrol.showSI() ) {
		table.setText( row,0,"SI" );
		rowSI = row;
		row++;
	}
	int rowSIerr = 0;
	if( outcontrol.showSIerr() ) {	
		table.setText( row,0,"err" );
		rowSIerr = row;
		row++;
	}
	int rowExpSI = 0;// for Stat.Test only
	int rowExpSIerr = 0;
	if( isStatTest ) {
		if( outcontrol.showSI() ) {
			table.setText( row,0,"ESI" );
			rowExpSI = row;
			row++;
		}
		if( outcontrol.showSIerr() ) {	
			table.setText( row,0,"err" );
			rowExpSIerr = row;
			row++;
		}
	}
	int rowSIg = 0;
	int rowSIerrG = 0;
	if (isgen) {
		if( outcontrol.showSIg() ) {	
			table.setText( row,0,"SIgen" );
			rowSIg = row;
			row++;
		}
		if( outcontrol.showSIgErr() ) {	
			table.setText( row,0,"err" );
			rowSIerrG = row;
			row++;
		}
	}

	/* we show only one DESI - shifted in generalized and non-shifted
		otherwise: */

	int rowDESI = 0;
	if( outcontrol.showDESI() ) {	
		if (isgen) {
			table.setText( row,0,"DESIg" );
		} else {
			table.setText( row,0,"DESI" );
		}
		rowDESI = row; row++;
	}

	int rowDESIerr = 0;
	if( outcontrol.showDESIerr() ) {	
		table.setText( row,0,"err" );
		rowDESIerr = row;
		row++;
	}

	int rowExpDESI = 0;
	int rowExpDESIerr = 0;
	if( isStatTest ) {
		if( outcontrol.showDESI() ) {	
			table.setText( row,0,"EDESI" );
			rowExpDESI = row; row++;
		}
		if( outcontrol.showDESIerr() ) {	
			table.setText( row,0,"err" );
			rowExpDESIerr = row;
			row++;
		}
	}

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		/* Both in Normal and Generalized mode we show "non-shifted"
			fluxes: */
		if( isgen ) {
			table.setNum( rowFgFn,iCounter+1,
				sc::getavgflux(sc::MODEGEN,iCounter)/
				sc::getavgflux(sc::MODENORM,iCounter),
				poisFormat );
		}
		if( outcontrol.showFlux() ) 
			table.setNum( rowFl,iCounter+1,
				sc::getavgflux( sc::MODENORM,iCounter ),fluxFormat );
		if( outcontrol.showFluxErr() ) 
			table.setNum( rowFlErr,iCounter+1,
				sc::geterrflux( sc::MODENORM,iCounter ),errFormat );
		if( outcontrol.showSI() ) 
			table.setNum( rowSI,iCounter+1,
				sc::getavgidx( sc::DSI,sc::MODENORM,iCounter ),SIFormat );
		if( outcontrol.showSIerr() ) 
			table.setNum( rowSIerr,iCounter+1,
				sc::geterridx( sc::DSI,sc::MODENORM,iCounter ),errFormat );
		if( isStatTest ) {
			if( outcontrol.showSI() ) 
				table.setNum( rowExpSI,iCounter+1,
					sc::getavgidx( sc::DSI,sc::MODEGEN,iCounter ),SIFormat );
			if( outcontrol.showSIerr() ) 
				table.setNum( rowExpSIerr,iCounter+1,
					sc::geterridx( sc::DSI,sc::MODEGEN,iCounter ),errFormat );
		}
		if( isgen ) {
			if( outcontrol.showFluxG() ) 
				table.setNum( rowFlg,iCounter+1,
					sc::getavgflux( sc::MODEGEN,iCounter ),fluxFormat );
			if( outcontrol.showFluxGerr() ) 
				table.setNum( rowFlErrG,iCounter+1,
					sc::geterrflux( sc::MODEGEN,iCounter ),errFormat );
			if( outcontrol.showSIg() ) 
				table.setNum( rowSIg,iCounter+1,
					sc::getavgidx( sc::DSI,sc::MODEGEN,iCounter ),SIFormat );
			if( outcontrol.showSIgErr() ) 
				table.setNum( rowSIerrG,iCounter+1,
					sc::geterridx( sc::DSI,sc::MODEGEN,iCounter ),errFormat );
			if( outcontrol.showDESI() ) 
				table.setNum( rowDESI,iCounter+1,
					sc::getavgidx( sc::DESI,sc::MODEGEN,iCounter ),SIFormat );
			if( outcontrol.showDESIerr() ) 
				table.setNum( rowDESIerr,iCounter+1,
					sc::geterridx( sc::DESI,sc::MODEGEN,iCounter ),errFormat );
		} else {
			if( outcontrol.showDESI() ) 
				table.setNum( rowDESI,iCounter+1,
					sc::getavgidx( sc::DESI,sc::MODENORM,iCounter ),SIFormat );
			if( outcontrol.showDESIerr() ) 
				table.setNum( rowDESIerr,iCounter+1,
					sc::geterridx( sc::DESI,sc::MODENORM,iCounter ),errFormat );
			if( isStatTest ) {
				if( outcontrol.showDESI() ) 
					table.setNum( rowExpDESI,iCounter+1,
						sc::getavgidx( sc::DESI,sc::MODEGEN,iCounter ),SIFormat );
				if( outcontrol.showDESIerr() ) 
					table.setNum( rowExpDESIerr,iCounter+1,
						sc::geterridx( sc::DESI,sc::MODEGEN,iCounter ),errFormat );
			}
		}
	}
/* Number of displayed DSI is equal to num.of channels */
//	const char* showDSI[MAX_N_COUNTERS] = {"AB","AD","BC","CD"};

	const char* dsiName[MAX_N_COUNTERS];
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		dsiName[i] = NULL;
	}
	if( outcontrol.showDSI() || outcontrol.showDSIg() ) {
		for( int i=0; ( (i<outcontrol.getNdsiName()) &&
				(i<MAX_N_COUNTERS) ); i++ ) {
			dsiName[i] = outcontrol.getDSIname()[i];
		}
		table.setHHead( row,"",dsiName[0],dsiName[1],
				dsiName[2],dsiName[3] );
		row++;
	}
	int rowDSI = 0;
	if( outcontrol.showDSI() ) {	
		table.setText( row,0,"DSI" );
		rowDSI = row;
		row++;
	}
	int rowDSIerr = 0;
	if( outcontrol.showDSIerr() ) {
		table.setText( row,0,"err" );
		rowDSIerr = row;
		row++;
	}
	int rowExpDSI = 0;
	int rowExpDSIerr = 0;
	if( isStatTest ) {
		if( outcontrol.showDSI() ) {	
			table.setText( row,0,"EDSI" );
			rowExpDSI = row;
			row++;
		}
		if( outcontrol.showDSIerr() ) {
			table.setText( row,0,"err" );
			rowExpDSIerr = row;
			row++;
		}
	}

	if( outcontrol.showDSI() ) {
		for( int iDSI=0; iDSI<MAX_N_COUNTERS; iDSI++ ) {
			if( dsiName[iDSI] ) {
				int idx = sc::ind_seqnum(MAX_N_COUNTERS, 
					dsiName[iDSI]);
				table.setNum( rowDSI,iDSI+1,
					sc::getavgidx( sc::DSI,sc::MODENORM,idx  ),SIFormat );

				if( outcontrol.showDSIerr() )
					table.setNum( rowDSIerr,iDSI+1,
						sc::geterridx( sc::DSI,sc::MODENORM,idx ),errFormat );
				if( isStatTest ) {
					table.setNum( rowExpDSI,iDSI+1,
						sc::getavgidx( sc::DSI,sc::MODEGEN,idx  ),SIFormat );
					if( outcontrol.showDSIerr() )
						table.setNum( rowExpDSIerr,iDSI+1,
							sc::geterridx( sc::DSI,sc::MODEGEN,idx ),errFormat );
				}
			}
		}
	}
	if( outcontrol.showDSIg() && isgen ) {
		table.setText( row,0,"DSIg" );
		int rowDSIg = row;
		row++;
		int rowDSIgerr = 0;
		if( outcontrol.showDSIgErr() ) {
			table.setText( row,0,"err" );
			rowDSIgerr = row;
			row++;
		}
		for( int iDSI=0; iDSI<MAX_N_COUNTERS; iDSI++ ) {
			if( dsiName[iDSI] ) {
				int idx = sc::ind_seqnum(MAX_N_COUNTERS, 
					dsiName[iDSI]);
				table.setNum( rowDSIg,iDSI+1,
					sc::getavgidx( sc::DSI,sc::MODEGEN,idx  ),SIFormat );
				if( outcontrol.showDSIgErr() ) {
					table.setNum( rowDSIgerr,iDSI+1,
						sc::geterridx( sc::DSI,sc::MODEGEN,idx ),errFormat );
				}
			}
		}
	}
	return( row );
}

void DevManager::showAccumResultAtmos( int row, bool isgen ) {

	jdebug( "Show Accum Res Atmos!!! row=%d", row );

	if( outcontrol.showSee() || outcontrol.showHeff() ) {
		table.clearRow( row ); row++;
		table.setHHead( row,"","FREE","err",
				"WHOLE","err" ); row++;

	}
	if( outcontrol.showSee() ) {
		table.clearRow( row );
		table.setText( row,0,"SEE " ); // seeng
		table.setNum( row, 1,
			atm::getval(atm::FSEE),seeFormat ); // mean free seeing
		table.setNum( row, 2,
			atm::geterr(atm::FSEE),errFormat ); // error 
		if (isgen) {
			table.setNum( row, 3,
				atm::getval(atm::WSEE),seeFormat ); // mean whole-atm. seeing
			table.setNum( row, 4,
				atm::geterr(atm::WSEE),errFormat ); // error 
		}
		row++;
	}
	if( outcontrol.showHeff() ) {
		table.setText( row,0,"HEFF" );
		table.setNum( row, 1,
			atm::getval(atm::FHEFF),heffFormat );// mean free Heff
		table.setNum( row, 2,
			atm::geterr(atm::FHEFF),errFormat );// error 
		if (isgen) {
			table.setNum( row, 3,
				atm::getval(atm::WHEFF),heffFormat );// mean whole-atm Heff
			table.setNum( row, 4,
				atm::geterr(atm::WHEFF),errFormat );// error 
		}
		row++;
	}
	if( outcontrol.showM0() ) {
		const double m0Scale = 1e13;
		table.setText( row,0,"M0*1e13:" );
		table.setNum( row, 1,
			atm::getval(atm::FM0)*m0Scale,m0Format );
		table.setNum( row, 2,
			atm::geterr(atm::FM0),errFormat );// error 
		if (isgen) {
			table.setNum( row, 3,
				atm::getval(atm::WM0)*m0Scale,m0Format );
			table.setNum( row, 4,
				atm::geterr(atm::WM0),errFormat );// error 
		}
		row++;
	}
	if( outcontrol.showIsopl() ) {
		table.clearRow( row ); row++;
		table.setText( row,0,"ISOPL:" ); // isoplan

		table.setNum( row, 1,
			atm::getval(atm::ISOPL),isoplFormat );// mean isoplan
		table.setNum( row, 2,
			atm::geterr(atm::ISOPL),errFormat );// error 
		row++;
	}
	if( outcontrol.showM2() ) {
		const double m2Scale = 1e5;
		table.setText( row,0,"M2*1e5:" );
// Attention! Order of magnitude of M2 is 1e-5!
		table.setNum( row, 1,
			atm::getval(atm::M2)*m2Scale,m2Format );// mean second moment of Cn2
		table.setNum( row, 2,
			atm::geterr(atm::M2),errFormat );// error 
		row++;
	}
//	double maxCn2 = 0;
//	for( int zidx=0; zidx<atm::getncn2(); zidx++ ) { 
//		if( maxCn2 < atm::getcn2(zidx) ) maxCn2 = atm::getcn2(zidx);
//	}
//	int orderCn2 = (maxCn2>0) ? int(log10(maxCn2)) : 0;
	int orderCn2 = -13;	// hardcoded on request of A.Tokovinin
	double scaleCn2 = pow(10,orderCn2);

	if( outcontrol.showTau() ) {
		table.setText( row,0,"TAU:" );
		table.setNum( row, 1,
			atm::getval(atm::TC),tauFormat );// mean atm. time constant
		table.setNum( row, 2,
			atm::geterr(atm::TC),errFormat );// error 
		row++;
	}

	if( outcontrol.showProfile() ) {
		table.clearRow( row ); row++;

		table.setText( row,0,"Z:" );
		table.setText( row+1,0,"Cn2:" );

		for (int zidx=0; zidx<atm::getncn2(); zidx++) {
			table.setNum( row, zidx+1,
				atm::getzcn2(zidx),zFormat );// layer altitude
			table.setNum( row+1, zidx+1,
				atm::getcn2(zidx)/scaleCn2,cn2Format );// its Cn2
		}
		row += 2;

		table.setText( row, 0, "10^");
		table.setNum( row, 1, orderCn2,orderFormat );// order of Cn2 scale
		table.setText( row, 2, "m^1/3");
		row++;

		table.setText( row, 0, "Chi2:");
		table.setNum( row, 1, 
			atm::getcn2(atm::getncn2()),chi2Format );// Chi2 in last Cn2[]
		table.setText( row, 3, "SeeCn:");
		table.setNum( row, 4,
			atm::getcn2see(),seeFormat ); //  seeing by profile

	}
}

int DevManager::catchAppendix( void ) {
//	int *p_timeout_flag = &count_timeout_flag;
	int n_read_data = exch::getNreadData();
	int ret = 0;

	if( !(exch::isTimeout()) ) {
		int block_appendix = 0;
		while( 1 ) {
			exch::wait( DEVICE_WAIT_INTERVAL );
			if( n_read_data == block_appendix ) break;
			if( srv_file ) fprintf( srv_file," appendix! %d != %d\n",n_read_data,block_appendix );
			block_appendix = n_read_data;
			ret = 1;
		}
	}
	return( ret );
}

void DevManager::outCountResult( void ) {
//	int n_read_data = exch::getNreadData();

	if( srv_file ) fprintf( srv_file,"EndTime: %s\n",
		trb_tm::getCurrentUT().toString().data() );
	int n_block = exch::getNblock();
	if( srv_file ) fprintf( srv_file,"tm_out=%d n_bl=%d\n",
			exch::isTimeout(),n_block );
}

int DevManager::runFluxEstimation( void ) {
	Time startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;

	int oldIllum = light.isIllumOn();

	if( !isScenario ) sendToScreen( emptyLine );
	openMode( estimName,getAccumTime( _count_mode ) );
	disablePolling();

	jdebug( "_________ FLUX_ESTIMATION_MODE:__________" );

	n_run = (int)ceil( _estimationAccumTime / _normalBaseTime );
	exch::setNcounters( MAX_N_COUNTERS );
	int expositionNumber;
	int timeoutInterval=4000;

	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		_abort=1;
		goto ABORT_ESTIM;
	}
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_ESTIM;
	}
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_ESTIM;
	}

	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );

	fprintf( out_file,"%s %s\n",getStamp( "M ",trb_tm::getCurrentUT() ),
		estimName.data() );

	sc::init( expositionNumber, n_run, MAX_N_COUNTERS );
	sc::writeavg( out_file,sc::AFLUX, sc::MODENORM,
			"#F   UT   ", 0,true );

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}
	
	sendFluxHead( ((_old_count_mode != _count_mode) && (!isScenario)),
			2,
			"CFLUX",
			"err" );

	for( int jj=0; jj<(*p_n_run); jj++ ) {

		startDevTimer( timeoutInterval );
		if( !runAllCounters() ) {
			sendErrorInfo( errCounter );
			_abort = 1;
			goto ABORT_ESTIM;
		}
		if( jj == 0 ) startAccumTime = _startBaseTime;

		exch::waitData();

		stopDevTimer();
		outCountResult();
		catchAppendix();
		cleanData( expositionNumber );

		sc::statflux( expositionNumber, _nCorrected,
			 _normalExposition, jj+1, 0 );

//		if( (_abort=testNrError()) ) goto ABORT_ESTIM;

		avgErrFluxToTable();

		if( (_abort=atBaseTime())  ) goto ABORT_ESTIM;
		setRemainTimeMode( ((*p_n_run) - jj)/_normalBaseTime );

	}	// jj-cycle
ABORT_ESTIM:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {
		Time midAccumTime = setMidTime( startAccumTime,
					_startBaseTime ); // first & last Base Time
		sc::writeavg( out_file,sc::AFLUX,sc::MODENORM,
			getStamp( "F ", midAccumTime ) );
		sendToScreen( emptyLine );
	}

	closeMode();
	enablePolling();
	turnIllum(oldIllum,0);
	if( isOverLight ) { handleOverLight(); _abort = 1; }
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runDetectorStatisticMeasurement( void ) {
	Time startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;
	double nonPois[MAX_N_COUNTERS];

	long firstAddr = -1;

	int oldIllum = light.isIllumOn();

	jdebug( "_________ DETECTOR_STATISTIC_MODE:__________" );

	disablePolling();

	if( !isScenario ) sendToScreen( emptyLine );
	double modeTime = getAccumTime( _count_mode );
	double elapsTime = 0;
	openMode( detStatMeasName, modeTime );

	n_run = (int)ceil( _detectorStatisticTime / _normalBaseTime );
	exch::setNcounters( MAX_N_COUNTERS );
	int _oldDoor = motor.isDoorOpen();
	int timeoutInterval=4000;

	infoExportMutex.lock();
	table.setHHead( 0,"Light","Param",
				nameA, nameB, nameC, nameD );
	infoExportMutex.unlock();

	sendTable( 0,0,1,6,true );

	fprintf( out_file, "%s %s\n",getStamp( "M ",trb_tm::getCurrentUT() ),
		detStatMeasName.data() );

	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		_abort=1;
		goto ABORT_DET_STAT;
	}
	if( setLight( _detectorStatisticLight[0],0 ) != 1 ) {
		sendErrorInfo( errLight );
		_abort=1;
		goto ABORT_DET_STAT;
	}
	if( turnLight(ON,0) != 1 ) {
		sendErrorInfo( errLight );
		_abort=1;
		goto ABORT_DET_STAT;
	}
	elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
	if( turnDoor( OFF,0,false ) != 1 ) {
		sendErrorInfo( notClosed );
		_abort = 1;
		goto ABORT_DET_STAT;
	}
	if( setAperture( _statisticApert,0 ) != 1 ) {
		sendErrorInfo( errAperture );
		_abort=1;
		goto ABORT_DET_STAT;
	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_DET_STAT;
	}
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_DET_STAT;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );

	sc::init( expositionNumber, n_run, MAX_N_COUNTERS );
	{
		const char* addrHead = 0;
		if( _techCountSave ) addrHead = "Address N";
		sc::writeavg( out_file,sc::AFLUXP, sc::MODENORM, "#L   UT    Light",
				addrHead, true );
	}
	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, 0, 0, -1, 
			(counters[iCounter]).getData() );
	}

	for( int iLight=0; iLight<_nDetectorStatisticLight; iLight++ ) {
		for( int jj=0; jj<*p_n_run; jj++ ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendErrorInfo( errCounter );
				_abort = 1;
				goto ABORT_DET_STAT;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			exch::waitData();

			stopDevTimer();
			elapsTime += _normalBaseTime;
			outCountResult();
			catchAppendix();

			if( _techCountSave ) {
				long addr = -1;
				cleanData( expositionNumber, &addr );
				if( jj == 0 ) firstAddr = addr;
			} else {
				cleanData( expositionNumber );
			}

			sc::statflux( expositionNumber, _nCorrected,
				 _normalExposition, jj+1, true, nonPois
//					,fluxAvg, sigAvg	// for TEST !!!
				);

			if( (_abort=atBaseTime())  ) goto ABORT_DET_STAT;
			setRemainTimeMode( modeTime - elapsTime );

		} // jj-cycle
		Time midAccumTime = setMidTime( startAccumTime,
					_startBaseTime ); // first & last Base Time

		infoExportMutex.lock();
		table.setNum( iLight*4+1,0,
			_detectorStatisticLight[iLight],lightFormat );
		table.clearRow( iLight*4+2 );
		table.clearRow( iLight*4+3 );
		table.setText( iLight*4+1,1, "AFLUX" );
		table.setText( iLight*4+2,1, "err" );
		table.setText( iLight*4+3,1, "P" );
		table.clearRow( iLight*4+4 );
		infoExportMutex.unlock();

		str_add.sprintf( "%5.2f", _detectorStatisticLight[iLight] );
		QString addr_str = "";
		if( firstAddr >= 0 ) {
			addr_str.setNum( *p_n_run );
			addr_str.prepend( " " );
			addr_str.prepend( getAddrStr( firstAddr ) );
			addr_str.prepend( " " );
		}
		sc::writeavg( out_file,sc::AFLUXP,sc::MODENORM,
			getStamp( "L ", midAccumTime, str_add.data() ),
			addr_str.data() );

		for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {

			infoExportMutex.lock();
			table.setNum( iLight*4+1,iCounter+2,
				sc::getavgflux(0,iCounter),fluxFormat );
			table.setNum( iLight*4+2,iCounter+2,
				sc::geterrflux(0,iCounter),errFormat );
			table.setNum( iLight*4+3,iCounter+2,
					nonPois[iCounter],poisFormat );
			infoExportMutex.unlock();

		}
		sendTable( iLight*4+1,0,4,6 );
		if( (iLight+1) < _nDetectorStatisticLight ) {
			jdebug( "_SET LIGHT %f", _detectorStatisticLight[iLight+1] );
			if( setLight( _detectorStatisticLight[iLight+1],0 ) != 1 ) {
				sendErrorInfo( errLight );
				_abort = 1;
				goto ABORT_DET_STAT;
			}
			elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
		}
	}  // iLight
ABORT_DET_STAT:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {
		sendToScreen( emptyLine );
	}
	turnLight(OFF,0);
	turnDoor( _oldDoor,0,false );
	closeMode();
	enablePolling();
	turnIllum(oldIllum,0);
	if( isOverLight ) { handleOverLight(); _abort = 1; }
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runDetectorCountingMeasurement( void ) {
	int *p_n_run = &n_run;
	Time startAccumTime;
	Time midAccumTime;
	int _abort = 0;
//	int iLight,iVolt,iLevel,iCounter;

	double **nonPois = nr::dmatrix(0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );
	double **avg = nr::dmatrix(0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );
	double **sig = nr::dmatrix(0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );

	QArray<double> _oldLevels(MAX_N_COUNTERS);
	QArray<double> _levels(MAX_N_COUNTERS);

	int oldIllum = light.isIllumOn();

	if( !isScenario ) sendToScreen( emptyLine );
	disablePolling();

	jdebug( "_________ DETECTOR_COUNTING_MODE:__________" );

	n_run = (int)ceil( _detectorCountingTime / _normalBaseTime );
	exch::setNcounters( MAX_N_COUNTERS );

	int timeoutInterval = 4000;
	int _oldDoor = motor.isDoorOpen();
	int _oldVoltage = getVoltage();
	getDiscriminations( _oldLevels );

	infoExportMutex.lock();
	table.setHHead( 0,"U","Level","Param",
			nameA, nameB, nameC, nameD );
	table.clearRow( 1 );
	infoExportMutex.unlock();

	sendTable( 0,0,1,7,true );

	fprintf( out_file,"%s %s\n",getStamp( "M ",trb_tm::getCurrentUT() ),
		detCountMeasName.data() );

	double modeTime = getAccumTime( _count_mode );
	double elapsTime = 0;
	openMode( detCountMeasName,modeTime );
	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		_abort=1;
		goto ABORT_DET_COUNT;
	}
	if( setLight( _detectorCountingLight[0],0 ) != 1 ) {
		sendErrorInfo( errLight );
		_abort=1;
		goto ABORT_DET_COUNT;
	}
	elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
	if( turnLight(ON,0) != 1 ) {
		sendErrorInfo( errLight );
		_abort=1;
		goto ABORT_DET_COUNT;
	}
	if( turnDoor( OFF,0,false ) != 1 ) {
		sendErrorInfo( notClosed );
		_abort = 1;
		goto ABORT_DET_COUNT;
	}
	if( setAperture( _countApert,0 ) != 1 ) {
		sendErrorInfo( errAperture );
		_abort=1;
		goto ABORT_DET_COUNT;
	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_DET_COUNT;
	}
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_DET_COUNT;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );

	sc::init( expositionNumber, n_run, MAX_N_COUNTERS );
	sc::writeavg( out_file,sc::AFLUXP,sc::MODENORM,"#U   UT    Light  U  Lev  ",
		0,true );
	jdebug( "write Header..." );

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter], 0, -1, 
			(counters[iCounter]).getData() );
	}

	for( int iLight=0; iLight<_nDetectorCountingLight; iLight++ ) {
		for( int iVolt=0; iVolt<_nDetectorCountingVoltage; iVolt++ ) {

			jdebug( "____ SET VOLTAGE %d", _detectorCountingVoltage[iVolt] );
			if( setVoltage( _detectorCountingVoltage[iVolt],0 ) != 1 ) {
				sendErrorInfo( errVoltage );
				_abort = 1;
				goto ABORT_DET_COUNT;
			}
			elapsTime += HV_WAIT_INTERVAL/S2MS;
			for( int jj=0; jj<*p_n_run; jj++ ) {
				for( int iLevel=0; iLevel<_nDetectorCountingDiscrimination; iLevel++ ) {
					for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
						_levels[iCounter] =  _detectorCountingDiscrimination[iLevel];
					} 
					if( setDiscriminations( _levels,0 ) != 1 ) {
						sendErrorInfo( errLevel );
						_abort = 1;
						goto ABORT_DET_COUNT;
					}

					startDevTimer( timeoutInterval );
					if( !runAllCounters() ) {
						sendErrorInfo( errCounter );
						_abort = 1;
						goto ABORT_DET_COUNT;
					}
					if( jj == 0 ) startAccumTime = _startBaseTime;

					exch::waitData();

					stopDevTimer();
					elapsTime += _normalBaseTime;
					outCountResult();
					catchAppendix();
					cleanData( expositionNumber );

					sc::statflux( expositionNumber, _nCorrected, _normalExposition, 
						jj+1, false, nonPois[iLevel], avg[iLevel], sig[iLevel] );

					if( (_abort=atBaseTime())  ) goto ABORT_DET_COUNT;

	// if end of jj-cycle - write to file
					if( jj == (*p_n_run-1) ) {
						if( iLevel == 0 ) {
							midAccumTime = setMidTime( startAccumTime,
								_startBaseTime ); // first & last Base Time
						}
						str_add.sprintf( "%5.2f%5d%5.2f",
							_detectorCountingLight[iLight],
							_detectorCountingVoltage[iVolt],
							_detectorCountingDiscrimination[iLevel] );
						sc::writeavg( out_file,sc::AFLUXP,sc::MODENORM,
							getStamp( "U ", midAccumTime, str_add.data() ) );
					}
				} // iLevel
				setRemainTimeMode( modeTime - elapsTime );
			} // jj-cycle

			infoExportMutex.lock();
			table.setText( 1,0,"Light:" );
			table.setNum( 1,1, _detectorCountingLight[iLight],
						lightFormat );
			infoExportMutex.unlock();

			for( int iLevel=0; iLevel<_nDetectorCountingDiscrimination; iLevel++ ) {

				infoExportMutex.lock();
				table.clearRow( 2+iLevel*3 );
				table.clearRow( 2+iLevel*3+1 );
				table.clearRow( 2+iLevel*3+2 );
				table.setNum( 2+iLevel*3,0,
					_detectorCountingVoltage[iVolt],hvFormat );
				table.setNum( 2+iLevel*3,1,
					_detectorCountingDiscrimination[iLevel],levFormat );
				table.setText( 2+iLevel*3,2,"AFLUX" );
				table.setText( 2+iLevel*3+1,2,"err" );
				table.setText( 2+iLevel*3+2,2,"P" );
				infoExportMutex.unlock();

				for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
					double sig_avg = sqrt( sig[iLevel][iCounter] );

					infoExportMutex.lock();
					table.setNum( 2+iLevel*3,iCounter+3,
							avg[iLevel][iCounter],fluxFormat );
					table.setNum( 2+iLevel*3+1,iCounter+3,
							sig_avg,errFormat );
					table.setNum( 2+iLevel*3+2,iCounter+3,
							nonPois[iLevel][iCounter],poisFormat );
					infoExportMutex.unlock();

				}
			}

			sendTable( 1,0,
				1+3*_nDetectorCountingDiscrimination,7 );
		}  // iVolt
		if( (iLight+1) < _nDetectorCountingLight ) {
			jdebug( "_SET LIGHT %f", _detectorCountingLight[iLight+1] );
			if( setLight( _detectorCountingLight[iLight+1],0 ) != 1 ) {
				sendErrorInfo( errLight );
				_abort = 1;
				goto ABORT_DET_COUNT;
			}
			elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
		}
	}  // iLight
ABORT_DET_COUNT:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {
		sendToScreen( emptyLine );
	}
	jdebug( "ABORT_DET_COUNT:" );
	setVoltage( _oldVoltage,0 );
	setDiscriminations( _oldLevels,0 );
	turnDoor( _oldDoor,0,false );
	turnLight(OFF,0);
	closeMode();
	enablePolling();
	turnIllum(oldIllum,0);
	if( isOverLight ) { handleOverLight(); _abort = 1; }

	nr::free_dmatrix( avg, 0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );
	nr::free_dmatrix( sig, 0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );
	nr::free_dmatrix( nonPois, 0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );

	if( _abort ) return(0);
	else return(1);
}

int DevManager::runExchangeTest( void ) {
	int *p_n_run = &n_run;
	Time startAccumTime;
	int timeout_error;
	double percentage = 0.0;
	int _abort = 0;
//	int iCounter = 0;

	disablePolling();

	if( !isScenario ) sendToScreen( emptyLine );
	openMode( commonTestName,getAccumTime( _count_mode ) );

	jdebug( "_________ EXCHANGE_TEST_MODE:__________" );

//	int _oldDoor = motor.isDoorOpen();
	
	n_run = (int)ceil( _exchangeTestTime / _normalBaseTime );

	int n_run0 = *p_n_run;
	int timeoutInterval = 4000;

//  --------------- Test Counters A,B,C,D -------------------

	int N_RUN_1 = 1;
	int jjReal = 0;

//	if( turnDoor( OFF,0,false ) != 1 ) {
//		sendErrorInfo( notClosed );
//		_abort = 1;
//		goto ABORT_TEST_ALL;
//	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_TEST_ALL;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_TEST_ALL;
	}

	exch::setNcounters( 1 );
	if( !resetTestCounter() ) {
		sendErrorInfo( errCounter );
		_abort=1;
		goto ABORT_TEST_ALL;
	}

	fprintf( out_file,"%s %s\n",getStamp( "M ",trb_tm::getCurrentUT() ),
		commonTestName.data() );

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		timeout_error = 0;
		resetTestData();
		for( int jj=0; jj<N_RUN_1; jj++ ) {

			startDevTimer( timeoutInterval );
			if( !runTestOneCounter() ) {
				sendErrorInfo( errCounter );
				_abort=1;
				break;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			exch::waitData();

			stopDevTimer();
			outCountResult();
			catchAppendix();

			searchTestErrorsOneCounter();

			jjReal++;
		}
		if( srv_file ) fprintf( srv_file," n_run=%d tm_out_err=%d\n",
			jjReal,timeout_error );
		if( jjReal < N_RUN_1 ) {
			if( testCounter ) {
				sendErrorInfo( abortProcess );
				if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
				_abort=1;
			}
			percentage += 1.0; 
		} else {
			percentage += resumeTestOneCounter() / (double)N_RUN_1;
		}

		if( !switchTestCounter() ) {
			sendErrorInfo( errCounter );
			_abort=1;
			goto TEST_ALL;
		}

	}
	if( percentage == 0 ) {
		str_edit = indTestName;
		str_edit.append( successfully );
	} else {
		percentage = 100.0 * percentage / (double)MAX_N_COUNTERS;
		str_edit.sprintf( "%s Fault = %5.2f per cent",
				indTestName.data(), percentage );
	}
	sendErrorInfo( str_edit );
// ---------------- Test All Counters ----------------

TEST_ALL: 
	exch::setNcounters( MAX_N_COUNTERS );
	resetTestData();
	if( !setInduceRegime(0) ) {
		sendErrorInfo( errCounter );
		_abort=1;
		goto ABORT_TEST_ALL;
	}
	timeout_error = 0;

	for( int jj=0; jj<*p_n_run; jj++ ) {
		startDevTimer( timeoutInterval );
		if( !runTestAllCounters() ) {
			sendErrorInfo( errCounter );
			_abort=1;
			break;
		}
		if( jj == 0 ) startAccumTime = _startBaseTime;

		exch::waitData();

		stopDevTimer();

		outCountResult();
		catchAppendix();

		searchTestErrorsAllCounters();

		if( (_abort=atBaseTime())  ) goto ABORT_TEST_ALL;
		setRemainTimeMode( ((*p_n_run) - jj)/_normalBaseTime );
	}
	if( srv_file ) fprintf( srv_file,"tm_out_err=%d\n",
		timeout_error );
ABORT_TEST_ALL:
	if( !_abort ) {
		percentage = resumeTestAllCounters() / (double)n_run0;
		if( percentage == 0 ) {
			str_edit = commonTestName;
			str_edit.append( successfully );
		} else {
			percentage = 100.0 * percentage;
			str_edit.sprintf( "%s Fault = %5.2f per cent",
				commonTestName.data(), percentage );
		}
		sendErrorInfo( str_edit );
	}

	closeMode();
//	turnDoor( _oldDoor,0,false );
	enablePolling();
	if( isOverLight ) { handleOverLight(); _abort = 1; }
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runDetectorTest( void ) {
	int *p_n_run = &n_run;
	Time startAccumTime;
	int _abort = 0;
	int success = 1;
	int timeoutInterval = 4000;
	double nonPois[MAX_N_COUNTERS];

	int oldIllum = light.isIllumOn();

	jdebug( "start of Det Test!" );
	if( !isScenario ) sendToScreen( emptyLine );
	openMode( detTestName, getAccumTime( _count_mode ) );

	disablePolling();

	jdebug( "_________ DETECTOR_TEST_MODE:__________" );

	n_run = (int)ceil( _detectorTestTime / _normalBaseTime );
	exch::setNcounters( MAX_N_COUNTERS );

//	int _oldDoor = motor.isDoorOpen();

	fprintf( out_file,"%s %s\n",getStamp( "M ",trb_tm::getCurrentUT() ),
		detTestName.data() );

	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		_abort=1;
		goto ABORT_DET_TEST;
	}
	if( setLight( _detectorTestLight,0,SHORT_LIGHT_WAIT_INTERVAL ) != 1 ) {
		sendErrorInfo( errLight );
		_abort=1;
		goto ABORT_DET_TEST;
	}
	if( turnLight(ON,0,SHORT_LIGHT_WAIT_INTERVAL) != 1 ) {
		sendErrorInfo( errLight );
		_abort=1;
		goto ABORT_DET_TEST;
	}

//	if( turnDoor( OFF,0,false ) != 1 ) {
//		sendErrorInfo( notClosed );
//		_abort = 1;
//		goto ABORT_DET_TEST;
//	}
	if( setAperture( _testApert,0 ) != 1 ) {
		sendErrorInfo( errAperture );
		_abort=1;
		goto ABORT_DET_TEST;
	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_DET_TEST;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_DET_TEST;
	}

	sc::init( expositionNumber, n_run, MAX_N_COUNTERS );
	sc::writeavg( out_file,sc::AFLUXP,sc::MODENORM,"#L   UT    Light",
			0,true );
	
	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}

	sendFluxHead( _old_count_mode != _count_mode,
			3,
			"CFLUX",
			"err",
			"P" );

	for( int jj=0; jj<*p_n_run; jj++ ) {
		startDevTimer( timeoutInterval );
		if( !runAllCounters() ) {
			sendErrorInfo( errCounter );
			_abort = 1;
			goto ABORT_DET_TEST;
		}
		if( jj == 0 ) startAccumTime = _startBaseTime;

		exch::waitData();

		stopDevTimer();
		outCountResult();
		catchAppendix();

		cleanData( expositionNumber );

		sc::statflux( expositionNumber, _nCorrected,
				 _normalExposition, jj+1, false, nonPois );

//		if( (_abort=testNrError()) ) goto ABORT_DET_TEST;

		avgErrPoisFluxToTable( nonPois );

		if( (_abort=atBaseTime())  ) goto ABORT_DET_TEST;
		setRemainTimeMode( ((*p_n_run) - jj) /_normalBaseTime );

	} // jj-cycle
ABORT_DET_TEST:
	if( !_abort ) {
		Time midAccumTime = setMidTime( startAccumTime,
					_startBaseTime ); // first & last Base Time
		str_add.sprintf( "%5.2f",_detectorTestLight );
		sc::writeavg( out_file,sc::AFLUXP,sc::MODENORM,
			getStamp( "L ", midAccumTime,str_add.data() ));

		for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			if( fabs(sc::getavgflux(0,iCounter)-_testCounts[iCounter])
					/_testCounts[iCounter] > _fluxTolerance ) {
				str_edit = fluxError;
				success = 0;
			} else if( fabs(nonPois[iCounter]-_nonPoisson[iCounter])
					> _nonPoisTolerance ) {
				str_edit = nonPoisError;
				success = 0;
			}
		}
		if( success == 0 ) {
			sendErrorInfo( str_edit );
		} else {
			sendErrorInfo( successfully );
		}
	}
	turnLight(OFF,0);
//	turnDoor( _oldDoor,0,false );
	closeMode();
	enablePolling();
	turnIllum(oldIllum,0);
	if( isOverLight ) { handleOverLight(); _abort = 1; }
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runStatisticTest( void ) {
	Time startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;

	int oldIllum = light.isIllumOn();

	if( !isScenario ) sendToScreen( emptyLine );
	disablePolling();

	jdebug( "_________ STATISTIC_TEST_MODE:__________" );

	n_run = (int)ceil( _normalAccumTime / _normalBaseTime );
	exch::setNcounters( MAX_N_COUNTERS );

	int timeoutInterval = 4000;

	double elapsTime = 0;
	double modeTime = getAccumTime( _count_mode );
	openMode( statTestName, modeTime );

	int _oldDoor = motor.isDoorOpen();

	jdebug( "old_mode = %d _count_mode = %d",_old_count_mode,_count_mode );

	sendFluxHead( _old_count_mode != _count_mode,
			1,
			"CFLUX" );
	fprintf( out_file,"%s %s\n",getStamp( "M ",trb_tm::getCurrentUT() ),
		statTestName.data() );

	{ // "try"
	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		_abort=1;
		goto ABORT_ST_TEST;
	}
	if( setLight( _statisticTestLight[0],0 ) != 1 ) {
		sendErrorInfo( errLight );
		_abort=1;
		goto ABORT_ST_TEST;
	}
	elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
	if( turnLight(ON,0) != 1 ) {
		sendErrorInfo( errLight );
		_abort=1;
		goto ABORT_ST_TEST;
	}
	if( turnDoor( OFF,0, false ) != 1 ) {
		sendErrorInfo( notClosed );
		_abort = 1;
		goto ABORT_ST_TEST;
	}
	if( setAperture( _testApert,0 ) != 1 ) {
		sendErrorInfo( errAperture );
		_abort=1;
		goto ABORT_ST_TEST;
	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_ST_TEST;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_ST_TEST;
	}

	sc::init( expositionNumber, n_run * 2, MAX_N_COUNTERS );
#ifdef WRITESI
	sc::write( out_file,sc::DSI,"#i   UT   ",0,true );
#endif
	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, 0,
		_nonLinearity[iCounter],
		_nonPoisson[iCounter],
		(counters[iCounter]).getData() );
	}

	for( int iLight=0; iLight<_nStatisticTestLight; iLight++ ) {
		double ampl = _testVAmpl * 
				_statisticTestLight[iLight];
		if( (_statisticTestLight[iLight] + ampl) > MAX_LIGHT_VALUE )
			ampl = MAX_LIGHT_VALUE - _statisticTestLight[iLight];
		if( (_statisticTestLight[iLight] - ampl) < MIN_LIGHT_VALUE )
			ampl = _statisticTestLight[iLight] - MIN_LIGHT_VALUE;
		fprintf( out_file,"%s VAmpl=%.2f\n",
			getStamp( "> ",trb_tm::getCurrentUT() ),
				ampl );

		if( light.setVAmpl( ampl,0 ) != 1 ) {
			sendErrorInfo( errLight );
			_abort=1;
			goto ABORT_ST_TEST;
		}
		if( light.setVaryOn( 0 ) != 1 ) {
			sendErrorInfo( errLight );
			_abort=1;
			goto ABORT_ST_TEST;
		}

		int nSkipped = 0 ;
		for( int jj=0; jj<*p_n_run; jj++ ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendErrorInfo( errCounter );
				_abort = 1;
				goto ABORT_ST_TEST;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			exch::waitData();

			stopDevTimer();
			elapsTime += _normalBaseTime;
			outCountResult();
			catchAppendix();
			bool isCorrected = cleanData( expositionNumber );
			if( isCorrected ) {
				if( srv_file ) fprintf( srv_file,"Skip baseTime \n" );
				nSkipped ++ ;
				continue;
			}

			// compute count stat.moments:
			sc::calcmom( expositionNumber, _nCorrected );
			// no write of stat.moments in tests

			if( (_abort=testNrError()) ) goto ABORT_ST_TEST;

			sc::calcidx( _normalExposition, sc::MODENORM );

#ifdef WRITESI
			sc::write( out_file,sc::DSI,
				getStamp("i ",_startBaseTime) );
#endif
			sc::stattest(expositionNumber, _normalExposition) ;
#ifdef WRITESI
			sc::write( out_file,sc::DSI,
				getStamp("e ",_startBaseTime) );
#endif
//			if( (_abort=testNrError()) ) goto ABORT_ST_TEST;

			fluxMeanToTable( false,true );

			if( (_abort=atBaseTime())  ) goto ABORT_ST_TEST;
			setRemainTimeMode( modeTime - elapsTime );

		} // jj-cycle
		Time midAccumTime = setMidTime( startAccumTime,
					_startBaseTime ); // first & last Base Time
		sc::average( true, sc::MAXLAG );
		sc::writeavg( out_file,sc::DSI,sc::MODENORM,
			"#I   UT   ", 0, true );
		sc::writeavg( out_file,sc::DSI,sc::MODENORM,
			getStamp( "I ",midAccumTime ) );
		sc::writeavg( out_file,sc::DSI,sc::MODEGEN ,
			getStamp( "E ",midAccumTime ) );
		sc::writeavg( out_file,sc::AFLUX,sc::MODENORM,
			"#F   UT   ", 0, true );
		sc::writeavg( out_file,sc::AFLUX,sc::MODENORM,
			getStamp("F ",midAccumTime) );

		infoExportMutex.lock();

		for( int i=0; i<MAX_ROW; i++ ) table.clearRow(i);
		showAccumResultScind( false,true );

		table.isChanged = true;
		table.startRow = 0;
		table.nRows = MAX_ROW;
		table.startCol = 0;
		table.nCols = MAX_COL;

		infoExportMutex.unlock();

		sc::again();

		if( (_abort=testNrError()) ) goto ABORT_ST_TEST;

		if( light.setVaryOff(0) != 1 ) {
			sendErrorInfo( errLight );
			_abort=1;
			goto ABORT_ST_TEST;
		}
		if( (iLight+1) < _nStatisticTestLight ) {
			if( setLight( _statisticTestLight[iLight+1],0 ) != 1 ) {
				sendErrorInfo( errLight );
				_abort = 1;
				break;
			}
			elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
		}
	} // iLight
	} // "catch"
ABORT_ST_TEST:
	if( !_abort ) {
		sendErrorInfo( successfully );
	}
	light.setVaryOff(0);
	turnLight(OFF,0);
	turnDoor( _oldDoor,0,false );
	closeMode();
	enablePolling();
	turnIllum(oldIllum,0);
	if( isOverLight ) { handleOverLight(); _abort = 1; }
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runBackgroundCorrection( void ) {
	Time startAccumTime;
	int nShift = 0;
	int seqNum = 0;
	const char* shift = "";
	const int nPoint = 3 ;
	double avg[nPoint][MAX_N_COUNTERS];
	double sig[nPoint][MAX_N_COUNTERS];

	int *p_n_run = &n_run;
	int _abort = 0;
	int success;

	int oldIllum = light.isIllumOn();

	if( !isScenario ) sendToScreen( emptyLine );
	double modeTime = getAccumTime( _count_mode );
	openMode( backCorrName, modeTime );
	double elapsTime = 0;

	disablePolling();

	jdebug( "_________ BACKGROUND CORRECTION MODE:__________" );

	n_run = (int)ceil( _backgroundTime / _normalBaseTime );
	exch::setNcounters( MAX_N_COUNTERS );

	int expositionNumber;
	int timeoutInterval = 4000;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_CORR;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_CORR;
	}
	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		_abort=1;
		goto ABORT_CORR;
	}

	sc::init( expositionNumber, n_run, MAX_N_COUNTERS );
	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}

	fprintf( out_file,"%s %s\n",getStamp("M ",trb_tm::getCurrentUT()),
		backCorrName.data() );

	sendFluxHead( _old_count_mode != _count_mode,
			2,
			"CFLUX",
			"err" );

	for( int k=0; k<4; k++ ) {	// Aperture shift-cycle (center-left-right)
		switch( k ) {
			case 0: 
			case 2:
				if( setAperture( _backgroundApert,0 ) != 1 ) {
						sendErrorInfo( errAperture );
						_abort = 1;
						goto ABORT_CORR;
				}
				nShift = 0;
				shift = "Center";
				break;	// center
			case 1: if( shiftAperture( _backgroundApShift,0 ) != 1 ) {
						sendErrorInfo( errShiftApert );
						_abort = 1;
						goto ABORT_CORR;
				}
				nShift = 1;
				shift = "Left";
				break;	// left
			case 3: if( shiftAperture( -2 * _backgroundApShift,0 ) != 1 ) {
						sendErrorInfo( errShiftApert );
						_abort = 1;
						goto ABORT_CORR;
				}
				nShift = 2;
				shift = "Right";
				break;	// right
		}

		if( !isScenario ) {
			str_edit = shift;
			sendToScreen( str_edit );
		}

		for( int jj=0; jj<*p_n_run; jj++ ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendErrorInfo( errCounter );
				_abort = 1;
				goto ABORT_CORR;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			exch::waitData();

			stopDevTimer();
			elapsTime += _normalBaseTime;
			outCountResult();
			catchAppendix();
			cleanData( expositionNumber );

			switch( k ) {
				case 0: seqNum = jj+1;
					break;		// center
				case 1: seqNum = jj+1;
					break;		// left
				case 2: seqNum = (*p_n_run) + jj;
					break;		// center
				case 3: seqNum = jj+1;
					break;		// right
			}

			sc::statflux( expositionNumber, _nCorrected, _normalExposition, 
				seqNum, false, 0, avg[nShift], sig[nShift] );

//			if( (_abort=testNrError()) ) goto ABORT_CORR;

			for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				fluxToTable( 2,
					iCounter,
					avg[nShift][iCounter],
					sqrt( sig[nShift][iCounter] )
					);
			}
			sendTable( 1,1,2,4 );

			if( (_abort=atBaseTime())  ) goto ABORT_CORR;
			setRemainTimeMode( modeTime - elapsTime );

		}	// jj-cycle
	}	// k-cycle
ABORT_CORR:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	}
	else {
		sendToScreen( emptyLine );
		if( coef ) {
			for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				fprintf( out_file,"%s Channel %d:",
					getStamp( "> ",trb_tm::getCurrentUT() ),
					iCounter );
				for( int nShift=1; nShift<3; nShift++ ) {
					if( avg[nShift][iCounter] != 0 ) {
						(*coef)[iCounter][nShift-1] =
						avg[0][iCounter]/avg[nShift][iCounter];
					} else (*coef)[iCounter][nShift-1] = 0.0;
					fprintf( out_file,"%10.3f", (*coef)[iCounter][nShift-1] );
				}
				(*coef)[iCounter][2] = ( (*coef)[iCounter][0] +
					(*coef)[iCounter][1] ) / 2.0;
				fprintf( out_file,"%10.3f\n", (*coef)[iCounter][2] );
			}
		}

		StateExport::setCorrectionReady(true);
	}
	if( _abort ) success=0;
	else success=1;
	setAperture( _backgroundApert,0 ,true ); // t.k. byla shifted
	closeMode();
	enablePolling();
	turnIllum(oldIllum,0);
	if( isOverLight ) { handleOverLight(); _abort = 1; }

	return( success );
}

const int best = 3;

int DevManager::runBackgroundMeasurement( void ) {
	Time startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;
	int success;

	const char* shift = "";
	const int nPoint = 2;
	double avg[nPoint][MAX_N_COUNTERS];
	double sig[nPoint][MAX_N_COUNTERS];

	int oldIllum = light.isIllumOn();

	if( !isScenario ) sendToScreen( emptyLine );
	double modeTime = getAccumTime( _count_mode );
	openMode( backMeasName, modeTime );
	double elapsTime = 0;

	disablePolling();

	jdebug( "_________ BACKGROUND MEASUREMENT MODE:__________" );

	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		_abort=1;
		goto ABORT_BACKG;
	}
	n_run = (int)ceil( _backgroundTime / (2.0*_normalBaseTime) );
	exch::setNcounters( MAX_N_COUNTERS );

	int expositionNumber;
	int timeoutInterval;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_BACKG;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_BACKG;
	}
	if( setAperture( _backgroundApert,0 ) != 1 ) {
		sendErrorInfo( errAperture );
		_abort = 1;
		goto ABORT_BACKG;
	}

	fprintf( out_file,"%s %s\n",getStamp("M ",trb_tm::getCurrentUT() ), backMeasName.data() );

	sc::init( expositionNumber, n_run, MAX_N_COUNTERS );
	exch::setNcounters( MAX_N_COUNTERS );
	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}

	sendFluxHead( ((_old_count_mode != _count_mode) && (!isScenario)),
			2,
			"CFLUX",
			"err" );

	for( int k=0; k<nPoint; k++ ) {	// Aperture(telescope) shift-cycle (left-right)
		switch( k ) {
			case 0:
				if( _backgroundMethod == APERTURE_SHIFT_METHOD ) {
					if( shiftAperture( _backgroundApShift,0 ) != 1 ) {
						sendErrorInfo( errShiftApert );
						_abort = 1;
						goto ABORT_BACKG;
					}
					shift = "Left";
				} else {
					if( greatTrack(DECLINATION,1,_backgroundTelShift) != 1 ) {
						sendErrorInfo( errShiftTelesc );
						_abort = 1;
						goto ABORT_BACKG;
					}
					shift = "DEC+";
				}
				break;	// left(dec+)

			case 1:
				if( _backgroundMethod == APERTURE_SHIFT_METHOD ) {
					if( shiftAperture( -2 * _backgroundApShift,0 ) != 1 ) {
						sendErrorInfo( errShiftApert );
						_abort = 1;
						goto ABORT_BACKG;
					}
					shift = "Right";
				} else {
					if( greatTrack(DECLINATION,-1,2*_backgroundTelShift) != 1 ) {
						sendErrorInfo( errShiftTelesc );
						_abort = 1;
						goto ABORT_BACKG;
					}
					shift = "DEC-";
				}
				break;	// right(dec-)
			}

			if( !isScenario ) {
				str_edit = shift;
				sendToScreen( str_edit );
			}

			for( int jj=0; jj<*p_n_run; jj++ ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendErrorInfo( errCounter );
				_abort = 1;
				goto ABORT_BACKG;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			exch::waitData();

			stopDevTimer();
			elapsTime += _normalBaseTime;

			outCountResult();
			catchAppendix();
			cleanData( expositionNumber );

			sc::statflux( expositionNumber, _nCorrected, _normalExposition, 
				jj+1, false, 0, avg[k], sig[k] );

//			if( (_abort=testNrError()) ) goto ABORT_BACKG;

			for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				fluxToTable( 2,
					iCounter,
					avg[k][iCounter],
					sqrt( sig[k][iCounter] )
					);
			}
			sendTable( 1,1,2,4 );

			if( (_abort=atBaseTime()) ) goto ABORT_BACKG;
			setRemainTimeMode( modeTime - elapsTime );

		}	// jj-cycle
	}	// k-cycle
ABORT_BACKG:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	}
	else {
		infoExportMutex.lock();
		table.clearRow( 2 );
		table.setText( 1, 0, "AFLUX" );
		infoExportMutex.unlock();
		
		str_edit = "";
		if( fabs( avg[0][best] - avg[1][best] )
				/avg[0][best] > _fluxTolerance ) {
			str_edit1 = "Wrong Background!";
			_abort = 1;
		} else {
			str_edit1 = "Background: ";
			for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				_backgrounds[iCounter] = (avg[0][iCounter] +
					avg[1][iCounter]) / 2.0;
				if( _backgroundMethod == APERTURE_SHIFT_METHOD ) {
					_backgrounds[iCounter] *=
						_backgroundCorrectionCoefficients[iCounter];
				}
				fluxToTable( 1, iCounter,
					_backgrounds[iCounter] );
				str_edit1 += str_add
					.setNum(_backgrounds[iCounter],'f',3 );
				str_edit += str_add.setNum(_backgrounds[iCounter],'f',3 );
				if( iCounter != MAX_N_COUNTERS-1 ) {
					str_edit1 += "  ";
					str_edit += ", ";
				}
			}
			sendTable( 1,0,2,5 );
			setLeafData( cfgFile,
				"Operations",
				"Background measurement",
				"Background",
				str_edit,
				out_file );
			}
		sendToScreen( str_edit1 );
	}
	if( _abort ) success=0;
	else success=1;
	if( _backgroundMethod == APERTURE_SHIFT_METHOD ) {
		setAperture( _backgroundApert,0 ,true ); // t.k. byla shifted
	} else {
		greatTrack(DECLINATION,1,_backgroundTelShift); // na mesto...
	}
	closeMode();
	enablePolling();
	turnIllum(oldIllum,0);
	if( isOverLight ) { handleOverLight(); _abort = 1; }
	return( success );
}

void DevManager::calcApertShift( double shiftX, double shiftY ) {
	_shiftRA = _mirror * _sinPosAngle * shiftX + _cosPosAngle * shiftY;
	_shiftDec = _cosPosAngle * shiftX - _sinPosAngle * shiftY;
}

bool DevManager::isStarInWorkZone( void ) {
	if( ( fabs(_shiftRA) >= _workZone ) ||
			( fabs(_shiftDec) >= _workZone ) ) {
		jdebug( "starPosition: Empty!" );
		return false; // no star in diaph.
	} else
		return true;
}

bool DevManager::isStarInCentralZone( void ) {
	if( ( fabs(_shiftRA) >= _centralZone ) ||
			( fabs(_shiftDec) >= _centralZone ) )
		return false;
	else return true;
}

void DevManager::starInDiaphragm( void ) {
	char pos[3];
	int code = -1;

	if( !isStarInWorkZone() ) code = -1;
	else {
		if( fabs(_shiftRA) < _centralZone ) pos[0] = '0'; // center
		else if( _shiftRA < 0 ) pos[0] = '1';	// left
		else pos[0] = '2';			// right

		if( fabs(_shiftDec) < _centralZone ) pos[1] = '0'; // center
		else if( _shiftDec < 0 ) pos[1] = '1';	// down
		else pos[1] = '2';			// up

		pos[2] = '\0';
		QString str( pos );
		jdebug( "starPosition: %s",str.data() );
		code = str.toInt();
		jdebug( "%d",code );
	}

	infoExportMutex.lock();
	InfoExport::starPosition = code;
	InfoExport::isStarPositionChanged  = true;
	infoExportMutex.unlock();

}

int DevManager::scanAperture( double accumTime, double baseTime,
			int blockNumber, double exposition, int aperture,
			int scanStart, const QString& modeName,
			bool isFocus ) {
	Time startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;

	double modeTime = getAccumTime( _count_mode );
	openMode( modeName, modeTime );
	double elapsTime = 0;

	disablePolling();

	jdebug( "_________ SCAN APERTURE:__________" );

	n_run = (int)ceil( accumTime / (2.0*baseTime) );
	exch::setNcounters( MAX_N_COUNTERS );
	
	int _nSteps = 0;
	int expositionNumber = 0;

	if( setAperture( aperture,0 ) != 1 ) {
		sendErrorInfo( errAperture );
		_abort=1;
		goto ABORT_SCAN;
	}
	if( shiftAperture( scanStart,0 ) != 1 ) {
		sendErrorInfo( errShiftApert );
		_abort = 1;
		goto ABORT_SCAN;
	}
	int timeoutInterval;
	if( setBlockNumber( blockNumber,expositionNumber,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_SCAN;
	}
	_nSteps = expositionNumber - 1;
	timeoutInterval = calcTimeoutInterval( exposition, expositionNumber );
	if( setExpos( exposition,0 ) != 1 ) {
		sendErrorInfo( errCounter );
		_abort = 1;
		goto ABORT_SCAN;
	}

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		(scan[iCounter]).set( expositionNumber,
			_nonLinearity[iCounter]/exposition );
	}

	fprintf( out_file,"%s %s\n",getStamp("M ",trb_tm::getCurrentUT() ), 
		modeName.data() );
	int iDirect;
	int numbError;
	for( int jj=0; jj<*p_n_run; jj++ ) {
		iDirect = 1;
		numbError = 0;
		for( int i=0; i<2; i++ ) {
			if( synchAt( iDirect*_nSteps,0 ) != 1 ) {
				sendErrorInfo( errShiftApert );
				_abort = 1;
				goto ABORT_SCAN;
			}
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendErrorInfo( errCounter );
				_abort = 1;
				goto ABORT_SCAN;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			exch::waitData();

			stopDevTimer();
			outCountResult();
			catchAppendix();

			bool isCorrected = cleanData( expositionNumber );
			if( isCorrected ) {
				numbError++;
				jdebug( "Retry scan!!! numbError=%d",numbError );
				if( numbError >= N_REPEAT_COMMAND ) {
					sendErrorInfo( errScan );
					_abort = 1;
					goto ABORT_SCAN;
				}
// Vozvrachaemsia dlia povtora scana:
				if( shiftAperture( -1*iDirect*_nSteps*SYNCH_STEP,0 ) != 1 ) {
					sendErrorInfo( errShiftApert );
					_abort = 1;
					goto ABORT_SCAN;
				}
				i--;
				continue;
			}
			elapsTime += baseTime;
			for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				scan[iCounter].accum( (counters[iCounter]).getData(),
				scan_t::direct( iDirect ) );
			}
			if( (_abort=testNrError( true )) ) goto ABORT_SCAN;
			if( (_abort=atBaseTime()) ) goto ABORT_SCAN;

			iDirect *= -1;  
			exch::wait( DEVICE_WAIT_INTERVAL );
		}
		setRemainTimeMode( modeTime - elapsTime );
	}	// jj-cycle
ABORT_SCAN:
	if( _abort ) {
		testNrError(true);
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {
		Time midAccumTime = setMidTime( startAccumTime,
					_startBaseTime ); // first & last Base Time
		for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			scan[iCounter].merge();
			GraphSet* gs = outcontrol.graphSet("SCAN",
				counters[iCounter].getChannel() );
			if(gs) {
				gs->copyScan(
					scan[iCounter].get(scan_t::MERG),
					scanStart,
					expositionNumber );
				outcontrol.setGraphReady( true );
			}
		}

		double cx1_f,cx2_f,cx1_b,cx2_b;
//		scan[best].center( &cx1,&cx2,scan_t::MERG );
		scan[best].center( &cx1_f,&cx2_f,scan_t::FORW );
		scan[best].center( &cx1_b,&cx2_b,scan_t::BACK );
		double cx1 = (cx1_f+cx1_b)/2.0;
		double cx2 = (cx2_f+cx2_b)/2.0;
		if( isFocus ) {
			double sharp;
			double defoc = scan[best].
				defocus( (cx1+cx2)/2,
					scan_t::MERG,
					&sharp );
			fprintf( out_file,"%s defoc = %.2f sharp = %.2f\n", 
					getStamp("V ",midAccumTime),
							defoc, sharp );
			str_edit.sprintf( "defoc=%.2f sharp=%.2f",
					defoc,sharp );
		} else {
			calcApertShift( scan[best].xshift(),
					scan[best].yshift() );
			fprintf( out_file,"%s shiftRA = %f shiftDec = %f\n", 
				getStamp("C ",midAccumTime),_shiftRA, _shiftDec );
			str_edit.sprintf( "shiftRA=%.2f shiftDec=%.2f",
					_shiftRA, _shiftDec );
			starInDiaphragm();
		}
		if( testNrError( true ) ) _abort = 1;
		sendToScreen( str_edit );
	}
	closeMode();
	enablePolling();
	if( isOverLight ) { handleOverLight(); _abort = 1; }
	if( _abort ) return(0);
	else return(1);;
}

int DevManager::runCenterCorrection( void ) {
	jdebug( "runCenterCorrection:" );
	int oldIllum = light.isIllumOn();
	int oldAperture = currentAperture();
	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		return 0;
	}
	if( !isScenario ) sendToScreen( emptyLine );
	int success = scanAperture( _centerAccumTime, _centerBaseTime,
		_centerBlockNumber, _centerExposition, _centerApert,
		_centerScanStart, centerCorName, false );
	if( (success == 1) && delta ) {
		delta->setX( _shiftRA );
		delta->setY( _shiftDec );
		StateExport::setCorrectionReady(true);
	}
	setAperture( oldAperture,0 );
	turnIllum(oldIllum,0);
	return success;
}

const double RA_PARAM = 0.9; // nedorabotka po RA
const double DEC_PARAM = 0.9; // nedorabotka po Dec

int DevManager::trackToCenter( void ) {
	if( !tracker.isReady() ) return 0;
	if( !_withAutoGuide ) return 0;
	if( _isTracking ) return 0;

	double tRA = RA_PARAM * (-1.0)*_shiftRA /
			cos( star.getDecVal() * RAD_IN_ASEC ) / _speedRA;
	double tDec = DEC_PARAM * (-1.0)*_shiftDec / _speedDec;

	int ret = shiftTelescope(RA,tRA,0);
	if( ret == 1 ) {
		openMode( trackDecName,fabs(tDec) );
		ret *= shiftTelescope(DECLINATION,tDec,0);
	}
	return ret;
}

int DevManager::runCenterControl( void ) {
	sendToScreen( emptyLine );
	if( !star.isStarSelected() ) {
		sendErrorInfo( noStar );
		return 0;
	}
	int oldAperture = currentAperture();
	int oldIllum = light.isIllumOn();
	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		return 0;
	}
	int success = 1;
 
	if( _withAutoGuide ) {
		for( int i=0; i<_maxIterCentering; i++ ) {
			jdebug( "scan:" );
			success = scanAperture( _centerAccumTime, _centerBaseTime,
				_centerBlockNumber, _centerExposition, _centerApert,
				_centerScanStart, starCenterName, false );

			if( trberr::quitProgram() ) quitProgram();

			if( success != 1 ) goto ABORT_CORRECT;
			if( !isStarInCentralZone() ) {
				jdebug( "correction" );
				success *= trackToCenter();
				if( success != 1 ) goto ABORT_CORRECT;
			} else {
				goto ABORT_CORRECT; // It's OK!
			}
		}
	} else {
		success = scanAperture( _centerAccumTime, _centerBaseTime,
			_centerBlockNumber, _centerExposition, _centerApert,
			_centerScanStart, starCenterName, false );

		if( trberr::quitProgram() ) quitProgram();
	}
// Correct. ne srabotala, no, mozhet, zvezda vsio zhe v diaphragme...
	if( !isStarInWorkZone() ) {
		str_edit = "Star is out of Work Zone!";
		sendErrorInfo( str_edit );
		success = 0;
	}
ABORT_CORRECT:
	setAperture( oldAperture,0 );
	turnIllum(oldIllum,0);
	return success;
}

int DevManager::runTrackCalibration( void ) {
	sendToScreen( emptyLine );
	if( !_withAutoGuide ) return 0;
	if( _isTracking ) return 0;
	if( !star.isStarSelected() ) {
		sendErrorInfo( noStar );
		return 0;
	}
	int oldAperture = currentAperture();
	int oldIllum = light.isIllumOn();
	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		return 0;
	}
	int success = 1;

	{ // "try-block"

// Vybirayem luft:

	jdebug( "luft (gap):" );

	success = shiftTelescope(RA,-1.0*_gapRA,0);
	if( success != 1 ) goto ABORT_CALIBR;

	success = shiftTelescope(RA,_gapRA,0);
	if( success != 1 ) goto ABORT_CALIBR;

	success = shiftTelescope(DECLINATION,-1.0*_gapDec,0);
	if( success != 1 ) goto ABORT_CALIBR;

	success = shiftTelescope(DECLINATION,_gapDec,0);
	if( success != 1 ) goto ABORT_CALIBR;

	jdebug( "scan:" );
	success = scanAperture( _centerAccumTime, _centerBaseTime,
		_centerBlockNumber, _centerExposition, _centerApert,
		_centerScanStart, starCenterName, false );

	if( trberr::quitProgram() ) quitProgram();

	if( success != 1 ) goto ABORT_CALIBR;
	double ra0 = _shiftRA;
	double dec0 = _shiftDec;

	jdebug( "luft (gap):" );

	success = shiftTelescope(RA,_gapRA,0);
	if( success != 1 ) goto ABORT_CALIBR;

	success = shiftTelescope(RA,_gapDec,0);
	if( success != 1 ) goto ABORT_CALIBR;

	jdebug( "scan:" );
	success = scanAperture( _centerAccumTime, _centerBaseTime,
		_centerBlockNumber, _centerExposition, _centerApert,
		_centerScanStart, starCenterName, false );

	if( trberr::quitProgram() ) quitProgram();

	double speedRA = (_shiftRA-ra0) / _gapRA / 
		cos(star.getDecVal()  * RAD_IN_ASEC);
	double speedDec = (_shiftDec-dec0) / _gapDec;

	if( (success == 1 ) && delta ) {
		delta->setX( speedRA );
		delta->setY( speedDec );
		StateExport::setCorrectionReady(true);
	}
	jdebug( "speedRA=%f speedDec=%f", _speedRA,_speedDec );
	endTrackCalibration();
	} // end of "try-block"

ABORT_CALIBR:
	setAperture( oldAperture,0 );
	turnIllum(oldIllum,0);
	return success;
}

int DevManager::shiftTelescope( int coordinate,
			double shift, int dialog ) {
	if( !_withAutoGuide ) return 0;
	if( !tracker.isReady() ) return 0;

	QString modeName;
	if( coordinate == RA ) modeName = trackRAName; 
	else modeName = trackDecName;

	openMode(modeName,fabs(shift));

	int success = 1;
	if( coordinate == RA ) success = tracker.shiftRA(shift,dialog);
	else success = tracker.shiftDec(shift,dialog);

	closeMode();

	if( trberr::quitProgram() ) quitProgram();
	if( testUserStop() ) success = 0;;
	return success;
}

void DevManager::endTrackCalibration( void ) {
	jdebug( "To Center:" );
	shiftTelescope(RA,-1.0*_gapRA,0);
	shiftTelescope(DECLINATION,-1.0*_gapDec,0);
}

int DevManager::runCentering( void ) {
	if( !star.isStarSelected() ) {
		sendErrorInfo( noStar );
		return 0;
	}
	int oldAperture = currentAperture();
	int oldIllum = light.isIllumOn();
	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		return 0;
	}
	if( !isScenario ) sendToScreen( emptyLine );
	int success = 1;
	while( success == 1 ) {
		success = scanAperture( _centerRefreshAccumTime, _centerBaseTime,
			_centerBlockNumber, _centerExposition, _centerApert,
			_centerScanStart, starCenterName, false );
		if( (success == 1) && delta ) {
			jdebug( "delta->setX, setY %f %f", _shiftRA, _shiftDec );
			delta->setX( _shiftRA );
			delta->setY( _shiftDec );
			StateExport::setCorrectionReady(true);
		}
	}
	setAperture( oldAperture,0 );
	turnIllum(oldIllum,0);
	return 1;
}

int DevManager::runFocusControl( void ) {
	int oldAperture = currentAperture();
	int oldIllum = light.isIllumOn();
	if( turnIllum(OFF,0) != 1 ) {
		sendErrorInfo( errIllum );
		return 0;
	}
	if( !isScenario ) sendToScreen( emptyLine );
	int ret = scanAperture( _focusAccumTime, _focusBaseTime,
		_focusBlockNumber, _focusExposition, _focusApert,
		_focusScanStart, focusContrName, true );
	setAperture( oldAperture,0 );
	turnIllum(oldIllum,0);
	return ret;
}

int DevManager::setVoltage( int _hvLev, int _dialog, bool _wait=true ) {
	if( !withDevice || !power.isReady() ) return(0);
	if( getVoltage() == _hvLev ) return 1;
	int ret = power.setVoltage( _hvLev,_dialog,_wait );
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

int DevManager::setLight( double value,
			int dialog,
			int waitInterval=LIGHT_WAIT_INTERVAL ) {
	if( !light.isReady() ) return(0);
	int ret = light.setLight( value,dialog,waitInterval );
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

int DevManager::setDiscriminations( QArray<double>& _levels,
						int _dialog ) {
	int ret = 1;
	if( !isCounterReady() ) return(0);
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		ret = ret * (counters[i]).setLevel( _levels[i],_dialog );
		if( trberr::quitProgram() ) quitProgram();
		jdebug( "_______ SET DISCRIM %f", _levels[i] );
	}
	return( ret );
}

void DevManager::getDiscriminations( QArray<double>& _levels ) {
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		_levels[i] = (counters[i]).getLevel();
		if( trberr::quitProgram() ) quitProgram();
	}
}

void DevManager::openProgressor( const QString& _text ) {

	infoExportMutex.lock();
	InfoExport::progress.message = _text;
	InfoExport::progress.progress = -1;
	InfoExport::isProgressChanged = true;
	infoExportMutex.unlock();

}

void DevManager::closeProgressor( void ) {

	infoExportMutex.lock();
	InfoExport::progress.progress = -2;
	InfoExport::isProgressChanged = true;
	infoExportMutex.unlock();
}

void DevManager::setProgress( double percentage ) {

	infoExportMutex.lock();
	InfoExport::progress.progress = percentage;
	InfoExport::isProgressChanged = true;
	infoExportMutex.unlock();

}

void DevManager::openMode( const QString& _text, double _time ) {

	cleanUserStop();

	jdebug( "Open Mode: %s",_text.data() );
	infoExportMutex.lock();
	InfoExport::mode.message = _text;
	InfoExport::mode.progress = _time;
	InfoExport::isModeChanged = true;
	infoExportMutex.unlock();

}

void DevManager::closeMode( void ) {

	infoExportMutex.lock();
	InfoExport::mode.message = "";
	InfoExport::mode.progress = 0;
	InfoExport::isModeChanged = true;
	infoExportMutex.unlock();

}

void DevManager::setRemainTimeMode( double _time_ ) {

	infoExportMutex.lock();
	InfoExport::mode.progress = rint(_time_);
	InfoExport::isModeChanged = true;
	infoExportMutex.unlock();
	
}

void DevManager::sendScenarioName( const QString& name ) {

	infoExportMutex.lock();
	InfoExport::scenario.message = name;
	InfoExport::isScenarioChanged = true;
	infoExportMutex.unlock();

}

void DevManager::openScenario( const QString& _text, double _time ) {

	infoExportMutex.lock();
	InfoExport::scenario.message = _text;
	InfoExport::scenario.progress = _time;
	InfoExport::isScenario = true;
	InfoExport::isScenarioStateChanged = true;
	InfoExport::isScenarioChanged = true;
	infoExportMutex.unlock();

}

void DevManager::closeScenario( void ) {

	infoExportMutex.lock();
	InfoExport::scenario.progress = 0;
	InfoExport::isScenario = false;
	InfoExport::isScenarioStateChanged = true;
	InfoExport::isScenarioChanged = true;
	infoExportMutex.unlock();

}

void DevManager::setRemainTimeScenario( double _time_ ) {

	infoExportMutex.lock();
	InfoExport::scenario.progress = _time_;
	InfoExport::isScenarioChanged = true;
	infoExportMutex.unlock();
	
}

double DevManager::resumeTestAllCounters( void ) {
	double percentage = 0;

	if( masterCounter ) {
		if( srv_file ) fprintf( srv_file,"%s: tst_err=%d\n",
		masterCounter->getName(),_total_test_err1 );
	}
	if( slaveCounter1 ) {
		if( srv_file ) fprintf( srv_file,"%s: tst_err=%d\n",
		slaveCounter1->getName(),_total_test_err2 );
	}
	if( slaveCounter2 ) {
		if( srv_file ) fprintf( srv_file,"%s: tst_err=%d\n",
		slaveCounter2->getName(),_total_test_err3 );
	}
	if( slaveCounter3 ) {
		if( srv_file ) fprintf( srv_file,"%s: tst_err=%d\n",
		slaveCounter3->getName(),_total_test_err4 );
	}

	percentage = (double)(_total_test_err1 + _total_test_err2
		+ _total_test_err3 + _total_test_err4) 
		* BYTE_PER_COUNT / _dataBlockSize;

	int n_block = exch::getNblock();
	if( n_block != 0 )
		percentage = percentage / n_block;
	else {
		percentage = 0;	// nichego ne prishlo... 
//		debug( "Error! n_counters*block_number = 0!" );
//		quitProgram();
	}

	return( percentage );
}

double DevManager::resumeTestOneCounter( void ) {
	double percentage = 0;

	if( srv_file ) fprintf( srv_file,"tst_err=%d\n",
		_total_test_err1 );

	percentage += (double)_total_test_err1 * 
		BYTE_PER_COUNT / _dataBlockSize;

	int n_block = exch::getNblock();
	if( n_block != 0 )
		percentage = percentage / n_block;
	else {
		percentage = 0;	// nichego ne prishlo... 
//		debug( "Error! n_counters*block_number = 0!" );
//		quitProgram();
	}
	return( percentage );
}

void DevManager::startDevTimer( int _interval ) {
	exch::setTimeout( false );
//	count_timeout_flag = 0;
	_baseTimeTimerPolling = 1;
	QString startTime = trb_tm::getCurrentUT().toString();
	if( srv_file ) fprintf( srv_file,"StartDevTimer: %s\n",
		startTime.data() );
	_baseTimeTimer = startTimer( _interval );
}

void DevManager::stopDevTimer( void ) {
	QString stopTime = trb_tm::getCurrentUT().toString();
	if( srv_file ) fprintf( srv_file,"StopDevTimer: %s\n",stopTime.data() );
	_baseTimeTimerPolling = 0;
	killTimer( _baseTimeTimer );
	if( srv_file ) fprintf( srv_file,"baseTimer killed... timerId=%d\n",_baseTimeTimer );
}

DevManager::DevManager( void ) {
	jdebug( "DevManager:  Constructor!" );

	isFirst = true;
	isFirstInit = true;
	isScenario = false;
	cleanUserStop();
	isOverLight = false;
	_isCounting = false;
	_isPlayBack = false;
	_isTracking = false;
	isParking = false;
	isQuiting = false;
	_scenarioStartPosition = 0;
	_shiftRA = 0;
	_shiftDec = 0;
	_discrLevels.resize( MAX_N_COUNTERS );
	_testCounts.resize( MAX_N_COUNTERS );
	_backgroundCorrectionCoefficients.resize( MAX_N_COUNTERS );
	_backgrounds.resize( MAX_N_COUNTERS );
	_nonLinearity.resize( MAX_N_COUNTERS );
	_nonPoisson.resize( MAX_N_COUNTERS );
	_nScenario = 0;
//	scenarioName.resize( _nScenario );
//	scenarioScript.resize( _nScenario );

	_polling = 1;
	_testOverLight_flag = 0;
	_processButtons_flag = 0;
	_getTemper_flag = 0;
	_temperature = -100.0;
	_temperPolling = 0;
	_overLightPolling = 0;
	_buttonsPolling = 0;
	_baseTimeTimerPolling = 0;

	coef = NULL;
	delta = NULL;

	_stopScenario = 0;
	_pauseScenario = false;
	_currentScenarioNumber = 0;
}

DevManager::~DevManager() {
	jdebug( "Destructor of devmanager..." );
	for( int i=0; i<_nScenario; i++ ) {
		jdebug( "delete scenarioName=%s",
				scenarioName[i]->data() );
		delete scenarioName[i];
	}
	sc::done();
	atm::done();
	sum::done();
	setAccess( _sumFileName.data(),false );
	setAccess( _dumpFileName.data(),false );

	if (nr::erget()) {
		jdebug(nr::ermessage().c_str());
		nr::erreset();
	}
}

void DevManager::create( void ) {
	masterCounter = NULL;
	slaveCounter1 = NULL;
	slaveCounter2 = NULL;
	slaveCounter3 = NULL;
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		(counters[i]).create( 1000 );
	}
}

int DevManager::runTestAllCounters( void ) {
	exch::startCounting();

	resetDriverBlockCounter();

	if( slaveCounter3 ) {
		slaveCounter3->blockCounter.reset();
		slaveCounter3->resetData();
		slaveCounter3->resetTest();
		int ret = slaveCounter3->runTest();
		if( ret != 1 ) return 0;
	}
	if( slaveCounter2 ) {
		slaveCounter2->blockCounter.reset();
		slaveCounter2->resetData();
		slaveCounter2->resetTest();
		int ret = slaveCounter2->runTest();
		if( ret != 1 ) {
			resetCounters(0);
			return 0;
		}
	}
	if( slaveCounter1 ) {
		slaveCounter1->blockCounter.reset();
		slaveCounter1->resetData();
		slaveCounter1->resetTest();
		int ret = slaveCounter1->runTest();
		if( ret != 1 ) {
			resetCounters(0);
			return 0;
		}
	}
	if( masterCounter ) {
		masterCounter->blockCounter.reset();
		masterCounter->resetData();
		masterCounter->resetTest();
		setStartBaseTime();
		int ret = masterCounter->runTest();
		if( trberr::quitProgram() ) quitProgram();
		if( ret != 1 ) {
			resetCounters(0);
			return 0;
		}
	}
	return 1;
}

int DevManager::runAllCounters( void ) {
	exch::startCounting();

	resetDriverBlockCounter();

	if( slaveCounter3 ) {
		slaveCounter3->blockCounter.reset();
		slaveCounter3->resetData();
		int ret = slaveCounter3->runMst();
		if( ret != 1 ) return 0;
	}
	if( slaveCounter2 ) {
		slaveCounter2->blockCounter.reset();
		slaveCounter2->resetData();
		int ret = slaveCounter2->runMst();
//  Peresmotret' reset, m.b. svoya expose!!!
// A master->slave oni pomniat???
		if( ret != 1 ) {
			resetCounters(0);
			return 0;
		}
	}
	if( slaveCounter1 ) {
		slaveCounter1->blockCounter.reset();
		slaveCounter1->resetData();
		int ret = slaveCounter1->runMst();
		if( ret != 1 ) {
			resetCounters(0);
			return 0;
		}
	}
	if( masterCounter ) {
		masterCounter->blockCounter.reset();
		masterCounter->resetData();
		setStartBaseTime();
		int ret = masterCounter->runMst();
		if( ret != 1 ) {
			resetCounters(0);
			return 0;
		}
	}
	return 1;
}

int DevManager::resetTestCounter( void ) {
	testCounter = &(counters[0]);
	for( int i=1; i<MAX_N_COUNTERS; i++ ) {
		int ret = counters[i].setMasterOff(0);
		if( trberr::quitProgram() ) quitProgram();
		if( !ret ) return 0;
		ret = counters[i].setActiveOff(0);
		if( trberr::quitProgram() ) quitProgram();
		if( !ret ) return 0;
	}
	int ret = counters[0].setMasterOn(0);
	if( trberr::quitProgram() ) quitProgram();
	if( !ret ) return 0;
	ret = counters[0].setActiveOn(0);
	if( trberr::quitProgram() ) quitProgram();
	if( !ret ) return 0;
	return 1;
}

int DevManager::switchTestCounter( void ) {

	if( testCounter == &(counters[0]) ) {
		if( !((counters[0]).setMasterOff(0)) ) return(0);
		if( !((counters[0]).setActiveOff(0)) ) return(0);
		if( !((counters[1]).setMasterOn(0)) ) return(0);
		if( !((counters[1]).setActiveOn(0)) ) return(0);
		testCounter = &(counters[1]);
		return 1;
	}
	if( testCounter == &(counters[1]) ) {
		if( !((counters[1]).setMasterOff(0)) ) return(0);
		if( !((counters[1]).setActiveOff(0)) ) return(0);
		if( !((counters[2]).setMasterOn(0)) ) return(0);
		if( !((counters[2]).setActiveOn(0)) ) return(0);
		testCounter = &(counters[2]);
		return(1);
	}
	if( testCounter == &(counters[2]) ) {
		if( !((counters[2]).setMasterOff(0)) ) return(0);
		if( !((counters[2]).setActiveOff(0)) ) return(0);
		if( !((counters[3]).setMasterOn(0)) ) return(0);
		if( !((counters[3]).setActiveOn(0)) ) return(0);
		testCounter = &(counters[3]);
		return(1);
	}
	if( testCounter == &(counters[3]) ) {
		if( !((counters[3]).setMasterOff(0)) ) return(0);
		if( !((counters[3]).setActiveOff(0)) ) return(0);
		if( !((counters[0]).setMasterOn(0)) ) return(0);
		if( !((counters[0]).setActiveOn(0)) ) return(0);
		testCounter = &(counters[0]);
		return(1);
	}
	testCounter = NULL;
	return 0;
}

int DevManager::runTestOneCounter( void ) {
	exch::startCounting();

	resetDriverBlockCounter();

	if( testCounter ) {
		if( srv_file ) fprintf( srv_file,"Test %s: \n",testCounter->getName() );
		testCounter->blockCounter.reset();
		testCounter->resetData();
		testCounter->resetTest();
		int ret = testCounter->runTest();
		if( ret != 1 ) return 0;
	}
	return 1;
}

void DevManager::searchTestErrorsAllCounters( void ) {
	int err;
	int block_number = exch::getBlockNumber();
	count_t test_number = (count_t)( block_number*_dataBlockSize
			/ BYTE_PER_COUNT );

	if( masterCounter ) {
		err = masterCounter->searchTestErrors( test_number );
		_total_test_err1 += err;
		if( srv_file ) fprintf( srv_file," master: tst_err=%d", err );
	}
	if( slaveCounter1 ) {
		err = slaveCounter1->searchTestErrors( test_number );
		_total_test_err2 += err;
		if( srv_file ) fprintf( srv_file," slave1: tst_err=%d\n",err );
	}
	if( slaveCounter2 ) {
		err = slaveCounter2->searchTestErrors( test_number );
		_total_test_err3 += err;
		if( srv_file ) fprintf( srv_file,"slave2: tst_err=%d", err );
	}
	if( slaveCounter3 ) {
		err = slaveCounter3->searchTestErrors( test_number );
		_total_test_err4 += err;
		if( srv_file ) fprintf( srv_file," slave3: tst_err=%d\n", err );
	}
	if( srv_file ) fprintf( srv_file," start: %s",
		_startBaseTime.toString().data() );
}

void DevManager::searchTestErrorsOneCounter( void ) {
	int block_number = exch::getBlockNumber();
	count_t test_number = (count_t)(block_number * _dataBlockSize
			/ BYTE_PER_COUNT );
	if( testCounter ) {
		_total_test_err1 += testCounter->searchTestErrors( test_number );
		if( srv_file ) fprintf( srv_file," %s: tst_err=%d",
			testCounter->getName(), _total_test_err1 );
	}
}

int DevManager::calcExpositionNumber( int _blockNumber ) {
	return _blockNumber * ( _dataBlockSize / BYTE_PER_COUNT );
}

int DevManager::setBlockNumber( int block_number,
			int& expositionNumber,
			int _dialog,
			bool isPlayBack=false ) {
	expositionNumber = calcExpositionNumber( block_number ) ;
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		if( isPlayBack ) {
			counters[i].resizeData( expositionNumber );
		} else {
			int ret = counters[i].setNumber( expositionNumber, _dialog );
			if( trberr::quitProgram() ) quitProgram();
			if( ret != 1 ) return 0;
		}
	}
	exch::setBlockNumber( block_number );
	return(1);
}

int DevManager::setExpos( double _expos,int _dialog ) {
	int i;
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		int ret = counters[i].setExpos( _expos, _dialog );
		if( trberr::quitProgram() ) quitProgram();
		if( ret != 1 ) return 0;
	}
	return 1;
}

int DevManager::setBlSize( int _size,int _dialog ) {
	int i;
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		int ret = counters[i].setBlSize( _size, _dialog );
		if( trberr::quitProgram() ) quitProgram();
		if( ret != 1 ) return 0;
	}
	return 1;
}

void DevManager::resetDriverBlockCounter( void ) {
	if( exch::withOldDriver() ) {
		exch::writeToProc( to_proc_file,size_to_proc ); // obnuleniye nomerov po addr counter-ov
	} else {
		exch::resetBlockCounter();	// to zhe, no cherez ioctl (bez /proc-ov...)
	}
}

int DevManager::resetCounters( int dialog ) {
	jdebug( "____________ Reset Counters ____________" );
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		int ret = counters[i].reset( dialog );
		if( ret != 1 ) return 0;
		if( trberr::quitProgram() ) quitProgram();
		counters[i].blockCounter.reset();
	}
	setBlSize( _dataBlockSize,dialog );
	int expositionNumber;
	setBlockNumber( _normalBlockNumber,expositionNumber,0 );
	setExpos( _normalExposition,dialog );

	
	resetDriverBlockCounter();

	return(1);	
}

int DevManager::stopCounters( int dialog ) {
	jdebug( "____________ Stop Counters ____________" );
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		int ret = counters[i].stop( dialog );
		if( trberr::quitProgram() ) quitProgram();
		if( ret != 1 ) return 0;
	}
	return 1;	
}

void DevManager::playScenario( void ) {
	jdebug( "current scenarioScript: %s",
			scenarioScript[_currentScenarioNumber]->data() );

	_old_count_mode = _count_mode;
	_count_mode = SCENARIO_MODE;

	_stopScenario = 0;

	startCounting();

	jdebug( "startScenario: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "startScenario: countMutex unlocked" );
}

void DevManager::setPause( bool isPause ) {
	_pauseScenario = isPause;
	jdebug( "&&&&&&&&&&&&&&&&&& pauseScenario = 1" );
	if( !_pauseScenario ) playScenario();
}

void DevManager::playBack( QString& fileName ) {
	_massInputFileName = fileName;
	_old_count_mode = _count_mode;
	_count_mode = PLAYBACK_MODE;

	startCounting();

	jdebug( "startPlayBack: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "startPlayBack: unlock lockcount" );
}

int DevManager::calcBlockNumber(double& baseTime, double exposition)
// baseTime: [sec]
// exposition: [ms]
// aim: compute number of blocks which contain all required counts and
// ask the device to produce more counts if needed: last block may thus
// contain a few "excessive" counts
{
	// Estimate fractional "exposition number" :
	double expositionNumber = S2MS * baseTime / exposition;
	// number of blocks should be integer and contain ALL counts:
	int blockNumber = (int)ceil(
			(expositionNumber / _dataBlockSize ) * BYTE_PER_COUNT);
	expositionNumber = blockNumber * 
		_dataBlockSize / BYTE_PER_COUNT;
	baseTime = expositionNumber * exposition / S2MS;
	return blockNumber;
}

void DevManager::prepareFiles( void ) {
	jdebug( "DevManager: prepareFiles" );

// FILES & DIR-s:

	_logDirName = *leafDataToStr( cfgFile,"General",
				"Outputs","LogDir" );
	prepareDirName( _logDirName );

	_dataDirName = *leafDataToStr( cfgFile,"General",
				"Outputs","DataDir" );
	prepareDirName( _dataDirName );

	QString tempDirName = *leafDataToStr( cfgFile,"General",
				"Outputs","TempDir" );
	prepareDirName( tempDirName );
	_checkFile = tempDirName + checkFileName;

	QString dateName;
	dateName.sprintf( "%02d%02d%02d",
			trb_tm::programStartDate().year()-2000,
			trb_tm::programStartDate().month(),
			trb_tm::programStartDate().day() );
	QString fileName;
	if( !out_file ) {
		fileName = dateName;
		fileName.prepend( _dataDirName );
		fileName.append( "." );
		fileName.append( indexFileExt );
		out_file = fopen( fileName.data(),"a+");
		setAccess(  fileName.data(),false );
		int nMassLines = rdm::grabparams( out_file );
		jdebug( "grabParam = %d lines",nMassLines );
	}
	if( !stm_file ) {
		fileName = dateName;
		fileName.prepend( _dataDirName );
		fileName.append( "." );
		fileName.append( momentsFileExt );
		stm_file = fopen( fileName.data(),"a+");
		setAccess(  fileName.data(),false );
	}
	_sciCountSave = leafDataOnOffToInt( cfgFile,"Operations","Common",
				"SciCountsSave" );
	_techCountSave = leafDataOnOffToInt( cfgFile,"Operations","Common",
				"TechCountsSave" );
	if( (_sciCountSave || _techCountSave) && !counts_file ) {
		fileName = dateName;
		fileName.prepend( _dataDirName );
		fileName.append( "." );
		fileName.append( countsFileExt );
		counts_file = fopen( fileName.data(),"a+b" );
		setAccess(  fileName.data(),false );
	}
	_logSave = leafDataOnOffToInt( cfgFile,"Operations","Common",
				"LogSave" );
	if( _logSave && !srv_file ) {
		fileName = dateName;
		fileName.prepend( _logDirName );
		fileName.append( "." );
		fileName.append( logFileExt );
		srv_file = fopen( fileName.data(),"a" );
		exch::setSrvFile( srv_file );
		setAccess(  fileName.data(),false );
	}

// Summary:

	_sumFileName = _dataDirName + summaryFile;
	_dumpFileName = tempDirName + dumpFile;

	long ldate = trb_tm::longDate(trb_tm::programStartDate());
	sum::init( _sumFileName.data(), ldate,
			MAX_N_COUNTERS, _dumpFileName.data() );
	if( nr::erget() ) {
		str_edit = nr::ermessage().c_str();
		showStrongMessage( str_edit );
		nr::erreset();
	}
	outcontrol.setGraphReady( true );
}

void DevManager::searchCountersSyncStatus( void ) {
	masterCounter = &(counters[0]);
	slaveCounter1 = &(counters[1]);
	slaveCounter2 = &(counters[2]);
	slaveCounter3 = &(counters[3]);

	if( masterCounter && slaveCounter1 && slaveCounter2 && slaveCounter3 )
		if( srv_file ) fprintf( srv_file,"%s -> %s -> %s -> %s\n",masterCounter->getName(),
		slaveCounter1->getName(),slaveCounter2->getName(),slaveCounter3->getName() );
}

void DevManager::loadConst( void ) {
	QString *p_str;
	jdebug( "DevManager: LoadConst" );


	if( withDevice )
		_withAutoGuide = leafDataOnOffToInt(cfgFile,"Operations",
					"Common","WithAutoGuide");
	else _withAutoGuide = false;
// Scenario:

	QString* scScrp;
	_nScenario = 3;
	scenarioScript.resize( _nScenario );
	scenarioName.resize( _nScenario );


// Initial scenario:

	if( isFirst ) {
		scScrp = leafDataToStr( cfgFile,"Operations",
				"Initialization","InitialScenario" );
		unroll( scScrp->data(),scenarioDict );
		if( nr::erget() ) {
			str_message = nr::ermessage().c_str();
			nr::erreset();
			showFatalError( str_message );
			quitProgram();
		}
		scenarioScript[0] = scScrp;
		scenarioName[0] = new QString( "InitScenario" );
	}

// Others:

	if( isLeafChanged( cfgFile,"Operations",
				"Scenario","Scenario_1" ) ) {
		scScrp = leafDataToStr( cfgFile,"Operations",
				"Scenario","Scenario_1" );
		unroll( scScrp->data(),scenarioDict );
		if( nr::erget() ) {
			str_message = nr::ermessage().c_str();
			nr::erreset();
			if( isFirst ) {
				showFatalError( str_message );
				quitProgram();
			} else {
				showStrongMessage( str_message,"Ignore" );
			}
		} else {
			scenarioScript[1] = scScrp;
			if( (!isFirst) && (_currentScenarioNumber==1) )
				setCurrentScenario( _currentScenarioNumber );
		}	
		if( isFirst ) scenarioName[1] = new QString( "Scenario 1" );
	}

	if( isLeafChanged( cfgFile,"Operations",
				"Scenario","Scenario_2" ) ) {
		scScrp = leafDataToStr( cfgFile,"Operations",
				"Scenario","Scenario_2" );
		unroll( scScrp->data(),scenarioDict );
		if( nr::erget() ) {
			str_message = nr::ermessage().c_str();
			nr::erreset();
			if( isFirst ) {
				showFatalError( str_message );
				quitProgram();
			} else {
				showStrongMessage( str_message,"Ignore" );
			}
		} else {
			scenarioScript[2] = scScrp;
			if( (!isFirst) && (_currentScenarioNumber==2) )
				setCurrentScenario( _currentScenarioNumber );
		}	
		if( isFirst ) scenarioName[2] = new QString( "Scenario 2" );
	}

	_stopOnError = leafDataOnOffToInt( cfgFile,"Operations",
				"Scenario","StopOnError" );
// Exchange parameters:
	
	_dataBlockSize = leafDataToInt( deviceCfgFile, "Counters","Common",
		"DataBlockSize", 1, 100, out_file );
	_gapRA = leafDataToDouble( deviceCfgFile, "Tracker","Common",
		"ControlBase" );
	if( !_gapRA ) _gapRA = 2.0;
	_gapDec = _gapRA;
	_dataBlockSize = exch::setBlockSize( _dataBlockSize );

	_normalExposition = leafDataToDouble( cfgFile,"Operations","Normal mode",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE, out_file ); // ???
	_generalExposition = leafDataToDouble( cfgFile,"Operations","Generalized",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE, out_file ); // ???
	_focusExposition = leafDataToDouble( cfgFile,"Operations","Focus control",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE, out_file ); // ???
	_centerExposition = leafDataToDouble( cfgFile,"Operations","Star centering",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE, out_file ); // ???
	_focusScanStart = leafDataToInt( cfgFile,"Operations","Focus control",
		"ScanStart",-64,0, out_file );
	_focusScanFinish = leafDataToInt( cfgFile,"Operations","Focus control",
		"ScanFinish",0,64, out_file );
	_centerScanStart = leafDataToInt( cfgFile,"Operations","Star centering",
		"ScanStart",-64,0, out_file );
	_centerScanFinish = leafDataToInt( cfgFile,"Operations","Star centering",
		"ScanFinish",0,64, out_file );
	_workZone = leafDataToInt( cfgFile,"Operations","Star centering",
				"WorkZone" );
	_centralZone = leafDataToInt( cfgFile,"Operations","Star centering",
				"CentralZone" );
	_maxIterCentering = leafDataToInt( cfgFile,"Operations","Star centering",
				"MaxIteration" );
	if( !_maxIterCentering ) {
			_maxIterCentering = 5;
			str_edit.setNum( _maxIterCentering );
			setLeafData( cfgFile,
				"Operations",
				"Star centering",
				"MaxIteration",
				str_edit );
	}
	_speedRA = leafDataToDouble( cfgFile,"Operations","Star centering",
				"SpeedRA" );
	_speedDec = leafDataToDouble( cfgFile,"Operations","Star centering",
				"SpeedDec" );
	_normalBaseTime = leafDataToDouble( cfgFile,"Operations","Normal mode",
		"BaseTime",0.0,10.0, out_file );
	_generalBaseTime = leafDataToDouble( cfgFile,"Operations","Generalized",
		"BaseTime",0.0,10.0, out_file );
	_generalSecondBaseTime = leafDataToDouble( cfgFile,"Operations","Generalized",
		"SecondBaseTime",0.0,10.0, out_file );

	QString method = *leafDataToStr( cfgFile,"Operations","Normal mode",
		"Cn2Method",out_file );
	if( method.upper() == "X" ) _normalFixedMethod = true;
	else _normalFixedMethod = false;

	method = *leafDataToStr( cfgFile,"Operations","Generalized",
		"Cn2Method",out_file );
	if( method.upper() == "X" ) _generalFixedMethod = true;
	else _generalFixedMethod = false;


// NORMAL:
	_normalBlockNumber = calcBlockNumber(_normalBaseTime,_normalExposition);

// GENERALIZED 1:
	_generalBlockNumber = calcBlockNumber(_generalBaseTime, _generalExposition);

// GENERALIZED 2:
	_generalSecondBlockNumber = calcBlockNumber(_generalSecondBaseTime, 
		_generalExposition);

// FOCUS:

	int expositionNumber = _focusScanFinish - _focusScanStart + 1;
	int _maxRealShift = (int)( floor(
		((double)(MAX_SYNCH_SHIFT+1)*BYTE_PER_COUNT)/_dataBlockSize )
			* _dataBlockSize/BYTE_PER_COUNT );
	_focusBlockNumber = (int)ceil( BYTE_PER_COUNT *
			(double)expositionNumber / _dataBlockSize );
	expositionNumber = _focusBlockNumber * 
				_dataBlockSize / BYTE_PER_COUNT;
	if( expositionNumber > (_maxRealShift) ) { 
		expositionNumber = _maxRealShift;
		_focusBlockNumber = expositionNumber*BYTE_PER_COUNT
				 / _dataBlockSize ;
	}
	_focusBaseTime = (double)expositionNumber * 
						(double)_focusExposition / S2MS ;

// CENTERING:

	expositionNumber = _centerScanFinish - _centerScanStart + 1;
	_centerBlockNumber = (int)ceil( BYTE_PER_COUNT *
			expositionNumber / (double)_dataBlockSize );
	expositionNumber = _centerBlockNumber * 
				_dataBlockSize / BYTE_PER_COUNT;
	if( expositionNumber > (_maxRealShift) ) { 
		expositionNumber = _maxRealShift;
		_centerBlockNumber = (int)( (double)(expositionNumber*BYTE_PER_COUNT)
				 / (double)_dataBlockSize );
	}
	_centerBaseTime = (double)expositionNumber * 
						(double)_centerExposition / S2MS;

// Accum. times:

	_normalAccumTime = leafDataToDouble( cfgFile,"Operations","Normal mode",
		"AccumTime",0.0,10000000.0, out_file );
	_generalAccumTime = leafDataToDouble( cfgFile,"Operations","Generalized",
		"AccumTime",0.0,10000000.0, out_file );
	_estimationAccumTime = leafDataToDouble( cfgFile,"Operations","Common",
		"FluxEstimationTime",0.0,10000000.0, out_file );
	_focusAccumTime = leafDataToDouble( cfgFile,"Operations","Focus control",
		"AccumTime",0.0,10000000.0, out_file );
	_centerAccumTime = leafDataToDouble( cfgFile,"Operations","Star centering",
		"AccumTime",0.0,10000000.0, out_file );
	_centerRefreshAccumTime = leafDataToDouble( cfgFile,"Operations","Star centering",
		"RefreshAccumTime",0.0,10000000.0, out_file );
	_exchangeTestTime = leafDataToDouble( cfgFile,"Operations","Tests",
		"ExchangeTestTime",0.0,10000000.0, out_file );
	_detectorTestTime = leafDataToDouble( cfgFile,"Operations","Tests",
		"DetectorTestTime",0.0,10000000.0, out_file );
	_detectorCountingTime = leafDataToDouble( cfgFile,"Operations",
		"Detectors counting measurement", "AccumTime",
				0.0,10000000.0, out_file );
	_detectorStatisticTime = leafDataToDouble( cfgFile,"Operations",
		"Detectors statistics measurement", "AccumTime",
				0.0,10000000.0, out_file );
	_backgroundTime = leafDataToDouble( cfgFile,"Operations",
		"Background measurement", "AccumTime",
				0.0,10000000.0, out_file );

// Correction parameters:

	_backgroundApShift = leafDataToInt( cfgFile,"Operations",
		"Background measurement", "ApertureShift",0,127, out_file );

	_backgroundTelShift = leafDataToDouble( cfgFile,"Operations",
		"Background measurement", "TelescopeShift",0,360000, out_file );

	p_str = leafDataToStr( cfgFile,"Operations",
		"Background measurement", "Method",out_file );
	if( p_str->upper() == "TELESCOPESHIFT" ) _backgroundMethod = TELESCOPE_SHIFT_METHOD;
	else _backgroundMethod = APERTURE_SHIFT_METHOD;
	if( isLeafChanged( cfgFile,"Operations",
			"Background measurement","Correction" ) ) {
		p_str = leafDataToStr( cfgFile,"Operations",
			"Background measurement","Correction", out_file );
		int _n_ = calcNumberDoubleInString( p_str,
			"turbina.cfg\\Operations\\Background measurement\\Correction\n" );
		if( _n_ != MAX_N_COUNTERS ) {
			str_message = "turbina.cfg\\Operations\\Background measurement"
				"\\Correction:\n Numb. of coeff != MAX_N_COUNTERS!";
			if( isFirst ) {
				showFatalError( str_message );
				quitProgram();
			} else {
				showStrongMessage( str_message,"Ignore" );
			}
		} else  {
			stringToDoubleArray( p_str,
			_backgroundCorrectionCoefficients,
			"turbina.cfg\\Operations\\Background measurement\\Correction\n",
			0.0,10.0 );
		}
	}

	if( isLeafChanged( cfgFile,"Operations",
			"Background measurement","Background" ) ) {

		p_str = leafDataToStr( cfgFile,"Operations",
			"Background measurement","Background", out_file );
		int _n_ = calcNumberDoubleInString( p_str,
			"turbina.cfg\\Operations\\Background measurement\\Background\n" );
		if( _n_ != MAX_N_COUNTERS ) {
			str_message = "turbina.cfg\\Operations\\Background measurement"
				"\\Background:\n Numb. of backgrounds != MAX_N_COUNTERS!";
			if( isFirst ) {
				showFatalError( str_message );
				quitProgram();
			} else {
				showStrongMessage( str_message,"Ignore" );
			}
		} else  {
			stringToDoubleArray( p_str,
			_backgrounds,
			"turbina.cfg\\Operations\\Background measurement\\Background\n",
			0.0,100000.0 );
		}
	}

// COUNTERS:
	_name_A = leafDataToStr( cfgFile,"Devices","Counters",
		"ChannelA", out_file )->data();
	(counters[0]).setChannel( "A" );
	(counters[0]).setName( _name_A );

	_name_B = leafDataToStr( cfgFile,"Devices","Counters",
		"ChannelB",out_file )->data();
	(counters[1]).setChannel( "B" );
	(counters[1]).setName( _name_B );

	_name_C = leafDataToStr( cfgFile,"Devices","Counters",
		"ChannelC",out_file )->data();
	(counters[2]).setChannel( "C" );
	(counters[2]).setName( _name_C );

	_name_D = leafDataToStr( cfgFile,"Devices","Counters",
		"ChannelD",out_file )->data();
	(counters[3]).setChannel( "D" );
	(counters[3]).setName( _name_D );

	Counter* p_counters[MAX_N_COUNTERS];
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		p_counters[i] = &counters[i];
	}
	exch::setCounters( p_counters );

	if( isFirst || _isPlayBack ) {
		for( int i=0; i<MAX_N_COUNTERS; i++ ) {
			(counters[i]).loadConst();
		}
	}

	sprintf( to_proc_file,"%02X %02X %02X %02X",
		(counters[0]).getModule(),(counters[1]).getModule(),
		(counters[2]).getModule(),(counters[3]).getModule() );

	searchCountersSyncStatus();

	if( isFirst || _isPlayBack ) {
		for( int i=0; i<MAX_N_COUNTERS; i++ ) {
			_nonLinearity[i] = NS2MS * leafDataToDouble( deviceCfgFile,"Counters",
					(counters[i]).getName(),"NonLinearity", out_file );
			_nonPoisson[i] = leafDataToDouble( deviceCfgFile,"Counters",
					(counters[i]).getName(),"NonPoisson", out_file );
			_discrLevels[i] = leafDataToDouble( deviceCfgFile,"Counters",
					(counters[i]).getName(),"Discrimination", out_file );
		}
	}
// LIGHT:
	if( isFirst || _isPlayBack ) {
		light.loadConst();
	}

// POOLING:
	_highPollingTime = leafDataToDouble( cfgFile,
			"Operations","Common","HighPollingTime" );
	_buttonPollingTime = leafDataToDouble( cfgFile,
			"Operations","Common","ButtonPollingTime" );
	_temperPollingTime = leafDataToDouble( cfgFile,
			"Operations","Common","TemperPollingTime" );

// HV:
	
	if( isLeafChanged( cfgFile,
			"Operations","Common","HighVoltage" ) ) {
		_hvLevel = leafDataToInt( cfgFile,
			"Operations","Common","HighVoltage", out_file );
		if( !isFirst ) setVoltage( _hvLevel,1,false );
	}

// TESTs param.:
	_testVAmpl = leafDataToDouble( cfgFile,"Operations",
		"Tests","Modulation", out_file );
	
	p_str = leafDataToStr( cfgFile,"Operations","Tests",
				"StatisticLight", out_file );
	_nStatisticTestLight = stringToDoubleArray( p_str,
		_statisticTestLight,"turbina.cfg\\Operations\\Tests\\Light",
			0,1 );
	testVAmpl(); // proverka: Light +- VAmpl

	_detectorTestLight = leafDataToDouble( cfgFile,
		"Operations","Tests","DetectorLight",0.0,1.0 );

	if( isLeafChanged( cfgFile,"Operations",
			"Tests","TestCounts" ) ) {

		p_str = leafDataToStr( cfgFile,"Operations",
			"Tests","TestCounts" );
		int _n_ = calcNumberDoubleInString( p_str,
			"turbina.cfg\\Operations\\Tests\\TestCounts\n" );
		if( _n_ != MAX_N_COUNTERS ) {
			str_message = "turbina.cfg\\Tests\\TestCounts"
				"\n Numb. of testCounts != MAX_N_COUNTERS!";
			if( isFirst ) {
				showFatalError( str_message );
				quitProgram();
			} else {
				showStrongMessage( str_message,"Ignore" );
			}
		} else  {
			stringToDoubleArray( p_str,
			_testCounts,
			"turbina.cfg\\Operations\\Tests\\TestCounts\n",
			0.0,100000.0 );
		}
	}

	_fluxTolerance = leafDataToDouble( cfgFile,"Operations",
			"Tests","FluxTolerance" ); 
	_nonPoisTolerance = leafDataToDouble( cfgFile,"Operations",
			"Tests","NonPoisTolerance" ); 


// Measur. device param:

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","Light" );

	_nDetectorCountingLight = stringToDoubleArray( p_str,
		_detectorCountingLight,"turbina.cfg\\Operations\\Detectors counting measurement\\Light",
			0,1 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","Voltage" );

	_nDetectorCountingVoltage = stringToIntArray( p_str,
		_detectorCountingVoltage,"turbina.cfg\\Operations\\Detectors counting measurement\\Voltage" );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","Discrimination" );

	_nDetectorCountingDiscrimination = stringToDoubleArray( p_str,
		_detectorCountingDiscrimination,
		"turbina.cfg\\Operations\\Detectors counting measurement\\Discrimination\n",
		0.0,10.0 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors statistics measurement","Light" );

	_nDetectorStatisticLight = stringToDoubleArray( p_str,
		_detectorStatisticLight,"turbina.cfg\\Operations\\Detectors statistic measurement\\Light",
			0,1 );

// MOTOR:
	if( isFirst || _isPlayBack ) {
		motor.loadConst();
	}
	if( trberr::quitProgram() ) quitProgram();

// TRACKER:
	if( isFirst ) {
		tracker.loadConst();
	}
	if( trberr::quitProgram() ) quitProgram();

// Init. parameters:
	if( isFirst ) {
		_inputDoor = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"InputDoor" );
		_illumValue = (int)rint( 15.0 * leafDataToDouble( cfgFile,"Operations",
				"Initialization","IlluminationLevel",0.0,1.0 ) );
		_illumination = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Illumination" );

		_buttons = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Buttons" );
		_highVoltage = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"HighVoltage" );
		_safety = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Safety" );
		_playInitialScenario = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"ScenarioRun" );
	}
	if( trberr::quitProgram() ) quitProgram();

// Parking:

	_inputDoorPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"InputDoor" );
	if( trberr::quitProgram() ) quitProgram();
	_illumPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"Illumination" );
	_buttonsPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"Buttons" );
	_hvPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"HighVoltage" );

	if( trberr::quitProgram() ) quitProgram();
// Apertures:

	QString _apName = *leafDataToStr( cfgFile,"Operations",
			"Initialization", "InitialStop" );
	if( trberr::quitProgram() ) quitProgram();
	_initialStop = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Parking", "WithAperture" );
	_closeApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Star Pointing","WithAperture", out_file );
	_showStarApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Normal mode","WithAperture", out_file );
	_normalApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Generalized","WithAperture", out_file );
	_generalApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Generalized","WithSecond", out_file );
	_generalSecondApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Focus control","WithAperture", out_file );
	_focusApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Star centering","WithAperture", out_file );
	_centerApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Background measurement","WithAperture", out_file );
	_backgroundApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement",
					"WithAperture", out_file );
	_countApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Detectors statistics measurement",
					"WithAperture", out_file );
	_statisticApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

	_apName = *leafDataToStr( cfgFile,"Operations",
			"Tests","WithAperture", out_file );
	_testApert = motor.getApertureNumber( _apName,1 );
	if( trberr::quitProgram() ) quitProgram();

// BUTTONS:

	if( isFirst ) {
		_aperturesMode = leafDataToInt( deviceCfgFile,"Lighters & Buttons",
			"ButtonModes","Apertures" );
		_illumMode = leafDataToInt( deviceCfgFile,"Lighters & Buttons",
			"ButtonModes","Illumination" );
		_shiftMode = leafDataToInt( deviceCfgFile,"Lighters & Buttons",
			"ButtonModes","FineShift" );
		_doorMode = leafDataToInt( deviceCfgFile,"Lighters & Buttons",
			"ButtonModes","InputDoor" );
		QString _bMode_str = *leafDataToStr( cfgFile,"Operations","Initialization",
				"ButtonMode" );
		_buttonMode = -1;
		if( _bMode_str.upper() == "APERTURES" ) _buttonMode = _aperturesMode;
		if( _bMode_str.upper() == "ILLUMINATION" ) _buttonMode = _illumMode;
		if( _bMode_str.upper() == "FINESHIFT" ) _buttonMode = _shiftMode;
		if( _bMode_str.upper() == "INPUTDOOR" ) _buttonMode = _doorMode;
		if( _buttonMode == -1 ) {
			str_edit = "Undefined initial Buttons Mode!";
			switch( showStrongWarning( str_edit ) ) {
				case  QMessageBox::Abort : quitProgram();
				case  QMessageBox::Ignore: break;
			}
		}
	}

// POWER:
	if( isFirst || _isPlayBack ) {
		power.loadConst();
	}


// Optics & WF

	_mirror = leafDataOnOffToInt( cfgFile,"Operations",
				"Star centering","Mirror" );
	if( _mirror == YES ) _mirror = -1;
	else _mirror = 1;
	double positionAngle = leafDataToDouble( cfgFile,"Operations",
			"Star centering","PositionAngle" );
	positionAngle *= RAD_IN_DEG;
	_sinPosAngle = sin( positionAngle );
	_cosPosAngle = cos( positionAngle );
	const double OPT_COEF = 206264.8;
	if( isFirst || _isPlayBack ) {
		double foclenFeed = leafDataToDouble( deviceCfgFile,"Optics","Feeding optics",
					"FocalLength", out_file );

		double wheelStep = leafDataToDouble( deviceCfgFile,"Optics",
			"Common","WheelStep" ); // for centering
		_scale = OPT_COEF / foclenFeed * wheelStep;

		double edgeAngle1 = leafDataToDouble( deviceCfgFile,
			"Optics","Common","EdgeAngle1" );
		double edgeAngle2 = leafDataToDouble( deviceCfgFile,
			"Optics","Common","EdgeAngle2" );
		edgeAngle1 *= RAD_IN_DEG;
		edgeAngle2 *= RAD_IN_DEG;
		edgeAngle1 = atan( edgeAngle1 );
		edgeAngle2 = atan( edgeAngle2 );
		double a = edgeAngle1 + edgeAngle2;
		double b = (edgeAngle1 - edgeAngle2) / 2;

		scan_t::loadconst( _scale,a,b );

		double foclen = motor.getFoclen( _normalApert );

		if( foclen != 0 ) {
			_zShiftNormal = wf::getzshift( foclenFeed, foclen );
		} else _zShiftNormal = 0.0;

		foclen = motor.getFoclen( _generalApert );
		if( foclen != 0 ) {
			_zShiftGeneral = wf::getzshift( foclenFeed, foclen );
		} else _zShiftGeneral = 0.0;
		foclen = motor.getFoclen( _generalSecondApert );
		if( foclen != 0 ) {
			_zShiftGeneralSecond = wf::getzshift( foclenFeed, foclen );
		} else _zShiftGeneralSecond = 0.0;
	}

	bool c = isLeafChanged( cfgFile,"Operations",
		"Star centering","CenterX1" );
	c = c || isLeafChanged( cfgFile,"Operations",
		"Star centering","CenterX2" );
	if( c ) {
		double centerX1 = leafDataToDouble( cfgFile,"Operations",
			"Star centering","CenterX1", out_file );
		double centerX2 = leafDataToDouble( cfgFile,"Operations",
			"Star centering","CenterX2", out_file );

		scan_t::loadcenter( centerX1, centerX2 );
	}

	c = isLeafChanged( cfgFile,"Operations",
		"Focus control","DefRange" );
	c = c || isLeafChanged( cfgFile,"Operations",
		"Focus control","Sharp0" );
	c = c || isLeafChanged( cfgFile,"Operations",
		"Focus control","Scale" );
	if( c ) {
		double defRange = leafDataToInt( cfgFile,
			"Operations","Focus control","DefRange" );
		jdebug( "defRange=%f",defRange );
		double sharp0 = leafDataToDouble( cfgFile,
			"Operations","Focus control","Sharp0" );
		double focScale = leafDataToDouble( cfgFile,
			"Operations","Focus control","Scale" );

		scan_t::loadfocus( defRange,sharp0,focScale );
	}

	if(isFirst ) {
		connect( tree, SIGNAL(changed()), this, SLOT(change()) );
		isFirst = false;
	}
	jdebug( "Device: end of loadConst" );
}

void DevManager::change( void ) {
	jdebug( "DevManager: change: ReLoadConst" );
	loadConst();
}

int DevManager::testShutter( Counter* counter ) {
	uchar status;

	if( !counter ) return(0);
	while( 1 ) {
		counter->getStatus( status );
		if( trberr::quitProgram() ) quitProgram();
		if( !(status & SHUTTER) ) {
			str_message.sprintf( "Channel %s: Shutter closed!",counter->getChannel() );
			switch ( showWarning( str_message ) ) {
				case  QMessageBox::Abort : quitProgram();
				case  QMessageBox::Ignore: return(0);
				case  QMessageBox::Retry : break;
			}
		} else return(1);
	}

}

void DevManager::init() {
	jdebug( "++++++ DevManager.init:" );

	int ret_first = light.reset(1);
	if( trberr::quitProgram() ) quitProgram();
	if( ret_first == -3 ) {		// no converter or no moxa102
		withDevice = false;
		_withAutoGuide = false;
		StateExport::setWithoutDevice();
		return;
	}
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		counters[i].reset(1);
		if( trberr::quitProgram() ) quitProgram();
	}

	int cycle = 1;
	while( cycle ) {
		int ret0 = light.getIdent();
		if( trberr::quitProgram() ) quitProgram();

		int ret1 = 1;
		for( int i=0; i<MAX_N_COUNTERS; i++ ) {
			int ret = counters[i].getIdent();
			if( trberr::quitProgram() ) quitProgram();
			if( ret == -1 ) ret1 = -1;
		}
		int ret5 = motor.getIdent();
		if( trberr::quitProgram() ) quitProgram();
		int ret6 = power.getIdent();
		if( trberr::quitProgram() ) quitProgram();
		int ret7 = -1;
		if( _withAutoGuide ) {
			ret7 = tracker.getIdent();
			if( trberr::quitProgram() ) quitProgram();
		}

		if( (ret0 == -1) && (ret1 == -1)
			&& (ret5 == -1) && (ret6 == -1) && (ret7 == -1) ) {
			str_message = "Device disconnected. Connect it!";
			switch ( showWarning( str_message ) ) {
				case  QMessageBox::Abort : quitProgram();
				case  QMessageBox::Ignore: 
						withDevice = false;
						_withAutoGuide = false;
						StateExport::setWithoutDevice();
						return;
				case  QMessageBox::Retry : break;
			}
		} else cycle=0;
	}

	light.getConst();
	if( trberr::quitProgram() ) quitProgram();
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		counters[i].getConst();
		if( trberr::quitProgram() ) quitProgram();
	}
	motor.getConst();
	if( trberr::quitProgram() ) quitProgram();
	power.getConst();
	if( trberr::quitProgram() ) quitProgram();
	if( _withAutoGuide ) {
		tracker.getConst();
		if( trberr::quitProgram() ) quitProgram();
	}

	light.testCrc();
	if( trberr::quitProgram() ) quitProgram();
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		counters[i].testCrc();
		if( trberr::quitProgram() ) quitProgram();
	}
	motor.testCrc();
	if( trberr::quitProgram() ) quitProgram();
	power.testCrc();
	if( trberr::quitProgram() ) quitProgram();
	if( _withAutoGuide ) {
		tracker.testCrc();
		if( trberr::quitProgram() ) quitProgram();
	}

	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		testShutter( &(counters[i]) );
	}
	setDiscriminations( _discrLevels,1 );

	startTemperPolling();

	setVoltage( _hvLevel,1,false );
	turnSafety( OFF,1 );	// sbros
	turnSafety( _safety,1 );
	turnHV( _highVoltage,1 );

	testMotorStatus(1);
	testButtons();
	testIllum();

	motor.loadStops(1);
	if( trberr::quitProgram() ) quitProgram();
	motor.setStep( SYNCH_STEP,1 );
	if( trberr::quitProgram() ) quitProgram();

	setBlSize( _dataBlockSize,1 );

	setBrightness( _illumValue,1 );
	turnIllum( _illumination,1 );
	turnLight(OFF,1);
	setLight(0,1);
	turnButtons( _buttons,1 );
	light.setBMode( _buttonMode,1 );
	if( trberr::quitProgram() ) quitProgram();
	if( !_playInitialScenario || !isFirstInit ) {
		turnDoor( _inputDoor,0,false );	// startovoe polozhenie
		setAperture( _initialStop,0 );	// tozhe
		setCurrentScenario(1); // t.k. "0" - initScenario
		jdebug( "setApert=%d, setDoor=%d",_initialStop,_inputDoor ); 
		isFirstInit = false;
	}

	setInduceRegime(1);
}

void DevManager::startTemperPolling( void ) {
	getTemper(0);
	_getTemper_flag = 0;
	if( !_temperPolling ) {
		_temperTimer = startTimer( (int)(_temperPollingTime*1000.0) );
		_temperPolling = 1;
	}
}

void DevManager::stopTemperPolling( void ) {
	if( _temperPolling ) {
		killTimer( _temperTimer );
		_temperPolling = 0;
	}
}

void DevManager::startOverLightPolling( void ) {
	_testOverLight_flag = 0;
	if( !_overLightPolling ) {
		_hvTimer = startTimer( (int)(_highPollingTime*1000.0) ); 
		_overLightPolling = 1;
	}
}

void DevManager::stopOverLightPolling( void ) {
	if( _overLightPolling ) {
		killTimer( _hvTimer );
		_overLightPolling = 0;
	}
}

void DevManager::startButtonsPolling( void ) {
	_processButtons_flag = 0;
	if( !_buttonsPolling ) {
		_buttonsTimer = startTimer( (int)(_buttonPollingTime*1000.0) );
		_buttonsPolling = 1;
	}
}

void DevManager::stopButtonsPolling( void ) {
	if( _buttonsPolling ) {
		killTimer( _buttonsTimer );
		_buttonsPolling = 0;
	}
}

void DevManager::timerEvent( QTimerEvent *e ) {
	int _timer = e->timerId();
	if( _baseTimeTimerPolling && (_timer == _baseTimeTimer) ) {
		if( srv_file ) fprintf( srv_file,"_timer=%d,_baseTimeTimer=%d\n",
						_timer,_baseTimeTimer );
		exch::timeoutAction();
		return;
	}
	if( _overLightPolling && (_timer == _hvTimer) ) {
		testOverLight(1);
		return;
	}
	if( _buttonsPolling && (_timer == _buttonsTimer) ) {
		processButtons(1);
		return;
	}
	if( _temperPolling && (_timer == _temperTimer) ) {
		getTemper(1);
		return;
	}
}

int DevManager::testOverLight( int _dialog ) {
//	jdebug( "testOverLight" );
	int ret = 0;
	int _oldPolling;
	if( !power.isReady() ) return(0);
	if( !_polling ) _testOverLight_flag = 1;
	else {
		if( srv_file ) fprintf( srv_file,"TestOverLight! _polling=%d",_polling );
		_oldPolling = _overLightPolling;
		_overLightPolling = 0;
		if( power.testOverLight( _dialog ) == 0 ) {

			StateExport::setHV(OFF);
/*
			stateExportMutex.lock();
			StateExport::state.hv = OFF;
			StateExport::isChanged = true;
			stateExportMutex.unlock();
*/
			isOverLight = true;	

			jdebug( "Overlight!!! signalHV(OFF) emitted!!!" );
			ret = 1;
		}
		if( trberr::quitProgram() ) quitProgram();
		_overLightPolling = _oldPolling;
	}
	return( ret );
}

void DevManager::processButtons( int _dialog ) {
	int _bParam;
	int _bMode;
	int _oldPolling;
	if( !light.isReady() ) return;
	if( !_polling ) _processButtons_flag = 1;
	else {
		_oldPolling = _buttonsPolling;
		_buttonsPolling = 0;
		light.getParam( _bParam,_dialog );
		if( trberr::quitProgram() ) quitProgram();
		if( _bParam ) {
			light.getBMode( _bMode,_dialog );
			if( trberr::quitProgram() ) quitProgram();
			if( _bMode == _aperturesMode ) { changeAperture( _bParam,_dialog ); }
			if( _bMode == _illumMode ) { changeBrightness( _bParam ); }
			if( _bMode == _shiftMode ) { fineShift( _bParam ); }
			if( _bMode == _doorMode ) { 
				if( _bParam == 0xFF ) _bParam = 0;
				turnDoor( _bParam,_dialog );
			}
		}
		_buttonsPolling = _oldPolling;
	}
	return;
}

void DevManager::getTemper( int _dialog ) {
	double _temper;
	int _oldPolling;

	if( !power.isReady() ) return;
	if( !_polling ) _getTemper_flag = 1;
	else {
		_oldPolling = _temperPolling;
		_temperPolling = 0;
		power.getTemper( _temper,_dialog );
		if( trberr::quitProgram() ) quitProgram();
//		jdebug( "Temperature = %f",_temper );
		if( _temper != _temperature ) {
			_temperature = _temper;
			fprintf( out_file, "%s Temperature: %.1f\n",
				getStamp( "# ",trb_tm::getCurrentUT() ),
				_temperature );

			StateExport::setTemperature(_temperature);
/*
			stateExportMutex.lock();
			StateExport::state.temperature = _temperature;
			StateExport::isChanged = true;
			stateExportMutex.unlock();
*/
		}
		_temperPolling = _oldPolling;
	}
}

int DevManager::setInduceRegime( int dialog ) {
	if( !slaveCounter3 || !slaveCounter2
		|| !slaveCounter1 || !masterCounter  ) return(0);

	int success = 1;
	int ret = slaveCounter3->setMasterOff( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = slaveCounter3->setActiveOff( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = slaveCounter3->setInduc( slaveCounter2->getModule(),dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = slaveCounter3->setInduceOn( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;

	ret = slaveCounter2->setMasterOff( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = slaveCounter2->setActiveOff( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = slaveCounter2->setInduc( slaveCounter1->getModule(),dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = slaveCounter2->setInduceOn( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;

	ret = slaveCounter1->setMasterOff( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = slaveCounter1->setActiveOff( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = slaveCounter1->setInduc( masterCounter->getModule(),dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = slaveCounter1->setInduceOn( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;

	ret = masterCounter->setInduceOff( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	if( trberr::quitProgram() ) quitProgram();
	ret = masterCounter->setMasterOn( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;
	ret = masterCounter->setActiveOn( dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret != 1 ) success = 0;

	return success;
}

void DevManager::resetTestData( void ) {
	_total_test_err1 = 0;
	_total_test_err2 = 0;
	_total_test_err3 = 0;
	_total_test_err4 = 0;
}

void DevManager::exchangeTest( void ) {
	_old_count_mode = _count_mode;
	_count_mode = EXCHANGE_TEST_MODE;

	startCounting();

	jdebug( "startExchangeTest: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "startExchangeTest: unlock lockcount" );
}

void DevManager::detectorTest( void ) {
	_old_count_mode = _count_mode;
	_count_mode = DETECTOR_TEST_MODE;

	startCounting();

	jdebug( "startDetectorTest: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "startDetectorTest: unlock lockcount" );
}

void DevManager::statisticTest( void ) {
	_old_count_mode = _count_mode;
	_count_mode = STATISTIC_TEST_MODE;

	startCounting();

	jdebug( "startStatisticTest: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "startStatisticTest: unlock lockcount" );
}

void DevManager::playInitialScenario( void ) {
	if( _playInitialScenario ) {
		setCurrentScenario( 0 );
		playScenario();
	}
}

char DevManager::decodeMode( char code ) {
	char mode = -1;
	switch( code ) {
		case 'N'	: mode = NORMAL_MODE;  break;
		case 'G'	: mode = GENERALIZED_MODE;  break;
		case 'V'	: mode = FOCUS_CONTROL_MODE;  break;
		case 'C'	: mode = CENTER_CONTROL_MODE;  break;
		case 'B'	: mode = BACKGROUND_MEASUR_MODE;  break;
		case 'D'	: mode = DETECTOR_TEST_MODE;  break;
		case 'S'	: mode = STATISTIC_TEST_MODE;  break;
		case 'X'	: mode = EXCHANGE_TEST_MODE;  break;
		case 'U'	: mode = DETECTOR_COUNTING_MODE;  break;
		case 'L'	: mode = DETECTOR_STATISTIC_MODE;  break;
		case 'Q'	: mode = QUIT_PROGRAM;  break;

		default		: mode = -1; break;
	}
	if( mode == -1 ) {
		str_message = "Invalid mode " + mode; 
		showFatalError( str_message );
		quitProgram();
	}
	return( mode );
}

QString& DevManager::getScenarioName( int number ) {
	if( number >= _nScenario ) {
		jdebug( "Invalid scenario number: %d",number );
		return( invalid );
	}
	return( (*scenarioName[number]) ); 

}

void DevManager::setCurrentScenario( int number ) {
	if( number >= _nScenario ) {
		jdebug( "Invalid scenario number: %d",number );
		return;
	}
	_scenarioStartPosition = 0;
	string scen_str = unroll( scenarioScript[number]->data(),
					scenarioDict );
	const char* scen = scen_str.c_str();
	jdebug( "Set scenario: %s: %s",
		(scenarioScript[number])->data(),scen );

	_currentScenarioNumber = number;
	int size = strlen( scen );
	currentScenario.resize( size );
	for( int i=0; i<size; i++ ) {
		char mode = decodeMode( scen[i] ); 
		if( mode < 0 ) return;
		else currentScenario[i] = mode;
	}
	sendScenarioName( getScenarioName(_currentScenarioNumber) );
}

void DevManager::detectorStatisticMeasurement( void ) {
	_old_count_mode = _count_mode;
	_count_mode = DETECTOR_STATISTIC_MODE;

	startCounting();

	jdebug( "start DetectorStatisticMeasurement: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "start DetectorStatisticMeasurement: unlock lockcount" );
}

void DevManager::detectorCountingMeasurement( void ) {
	_old_count_mode = _count_mode;
	_count_mode = DETECTOR_COUNTING_MODE;

	startCounting();

	jdebug( "start DetectorCountingMeasurement: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "start DetectorCountingMeasurement: unlock lockcount" );
}

void DevManager::focusControl( void ) {
	_old_count_mode = _count_mode;
	_count_mode = FOCUS_CONTROL_MODE;

	startCounting();

	jdebug( "start Focus Control: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "start Focus Control: unlock lockcount" );
}

void DevManager::centerControl( void ) {
	_old_count_mode = _count_mode;
	_count_mode = CENTER_CONTROL_MODE;

	startCounting();

	jdebug( "start Star Centering: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "start Start Centering: unlock lockcount" );
}

void DevManager::normal( void ) {
	_old_count_mode = _count_mode;
	_count_mode = NORMAL_MODE;

	startCounting();

	jdebug( "start Normal Mode: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "start Normal Mode: unlocked lockcount" );
}

void DevManager::generalized( void ) {
	_old_count_mode = _count_mode;
	_count_mode = GENERALIZED_MODE;

	startCounting();

	jdebug( "start Generalized Mode: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "start Generalized Mode: unlock lockcount" );
}

void DevManager::acceptBackgroundCorrection( void ) {
	if( !coef ) return;
	str_edit = "";
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		_backgroundCorrectionCoefficients[i] = (*coef)[i][2];
		str_add.setNum(_backgroundCorrectionCoefficients[i],'f',2 );
		str_edit += str_add;
		if( i != MAX_N_COUNTERS-1 ) str_edit += ", "; 
	}
	setLeafData( cfgFile,"Operations",
		"Background measurement",
		"Correction",
		str_edit,
		out_file );
}

void DevManager::acceptCenterCorrection( void ) {
	double cx1,cx2;
	scan[best].center( &cx1,&cx2,scan_t::MERG,true );
	if( testNrError() ) return;

	str_edit.setNum( cx1 );
	setLeafData( cfgFile,"Operations",
		"Star centering",
		"CenterX1",
		str_edit,
		out_file );

	str_edit.setNum( cx2 );
	setLeafData( cfgFile,"Operations",
		"Star centering",
		"CenterX2",
		str_edit,
		out_file );
}

void DevManager::acceptTrackCalibration( void ) {
	jdebug( "Accepted: speedRA=%f speedDec=%f", _speedRA,_speedDec );

	str_edit.setNum(_speedRA);
	setLeafData( cfgFile,
		"Operations",
		"Star centering",
		"SpeedRA",
		str_edit,
		out_file );
	str_edit.setNum(_speedDec);
	setLeafData( cfgFile,
		"Operations",
		"Star centering",
		"SpeedDec",
		str_edit,
		out_file );  
}

void DevManager::getBackgroundCorrectionCoefficients( matrix43* _p_matrix )
{
	if( !_p_matrix ) return;
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		(*_p_matrix)[i][2] = _backgroundCorrectionCoefficients[i];
	}
}

void DevManager::backgroundCorrection( matrix43* _p_matrix ) {
	coef = _p_matrix;
	_old_count_mode = _count_mode;
	_count_mode = BACKGROUND_CORR_MODE;
	startCounting();
	jdebug( "startBackgroundCorrection: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
	jdebug( "startBackgroundCorrection: unlock lockcount" );
}

void DevManager::backgroundMeasurement( void ) {
	_old_count_mode = _count_mode;
	_count_mode = BACKGROUND_MEASUR_MODE;
	startCounting();
	jdebug( "startBackgroundMeasurement: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
}

void DevManager::fluxEstimation( void ) {
	_old_count_mode = _count_mode;
	_count_mode = FLUX_ESTIMATION_MODE;
	startCounting();
	jdebug( "startFluxEstimation: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
}

void DevManager::centerCorrection( PointDouble* _p_point ) {
	delta = _p_point;
	_old_count_mode = _count_mode;
	_count_mode = CENTERING_CORR_MODE;
	startCounting();
	jdebug( "startCenterCorrection: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
}

void DevManager::trackCalibration( PointDouble* _p_point ) {
	delta = _p_point;
	_old_count_mode = _count_mode;
	_count_mode = TRACK_CALIBRATION_MODE;
	startCounting();
	jdebug( "startTarckCalibtarion: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
}

void DevManager::centering( PointDouble* _p_point ) {
	delta = _p_point;
	_old_count_mode = _count_mode;
	_count_mode = CENTERING_MODE;
	startCounting();
	jdebug( "startCentering: try unlock lockcount..." );
	if( countMutex.locked() ) countMutex.unlock();
}

void DevManager::testVAmpl( void ) {
//	proverka amplitudy Var. control Light v Statistic Test-e

	int changed = 0;
	for( int i=0; i<_nStatisticTestLight; i++ ) {
		double ampl = _testVAmpl * 
				_statisticTestLight[i];
		if( (_statisticTestLight[i] + ampl) > MAX_LIGHT_VALUE ) {
			changed = 1;
			break;
		}
		if( (_statisticTestLight[i] - ampl) < MIN_LIGHT_VALUE ) {
			changed = 1;
			break;
		}
	}
	if( changed ) {
		str_message = "Statistic test:\nturbina.cfg/Operations/Tests/Amplitude\n"
			"is too high and will be modified";
		switch( showStrongWarning( str_message ) ) {
			case  QMessageBox::Abort : quitProgram();
			case  QMessageBox::Ignore: break;
			default : break;
		}
	}
}

void DevManager::startCounting( void ) {
	jdebug( "------------------ Start Counting -----------" );
	_isCounting = true;
	StateExport::setStartCounting();
}

void DevManager::disablePolling( void ) {
	_polling = 0;	// disable
}

void DevManager::enablePolling( void ) {
	_polling = 1;	// enable

	star.updateCurrent();
	if( _testOverLight_flag ) {
		_testOverLight_flag = 0;
	}
	if( _processButtons_flag ) {
		_processButtons_flag = 0;
		processButtons(0);
	}
	if( _getTemper_flag ) {
		_getTemper_flag = 0;
		getTemper(0);
	}
}

void DevManager::endCounting( void ) {
	jdebug( "------------------- End Counting -------------------" );
	_isCounting = false;
	StateExport::setEndCounting();
}

int DevManager::testMotorStatus( int dialog ) {
	if( !motor.isReady() ) return(0);
	int ret = motor.testInitStatus( dialog );
	if( trberr::quitProgram() ) quitProgram();
	StateExport::setDoor(motor.isDoorOpen());
	return ret;
}

int DevManager::testPowerStatus( int dialog ) {
	if( !power.isReady() ) return(0);
	int ret = power.testOverLight( dialog );
	if( trberr::quitProgram() ) quitProgram();
	StateExport::setHV(power.isHVOn());
	return( ret );
}

int DevManager::testButtons( void ) {
	if( !light.isReady() ) return(0);
	int ret = light.testButtons();
	if( trberr::quitProgram() ) quitProgram();
	StateExport::setButtons(isButtonsOn());
	return ret;
}

int DevManager::testIllum( void ) {
	if( !light.isReady() ) return(0);
	int ret = light.testIllum();
	if( trberr::quitProgram() ) quitProgram();
	StateExport::setIllum(isIllumOn());
	return ret;
}

int DevManager::turnDoor( const QString& open, int dialog,
					bool backApert = true ) {

	if( !motor.isReady() ) return(0);
	int oldAperture = currentAperture();
	int ret = motor.turnDoor( open,dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret == 1 )
		StateExport::setDoor(motor.isDoorOpen());
	if( backApert ) setAperture( oldAperture,dialog ); // T.k. shtorka dvigaet apertureu
	return ret;
}

int DevManager::turnDoor( int open, int dialog,
					bool backApert = true ) {

	if( !motor.isReady() ) return(0);
	int oldAperture = currentAperture();
	int ret = motor.turnDoor( open,dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret == 1 ) StateExport::setDoor(open);
	if( backApert ) setAperture( oldAperture,dialog ); // T.k. shtorka dvigaet apertureu
	return ret;
}

void DevManager::turnHV( int on, int dialog ) {
	if( !power.isReady() ) return;
	jdebug( "DevManager: turnHV %d", on );
	int ret = power.turnHV( on, dialog );
	if( trberr::quitProgram() ) quitProgram();
	jdebug( "ret=%d",ret );
	if(  ret == 1 ) StateExport::setHV(on);
	jdebug( "state.hv = %d",StateExport::hv() );
}

int DevManager::synchAt( int shift, int dialog ) {
	if( !motor.isReady() ) return(0);
	if( shift > MAX_SYNCH_SHIFT ) return(0);
	if( shift < (-1)*MAX_SYNCH_SHIFT ) return(0);
	int ret = motor.synchAt( (char)shift,dialog );
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

int DevManager::shiftAperture( int shift, int dialog ) {
	if( !motor.isReady() ) return(0);
	int ret = motor.shiftAperture( shift,dialog );
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

int DevManager::setAperture( const QString& _apName, int _dialog,
			bool _hard=false ) {
	int ret = 0;
	if( !motor.isReady() ) return(0);
	if( (ret=motor.setAperture( _apName, _dialog, _hard )) == 1 )
		StateExport::setAperture(currentAperture());
	if( trberr::quitProgram() ) quitProgram();
	return( ret );
}

int DevManager::setAperture( int stop_number, int _dialog,
			bool _hard=false ) {
	if( !motor.isReady() ) return 0;
	int ret;
	if( (ret=motor.setAperture( stop_number,_dialog,_hard )) == 1 )
		StateExport::setAperture(stop_number);
	if( trberr::quitProgram() ) quitProgram();
	return( ret );
}

int DevManager::changeAperture( int _shift, int _dialog ) {
	if( !motor.isReady() ) return(0);
	int _aperture_ = currentAperture();
	if( _shift == 1 ) {
		_aperture_++;
		if( _aperture_ >= motor.getNStops() ) _aperture_= 0;
	} else if( _shift == 0xFF ) {
		_aperture_--;
		if( _aperture_ < 0 ) _aperture_= motor.getNStops()-1;
	} else return(0);
	int ret = 0;
	if( (ret=motor.setAperture( _aperture_,_dialog )) == 1 )
		StateExport::setAperture(currentAperture());
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

int DevManager::track( char coordinate, char direct ) {
	if( !tracker.isReady() ) return 0;
	if( !_withAutoGuide ) return 0;
	if( _isTracking ) return 0;
//	if( _isCounting ) return 0;
	int dialog = 1;
	int success = 0;
	if( coordinate == RA ) // RA
		success = tracker.moveRA(-1*_mirror*direct,dialog); 
	else		// Dec
		success = tracker.moveDec(direct,dialog);
	if( trberr::quitProgram() ) quitProgram();
	if( success == 1 )
		_isTracking = true;
	return success;
}

int DevManager::stopTrack( void ) {
	if( !tracker.isReady() ) return 0;
	if( !_withAutoGuide ) return 0;
	int dialog = 0;
	int ret = tracker.stopMotion(dialog);
	if( ret == 1 ) _isTracking = false;
	return ret;
}

int DevManager::forceTrack( void ) {
	if( !tracker.isReady() ) return 0;
	if( !_withAutoGuide ) return 0;
	int dialog = 0;
	return tracker.forceMotion(dialog);
}

int DevManager::greatTrack( char coordinate, char direct, double trackingTime ) {
	int ret = track( coordinate, direct );
	sleep(1);
	ret *= forceTrack();
	QTime t;
	t.start();
	while( t.elapsed() < int(trackingTime*1000) );
	ret *= stopTrack();
	return ret;
}

int DevManager::turnIllum( const QString& on, int dialog ) {
	if( !light.isReady() ) return 0;
	int ret = 0;
	if( (ret=light.turnIllum( on,dialog )) ) 
			StateExport::setIllum(light.isIllumOn());
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

int DevManager::turnIllum( int on, int dialog ) {
	if( !light.isReady() ) return 0;
	int ret = 0;
	if( (ret=light.turnIllum( on, dialog )) ) {
		StateExport::setIllum(on);
	}
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

void DevManager::turnIllumOn( void ) {
	turnIllum( 1,1 );
}

void DevManager::turnIllumOff( void ) {
	turnIllum( 0,1 );
}

int DevManager::turnLight( int on, int dialog,
			int waitInterval=LIGHT_WAIT_INTERVAL ) {
	if( !light.isReady() ) return 0;
	int ret = light.turnLight(on,dialog,waitInterval);
	if( trberr::quitProgram() ) quitProgram();
	if( ret == 1 ) StateExport::setLight(on);
	return ret;
}

void DevManager::turnButtons( const QString& _on, int _dialog ) {
	if( !light.isReady() ) return;
	if( light.turnButtons( _on,_dialog ) ) {
		int on_int = isButtonsOn();
		if( on_int == 1 ) startButtonsPolling();
		else stopButtonsPolling();
		StateExport::setButtons(on_int);
/*
		stateExportMutex.lock();
		StateExport::state.buttons = _on_int;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
*/
	}
	if( trberr::quitProgram() ) quitProgram();
}

void DevManager::turnButtons( int _on, int _dialog ) {
	if( !light.isReady() ) return;
	if( light.turnButtons( _on,_dialog ) ) {
		_on = isButtonsOn();
		if( _on == 1 ) startButtonsPolling();
		else stopButtonsPolling();
		StateExport::setButtons(_on);
/*
		stateExportMutex.lock();
		StateExport::state.buttons = _on;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
*/
	}
	if( trberr::quitProgram() ) quitProgram();
}

void DevManager::turnSafety( const QString& _on_str, int _dialog ) {
	if( !power.isReady() ) return;
	int ret =  power.turnSafety( _on_str,_dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret ) {
		int on_int = isSafetyOn();
		StateExport::setSafety(on_int);
/*
		stateExportMutex.lock();
		StateExport::state.safety = _on_int;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
*/
		if( on_int == ON ) startOverLightPolling();
		else stopOverLightPolling();
	}
}

void DevManager::turnSafety( int on, int dialog ) {
	if( !power.isReady() ) return;
	int ret = power.turnSafety( on,dialog );
	if( trberr::quitProgram() ) quitProgram();
	if( ret ) {
		on = isSafetyOn();
		StateExport::setSafety(on);
/*
		stateExportMutex.lock();
		StateExport::state.safety = _on;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
*/
		if( on == ON ) startOverLightPolling();
		else stopOverLightPolling();
	}
}

void DevManager::nullSet( void ) {
	if( !motor.isReady() ) return;
	int oldAperture = currentAperture();
	int oldDoor = motor.isDoorOpen();

	motor.nullSet(1);
	if( trberr::quitProgram() ) quitProgram();

	turnDoor( oldDoor,1 );
	setAperture( oldAperture,1 ); // T.k. shtorka dvigaet apertureu
	StateExport::setDoor(motor.isDoorOpen());
/*
	stateExportMutex.lock();
	StateExport::state.door = motor.isDoorOpen();
	StateExport::state.aperture = currentAperture();
	StateExport::isChanged = true;
	stateExportMutex.unlock();
*/
}

void DevManager::setBrightness( int brightness,int dialog ) {
	if( light.isReady() ) {
		light.setBrightness( brightness,dialog );
		if( trberr::quitProgram() ) quitProgram();
		StateExport::setBrightness(getCurrentBrightness());
/*
		stateExportMutex.lock();
		StateExport::state.brightness = getCurrentBrightness();
		StateExport::isChanged = true;
		stateExportMutex.unlock();
*/
	}
}

void DevManager::changeBrightness( int _up ) {
	if( _up == 1 ) { setBrightnessUp(); return; }
	if( _up == 0xFF ) { setBrightnessDown(); return; }
}

void DevManager::setBrightnessUp( void ) {
	if( light.isReady() ) {
		light.setBrightnessUp( 1 );
		if( trberr::quitProgram() ) quitProgram();
		StateExport::setBrightness(getCurrentBrightness());
/*
		stateExportMutex.lock();
		StateExport::state.brightness = getCurrentBrightness();
		StateExport::isChanged = true;
		stateExportMutex.unlock();
*/
	}
}

void DevManager::setBrightnessDown( void ) {
	if( light.isReady() ) {
		light.setBrightnessDown( 1 );
		if( trberr::quitProgram() ) quitProgram();
		StateExport::setBrightness(getCurrentBrightness());
/*
		stateExportMutex.lock();
		StateExport::state.brightness = getCurrentBrightness();
		StateExport::isChanged = true;
		stateExportMutex.unlock();
*/
	}
}

int DevManager::fineShift( int _direct ) {
	if( _direct == 1 ) return( movePlus() );
	if( _direct == 0xFF ) return( moveMinus() );
	str_message = "Invalid shift direction!";
	switch( showStrongWarning( str_message ) ) {
		case  QMessageBox::Abort : quitProgram();
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
}

int DevManager::movePlus( void ) {
	if( !motor.isReady() ) return 0;
	int ret = motor.movePlus(1);
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

int DevManager::moveMinus( void ) {
	if( !motor.isReady() ) return 0;
	int ret = motor.moveMinus(1);
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

QString& DevManager::getStopName( int _stopNumber ) {
	return( motor.getStopName(_stopNumber) );
}

bool DevManager::isCounterReady( void ) {
	bool ready = true;
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		ready = ready && (counters[i]).isReady();
	}
	return ready;
}

int DevManager::isHVOn( void ) {
	if( !power.isReady() ) return(-1);
	int ret = power.isHVOn();
	if( trberr::quitProgram() ) quitProgram();
	return ret;
}

int DevManager::startShowStar( void ) {
	int oldAperture = currentAperture();
	if( !_isCounting && !isScenario ) setAperture( _showStarApert,1 );
	return oldAperture;
}

void DevManager::finishShowStar( int oldAperture ) {
	if( !_isCounting && !isScenario ) setAperture( oldAperture,1 );
}

void DevManager::startAdjustAperture( void ) {
	motor.startCorrection();
}

void DevManager::acceptAdjustAperture( int _dialog ) {
	motor.acceptCorrection( _dialog );
	if( trberr::quitProgram() ) quitProgram();
}

void DevManager::acceptComment( const QString& comment ) {
	sendToOutFile( comment, "##" );
}

void DevManager::park( void ) {

	if( isParking ) return;	// ne popast' v beskonechnyi cycle
	if( _isCounting ) return;

	isParking = true;
	jdebug( "Parking..." );

	str_edit = "Parking...";
	showMessage( str_edit,-1 );
	stopOverLightPolling();
	stopButtonsPolling();
	stopTemperPolling();

	if( _inputDoorPark == OFF ) {
		if( motor.isReady() ) {

			if( motor.isDoorOpen() ) turnDoor( OFF,0,false );
			else motor.setAperture( _closeApert,0 ); 

		}
	}
	turnHV( _hvPark,0 );
	if( light.isReady() ) light.reset(0); // Perevodim buttons v avtonom.rezhim
	turnIllum( _illumPark,0 );
	turnLight(OFF,0);
	isParking = false;
	jdebug( "end of Parking!" );
}

void DevManager::quitProgram( bool withPark=true ) {
	if( isQuiting ) return;
	isQuiting = true;

	if( withDevice && withPark ) park();

	exch::quitDataCycle();

	quit_count_cycle();

	if( out_file ) fclose( out_file );
	if( stm_file ) fclose( stm_file );
	if( counts_file ) fclose( counts_file );
	if( srv_file ) fclose( srv_file );

	isQuiting = false;
	exit(0);
}
